//	AppMenu.cpp
//
//	Tetsuo Yamada

#include	"AppMenu.h"
#include	"BarWindow.h"


extern const settings* fb_settings ;
extern LayerManager* fb_manager ;
extern const char *fb_app_name ;


//=============================================================
//	class AppMenuItem
//=============================================================
AppMenuItem:: AppMenuItem () 
		: BMenuItem (new BMenu (fb_app_name)) {
	fIcon = NULL ;
	
	status_t err = B_OK ;
	app_info info ;
	
	err = be_app->GetAppInfo (&info) ;
	if (err!=B_OK) return ;
	
	fIcon = new BBitmap (P_MINI_RECT, B_CMAP8) ;
	err = BNodeInfo::GetTrackerIcon (&(info.ref), fIcon, B_MINI_ICON) ;
	if (err==B_OK) return ;
	
	delete fIcon ;
	fIcon = NULL ;
} ;

AppMenuItem:: ~AppMenuItem () {
	if (fIcon) delete fIcon ;
} ;

void AppMenuItem:: GetContentSize (float *width, float *height) {
	fb_settings->get_item_size (width, height) ;
	if (IS_COLUMN) *height = 15 ;
	else *width = 15 ;
} ;

void AppMenuItem:: DrawContent () {
	BMenu *menu = Menu() ;
	BRect frame = Frame() ;
	BPoint p ;
	
	if (fIcon) {
		p.x = ceil (frame.left + (frame.Width() - 16)/2) ;
		p.y = ceil (frame.top + (frame.Height() - 16)/2) ;
		
		menu->SetDrawingMode (B_OP_OVER) ;
		menu->DrawBitmap (fIcon, p) ;
		menu->SetDrawingMode (B_OP_COPY) ;
	} else {
		char label [8] ;
		BFont font (be_plain_font) ;
		BFont cache ;
		menu->GetFont (&cache) ;
		
		if (IS_COLUMN) {
			menu->SetFont (&font) ;
			TruncateLabel (frame.Width() - 4, label) ;
			p.x = frame.left + (frame.Width() - font.StringWidth (label))/2 ;
			p.y = frame.top + (frame.Height() + font.Size())/2 ;
			menu->DrawString (label, p) ;
			menu->SetFont (&cache) ;
		} else {
			// draw label from bottom to top
			menu->SetFont (&font) ;
			strcpy (label, Label()) ;
			TruncateLabel (frame.Height() - 4, label) ;
			p.x = frame.left + (frame.Width() + font.Size())/2 ;
			p.y = frame.top + (frame.Height() + font.StringWidth (label))/2 ;
			font.SetRotation (90) ;
			menu->SetFont (&font, B_FONT_ROTATION) ;
			menu->DrawString (label, p) ;
			font.SetRotation (0) ;
			menu->SetFont (&cache) ;
		}
	}
	
	// apply enablity
	if (!(menu->Window()->IsActive())) {
		rgb_color high = menu->HighColor() ;
		menu->SetDrawingMode (B_OP_BLEND) ;
		menu->SetHighColor (menu->ViewColor()) ;
		menu->FillRect (frame) ;
		//menu->FillRect (frame) ;
		menu->SetHighColor (high) ;
		menu->SetDrawingMode (B_OP_COPY) ;
	}
} ;

bool AppMenuItem:: IsSelected2 () const {
	return IsSelected() ;
} ;

//=============================================================
//	class AppMenu
//=============================================================
AppMenu:: AppMenu () 
		: BMenuBar (BRect (0, 0, 1, 1), "app_menu", B_FOLLOW_NONE, 
			(IS_COLUMN) ? B_ITEMS_IN_COLUMN : B_ITEMS_IN_ROW, false) {
	AddItem (new AppMenuItem ()) ;
	fLayerMenu = NULL ;
	fSpaceMenu = NULL ;
	
	BuildItems () ;
} ;

AppMenu:: ~AppMenu () {
	/* do nothing */
} ;

void AppMenu:: BuildItems () {
	BMenu *parent = ItemAt (0)->Submenu() ;
	BMenuItem *item = NULL ;
	BMenu *menu = NULL ;
	BMessage *msg = NULL ;
	BMessage layerList ;
	int32 count = 0 ;
	list_which which = FB_NO_LIST ;
	
	count = parent->CountItems() ;
	for (int32 i=count - 1 ; i>=0 ; i--) delete parent->RemoveItem (i) ;
	
	// 0. switch layer
	fLayerMenu = new BMenu ("Switch Layer") ;
	parent->AddItem (fLayerMenu) ;
	
	if (fb_manager->GetLayerList (&layerList)==B_OK) {
		int32 index = 0 ;
		char *name = NULL ;
		while (layerList.FindString ("name", index++, &name)==B_OK) {
			if (strcmp (name, B_EMPTY_STRING)==0) continue ;
			msg = new BMessage (FB_SWITCH_LAYER) ;
			msg->AddString ("layer_name", name) ;
			item = new BMenuItem (name, msg) ;
			fLayerMenu->AddItem (item) ;
		}
	} else {
		fLayerMenu->Superitem()->SetEnabled (false) ;
	}
	
	// 1. switch workspace
	if (fb_settings->flags & FB_WORKSPACE_CMD) {
		fSpaceMenu = new BMenu ("Switch Workspace") ;
		parent->AddItem (fSpaceMenu) ;
		
		count = count_workspaces () ;
		for (int32 i=0 ; i<count ; i++) {
			char label [3] ;
			msg = new BMessage (FB_SWITCH_WORKSPACE) ;
			msg->AddInt32 ("workspace", i) ;
			sprintf (label, "%2d", i + 1) ;
			if (i<9) {
				fSpaceMenu->AddItem (new BMenuItem (label, msg, 49 + i)) ;
			} else {
				fSpaceMenu->AddItem (new BMenuItem (label, msg)) ;
			}
		}
	} else {
		fSpaceMenu = NULL ;
	}
	
	// 2. shelf
	msg = new BMessage (FB_SWITCH_SHELF) ;
	msg->AddBool ("show", !(fb_settings->flags & FB_ENABLE_SHELF)) ;
	if (fb_settings->flags & FB_ENABLE_SHELF) {
		item = new BMenuItem ("Hide Shelf", msg) ;
	} else {
		item = new BMenuItem ("Show Shelf", msg) ;
	}
	parent->AddItem (item) ;
	
	// 3. separator
	parent->AddSeparatorItem () ;
	
	// 4. preferences
	msg = new BMessage (FB_SHOW_PREFS) ;
	item = new BMenuItem ("Preferences"B_UTF8_ELLIPSIS, msg, 'R') ;
	item->SetTarget (be_app) ;
	parent->AddItem (item) ;
	
	// 5. load settings
	msg = new BMessage (FB_LOAD_PREFS) ;
	item = new BMenuItem ("Load Preferences", msg, 'L') ;
	parent->AddItem (item) ;
	item->SetTarget (be_app) ;
	
	// 6. separator
	parent->AddSeparatorItem () ;
	
	// 7. about window
	msg = new BMessage (B_ABOUT_REQUESTED) ;
	item = new BMenuItem ("About FakeBar"B_UTF8_ELLIPSIS, msg) ;
	item->SetTarget (be_app) ;
	parent->AddItem (item) ;
	
	// 8. quit app
	msg = new BMessage (B_QUIT_REQUESTED) ;
	item = new BMenuItem ("Quit", msg, 'Q') ;
	item->SetTarget (be_app) ;
	parent->AddItem (item) ;
} ;

void AppMenu:: Modify () {
	
	if ((fSpaceMenu!=NULL)!=(fb_settings->flags & FB_WORKSPACE_CMD)) BuildItems() ;
	
	BView *view = Window()->FindView ("layer_bar") ;
	LayerBar *menu = (view) ? cast_as (view, LayerBar) : NULL ;
	LayerInfo* info = (menu) ? menu->Layer() : NULL ;
	int32 index = current_workspace () ;
	BMenuItem *item = NULL ;
	int32 count = 0 ;
	bool shown = false ;
	
	// modify "Switch Layer" menu
	count = fLayerMenu->CountItems () ;
	for (int32 i=0 ; i<count ; i++) {
		item = fLayerMenu->ItemAt (i) ;
		if (info) item->SetEnabled (strcmp (info->LayerName(), item->Label())!=0) ;
		else item->SetEnabled (true) ;
	}
	
	// modify "Switch Workspaces"
	if (fb_settings->flags & FB_WORKSPACE_CMD) {
		count = fSpaceMenu->CountItems () ;
		for (int32 i=0 ; i<count ; i++) {
			fSpaceMenu->ItemAt (i)->SetEnabled (index!=i) ;
		}
	}
	
	// modify "Show Shelf" or "Hide Shelf"
	item = (fb_settings->flags & FB_WORKSPACE_CMD) ? SubmenuAt (0)->ItemAt (2) : 
			SubmenuAt (0)->ItemAt (1) ;
	if (cast_as (Window(), BarWindow)->IsShowingShelf()) {
		item->SetLabel ("Hide Shelf") ;
		item->Message()->ReplaceBool ("show", false) ;
	} else {
		item->SetLabel ("Show Shelf") ;
		item->Message()->ReplaceBool ("show", true) ;
	}
	item->SetEnabled (fb_settings->flags & FB_ENABLE_SHELF) ;
} ;

void AppMenu:: ResizeToPreferred () {
	AppMenuItem *item = cast_as (ItemAt (0), AppMenuItem) ;
	float hor, ver, width, height ;
	
	item->GetContentSize (&width, &height) ;
	fb_settings->get_item_margins (&hor, &ver) ;
	ResizeTo (width + 2*hor, height + 2*ver) ;
	
	SetItemMargins (hor, ver, hor, ver) ;
	SetMaxContentWidth (width) ;
} ;

void AppMenu:: AttachedToWindow () {
	float hor, ver ;
	fb_settings->get_item_margins (&hor, &ver) ;
	SetItemMargins (hor, ver, hor, ver) ;
	ResizeToPreferred () ;
	Modify () ;
	BMenuBar::AttachedToWindow () ;
} ;

void AppMenu:: WindowActivated (bool active) {
	Invalidate () ;
	BMenuBar::WindowActivated (active) ;
} ;

void AppMenu:: MouseMoved (BPoint p, uint32 t, const BMessage *msg) {
	if (	(fb_settings->flags & FB_AUTOFOCUS) || 
			(msg && (fb_settings->flags & FB_DROP_EXTENTION))	) {
		if (!(Window()->IsActive())) Window()->Activate (true) ;
	}
} ;

void AppMenu:: MessageReceived (BMessage *msg) {
	switch (msg->what) {
		case FB_UPDATE_ICON : 
			UpdateIcon () ;
			break ;
		case FB_MODIFY_ITEMS : 
			Modify () ;
			break ;
		default : 
			BMenuBar::MessageReceived (msg) ;
			break ;
	}
} ;

void AppMenu:: MouseDown (BPoint p) {
	uint32 modifiers = 0 ;
	uint32 buttons = 0 ;
	BPoint lastLoc = p ;
	Window()->CurrentMessage()->FindInt32 ("modifiers", (int32*)&modifiers) ;
	
	if (modifiers & B_OPTION_KEY) {
		ConvertToScreen (&p) ;
		ConvertToScreen (&lastLoc) ;
		do {
			if (lastLoc!=p) {
				Window()->MoveBy (p.x - lastLoc.x, p.y - lastLoc.y) ;
			}
			
			snooze (20*1000) ;
			lastLoc = p ;
			GetMouse (&p, &buttons) ;
			ConvertToScreen (&p) ;
		} while (buttons) ;
	} else {
		BMenuBar::MouseDown (p) ;
	}
} ;

void AppMenu::Draw (BRect update) {
	BMenu::Draw (update) ;
	
	BRect bounds = Bounds() ;
	AppMenuItem *item = cast_as (ItemAt (0), AppMenuItem) ;
	if (!(item->IsSelected2())) {
		stroke_3d_rect (this, bounds, LowColor(), true) ;
	}
} ;

void AppMenu:: UpdateIcon () {
	// do nothing...
} ;