//	BarWindow.cpp
//
//	Tetsuo Yamada

#include	"BarWindow.h"
#include	"FakeBar.h"


const settings *fb_settings = NULL ;
LayerManager* fb_manager = NULL ;
extern const char *fb_app_name ;
extern const char *fb_targets_name ;
extern const char *fb_shelf_name ;
extern IOSlave* fb_slave ;

/*
void PrintKeyMessage (	const char *text, const BMessage *message) {
	const char *bytes = NULL ;
	uint32 modifiers = 0 ;
	uint8 *states = NULL ;
	ssize_t num = 0 ;
	int8 byte = 0 ;
	
	message->FindInt32 ("modifiers", (int32*)&modifiers) ;
	message->FindString ("bytes", &bytes) ;
	message->FindData ("states", B_UINT8_TYPE, &states, &num) ;
	message->FindInt8 ("byte", &byte) ;
	
	printf ("%s: \n", text) ;
	printf ("bytes = [%s][", bytes) ;
	for (int32 i=0 ; bytes[i]!='\0' ; i++) printf ("\\0x%x", bytes[i]) ;
	printf ("]\nstates = [") ;
	for (int32 i=0 ; i<num ; i++) printf ("0x%x,", states[i]) ;
	printf ("]\nbyte = [0x%x][%c]\n", byte, byte) ;
	
	message->PrintToStream () ;
} ;*/

//==================================================================
//	class KeyDownFilter
//==================================================================
KeyDownFilter:: KeyDownFilter (BarWindow *owner) 
			: BMessageFilter (B_ANY_DELIVERY, B_ANY_SOURCE, B_KEY_DOWN) {
	fOwner = owner ;
} ;

KeyDownFilter:: ~KeyDownFilter () {
	// do nothing
} ;

filter_result KeyDownFilter:: Filter (BMessage *message, BHandler **target) {
	
	uint32 modifiers = 0 ;
	char * bytes = NULL ;
	BMenuBar *focused = cast_as (fOwner->CurrentFocus (), BMenuBar) ;
	
	message->FindInt32 ("modifiers", (int32*)&modifiers) ;
	message->FindString ("bytes", &bytes) ;
/*#ifdef		CAUTION
	PrintKeyMessage ("key down filter received message", message) ;
#endif*/
	
	switch (bytes[0]) {
		case B_UP_ARROW : 
		case B_DOWN_ARROW : 
		case B_RIGHT_ARROW : 
		case B_LEFT_ARROW : // focus LayerBar if there's no focused menu bar.
			if (modifiers!=0 || !(fOwner->fLayerBar)) {
				// do nothing
			} else if (!focused) {
				fOwner->SetKeyMenuBar (fOwner->fLayerBar) ;
				message->ReplaceInt32 ("modifiers", B_COMMAND_KEY) ;
				message->ReplaceInt8 ("byte", B_ESCAPE) ;
				message->ReplaceString ("bytes", "\x1b") ;
				*target = fOwner->fLayerBar ;
			} else {
				fOwner->SetKeyMenuBar (fOwner->fAppMenu) ;
			}
			break ;
		case B_TAB : 
			if (modifiers!=0 && !(modifiers & B_SHIFT_KEY)) 
				break ;
			if (!focused) {
				fOwner->SetKeyMenuBar (fOwner->fLayerBar) ;
				break ;
			}
			
			// reject focus from current focused menu...
			message->ReplaceInt32 ("modifiers", 0) ;
			message->ReplaceInt8 ("byte", B_ESCAPE) ;
			message->ReplaceString ("bytes", "\x1b") ;
			*target = focused ;
			fOwner->SetKeyMenuBar (focused) ;
			
			break ;
		case B_ESCAPE : 
			if (modifiers & B_COMMAND_KEY) {
				fOwner->SetKeyMenuBar (fOwner->fAppMenu) ;
			} else if (!focused) {
				fOwner->Activate (false) ;
				return B_SKIP_MESSAGE ;
			}
			break ;
	}
	
	return B_DISPATCH_MESSAGE ;
} ;

//==================================================================
//	class KeyUpFilter
//==================================================================

KeyUpFilter:: KeyUpFilter (BarWindow *owner) 
		: BMessageFilter (B_ANY_DELIVERY, B_ANY_SOURCE, B_KEY_UP) {
	fOwner = owner ;
} ;

KeyUpFilter:: ~KeyUpFilter () {
	// do nothing
} ;

BMenuBar* KeyUpFilter::NextMenuBar (bool positive) const {
	BMenuBar *keyBar = fOwner->KeyMenuBar () ;
	BMenuBar *bar = NULL ;
	int32 index = 0 ;
	int32 count = fOwner->CountHandlers () ;
	bool indexed = false ;
	
	for (int32 i=0 ; i<2*count ; i++) {
		bar = cast_as (fOwner->HandlerAt (index), BMenuBar) ;
		if (bar) {
			if (indexed && !is_kind_of (bar, ShelfBar) && 
				bar->Frame().LeftTop()!=FB_UNSHOWN_POINT) return bar ;
			if (!keyBar) keyBar = bar ;
			if (keyBar==bar) indexed = true ;
		}
		
		if (positive) {
			index++ ;
			if (index>=count) index -= count ;
		} else {
			index-- ;
			if (index<0) index += count ;
		}
	}
	return NULL ;
} ;

filter_result KeyUpFilter:: Filter (BMessage *message, BHandler **target) {
	uint32 modifiers = 0 ;
	const char *bytes = NULL ;
	BMenuBar *bar = NULL ;
	
	message->FindInt32 ("modifiers", (int32*)&modifiers) ;
	message->FindString ("bytes", &bytes) ;
/*#ifdef		CAUTION
	PrintKeyMessage ("key up filter received message", message) ;
#endif*/
	
	switch (bytes[0]) {
		case B_TAB : 
			if (modifiers!=0 && !(modifiers & B_SHIFT_KEY)) break ;
			
			bar = NextMenuBar (!(modifiers & B_SHIFT_KEY)) ;
			fOwner->SetKeyMenuBar (bar) ;
			*target = bar ;
			message->what = B_KEY_DOWN ;
			message->ReplaceInt32 ("modifiers", B_COMMAND_KEY) ;
			message->ReplaceInt8 ("byte", B_ESCAPE) ;
			message->ReplaceString ("bytes", "\x1b") ;
			
			break ;
		case B_ENTER : 
		case B_SPACE : 
			bar = fOwner->KeyMenuBar() ;
			if (bar) {
				bar->MakeFocus (false) ;
				fOwner->SetKeyMenuBar (NULL) ;
			}
			break ;
	}
	
	return B_DISPATCH_MESSAGE ;
} ;

//==================================================================
//	class BarWindow
//==================================================================
BarWindow:: BarWindow () 
		: LayerManager (BRect (-300, -300, -100, -100), fb_app_name, 
			B_NOT_MINIMIZABLE|B_WILL_ACCEPT_FIRST_CLICK|B_NOT_MOVABLE|
			B_NOT_RESIZABLE|B_NO_WORKSPACE_ACTIVATION|B_AVOID_FRONT, 
			FB_LAYER_ITEM_LIST, FB_MENU_ITEM_LIST) {
	
	fShelfShown = true ;
	
	SetLook (B_BORDERED_WINDOW_LOOK) ;
	SetFeel (B_NORMAL_WINDOW_FEEL) ;
	SetSizeLimits (0, 3000, 0, 3000) ;
	
	SetWorkspaces (fSettings.workspaces) ;
	
	fb_manager = this ;
	fb_settings = &fSettings ;
	
	LayerInfo* info = NULL ;
	BMessage *shortcut = NULL ;
	fLayerBar = NULL ;
	fAppMenu = NULL ;
	entry_ref shelf ;
	
	//	bottom view
	BView* bottom = new BView (Bounds(), "bottom", B_FOLLOW_ALL, B_WILL_DRAW) ;
	bottom->SetViewColor (tint_color (ui_color2 (P_STANDARD_GRAY), B_DARKEN_4_TINT)) ;
	AddChild (bottom) ;
	
	//	LayerBar
	fLayerBar = new LayerBar () ;
	bottom->AddChild (fLayerBar) ;
	info = FindLayer (fSettings.layers[current_workspace()]) ;
	if (info && strcmp (info->LayerName(), B_EMPTY_STRING)==0) info = NULL ;
	fLayerBar->SetLayer (info) ;
	
	//	AppMenu
	fAppMenu = new AppMenu () ;
	bottom->AddChild (fAppMenu) ;
	SetKeyMenuBar (fAppMenu) ;
	
	//	ShelfBar
	fShelfBar = new ShelfBar () ;
	if (get_settings (fb_shelf_name, &shelf)==B_OK) 
		fShelf = new FBShelf (&shelf, fShelfBar) ;
	else fShelf = new FBShelf (fShelfBar) ;
	bottom->AddChild (fShelfBar) ;
	
	//	filters
	AddCommonFilter (new KeyDownFilter (this)) ;
	AddCommonFilter (new KeyUpFilter (this)) ;
	
	//	shortcuts
	shortcut = new BMessage (FB_SWITCH_LAYER) ;
	shortcut->AddBool ("direction", true) ;
	AddShortcut (B_DOWN_ARROW, 0, shortcut) ;
	AddShortcut (B_RIGHT_ARROW, 0, new BMessage (shortcut)) ;
	
	shortcut = new BMessage (FB_SWITCH_LAYER) ;
	shortcut->AddBool ("direction", false) ;
	AddShortcut (B_UP_ARROW, 0, shortcut) ;
	AddShortcut (B_LEFT_ARROW, 0, new BMessage (shortcut)) ;
	
	Reshape (true) ;
} ;

BarWindow:: ~BarWindow () {
	fChores = -1 ;
	if (fLayerBar) delete fLayerBar ;
	if (fAppMenu) delete fAppMenu ;
	if (fShelfBar) delete fShelfBar ;
	fLayerBar = NULL ;
	fAppMenu = NULL ;
	fShelfBar = NULL ;
	fShelf = NULL ;
} ;

bool BarWindow:: QuitRequested () {
	fChores = -1 ;
	be_app->PostMessage (B_QUIT_REQUESTED) ;
	return LayerManager::QuitRequested () ;
} ;

void BarWindow:: Quit () {
	// first of all, delete shelf and save its data.
	if (fShelf && fShelf->Looper()) {
		RemoveHandler (fShelf) ;
		delete fShelf ;
	}
	// next, remove views which associates LayerInfo before LayerInfo(s) deleted.
	if (fLayerBar && fLayerBar->Looper()) fLayerBar->RemoveSelf () ;
	if (fAppMenu && fAppMenu->Looper()) fAppMenu->RemoveSelf () ;
	if (fShelfBar && fShelfBar->Looper()) fShelfBar->RemoveSelf () ;
	
	fChores = -1 ;
	LayerManager::Quit () ;
} ;

void BarWindow:: Reshape (bool maximize) {
#define		FB_NUM_BARS		3
	BRect rect (0, 0, -2, -2) ;
	BRect screen = BScreen (this).Frame() ;
	BRect frame = Frame() ;
	BMenuBar *bars[FB_NUM_BARS] ;
	bool showLayer = (fLayerBar->Layer()!=NULL) ;
	
	if (fb_settings->flags & FB_SHELF_BOTTOM) {
		if (IS_POSITIVE) {
			bars[0] = fAppMenu ;
			bars[1] = fLayerBar ;
			bars[2] = fShelfBar ;
		} else {
			bars[0] = fShelfBar ;
			bars[1] = fLayerBar ;
			bars[2] = fAppMenu ;
		}
	} else {
		if (IS_POSITIVE) {
			bars[0] = fAppMenu ;
			bars[1] = fShelfBar ;
			bars[2] = fLayerBar ;
		} else {
			bars[0] = fLayerBar ;
			bars[1] = fShelfBar ;
			bars[2] = fAppMenu ;
		}
	}
	
	// layout views and get preferred size.
	for (int32 i=0 ; i<FB_NUM_BARS ; i++) {
		if (	(!showLayer && is_kind_of (bars[i], LayerBar)) || 
				(!maximize && !is_kind_of (bars[i], AppMenu)) || 
				(is_kind_of (bars[i], ShelfBar) && 
					(!(fb_settings->flags & FB_ENABLE_SHELF) || !fShelfShown))  ) {
			bars[i]->MoveTo (FB_UNSHOWN_POINT) ;
			continue ;
		}
		
		if (IS_COLUMN) bars[i]->MoveTo (0, rect.bottom + 2) ;
		else bars[i]->MoveTo (rect.right + 2, 0) ;
		
		bars[i]->InvalidateLayout () ;
		bars[i]->ResizeToPreferred () ;
		rect = bars[i]->Frame() ;
	}
	
	// resize window if needed.
	if (frame.Width()!=rect.right || frame.Height()!=rect.bottom) {
		ResizeTo (rect.right, rect.bottom) ;
	}
	
	// move window to the preferred loc.
	rect = fAppMenu->Frame() ;
	switch (fSettings.location) {
		case FB_LEFT_TOP : 
			MoveTo (-rect.left, -rect.top) ;
			break ;
		case FB_RIGHT_TOP :
			MoveTo (screen.right - rect.right, -rect.top) ;
			break ;
		case FB_RIGHT_BOTTOM :
			MoveTo (screen.right - rect.right, screen.bottom - rect.bottom) ;
			break ;
		case FB_LEFT_BOTTOM :
			MoveTo (-rect.left, screen.bottom - rect.bottom) ;
			break ;
	}
} ;

void BarWindow:: DispatchMessage (BMessage *msg, BHandler *handler) {
	switch (msg->what) {
		case FB_ITEMS_FOUND : {
			LayerManager::DispatchMessage (msg, handler) ;
			Lock () ;
			
			LayerInfo *info = cast_as (handler, LayerInfo) ;
			if (info && fLayerBar->Layer()==info) {
				fLayerBar->RescanItems () ;
				if (!IsHidden()) Reshape (true) ;
			}
			
			Unlock () ;
			break ;
		}
		case FB_UPDATE_ICON : 
		case FB_MODIFY_ITEMS : // message to AppMenu
			handler = fAppMenu ;
			LayerManager::DispatchMessage (msg, handler) ;
			break ;
			
		case FB_CLEAR_REPLICANTS : // message to ShelfBar
			handler = fShelf ;
			LayerManager::DispatchMessage (msg, handler) ;
			break ;
			
		default : 
			LayerManager::DispatchMessage (msg, handler) ;
			break ;
	}
} ;

void BarWindow:: MessageReceived (BMessage *msg) {
	switch (msg->what) {
		case FB_SWITCH_LAYER : 
			SwitchLayer (msg) ;
			break ;
		case FB_SWITCH_WORKSPACE : {
			int32 workspace = current_workspace() ;
			msg->FindInt32 ("workspace", &workspace) ;
			if (workspace!=current_workspace()) activate_workspace (workspace) ;
			break ;
		}
		case FB_SWITCH_SHELF : // show or hide shelf
			BeginViewTransaction () ;
			msg->FindBool ("show", &fShelfShown) ;
			Reshape (true) ;
			EndViewTransaction () ;
			fAppMenu->Modify () ;
			break ;
		default : 
			LayerManager::MessageReceived (msg) ;
			break ;
	}
} ;

void BarWindow:: SwitchLayer (BMessage *msg) {
	
	bool positive = true ;
	status_t err = B_OK ;
	const char *name = NULL ;
	LayerInfo *info = NULL ;
	
	if (msg->FindString ("layer_name", &name)==B_OK) {
		info = FindLayer (name) ;
	} else if (msg->FindBool ("direction", &positive)==B_OK) {
		info = fLayerBar->Layer() ;
		
		while (true) {
			info = FindNextLayer (info, (positive) ? 1 : -1) ;
			if (!info) break ;
			if (strcmp (info->LayerName(), B_EMPTY_STRING)!=0) break ;
		}
	}
	
	if (!info) {
		// why ?
		beep () ;
		return ;
	}
	
	BeginViewTransaction () ;
	fLayerBar->SetLayer (info) ;
	SetKeyMenuBar (fLayerBar) ;
	Reshape (true) ;
	EndViewTransaction () ;
	fAppMenu->Modify () ;
} ;

void BarWindow:: UpdateSettings (settings *s) {
	
	LayerInfo *info = NULL ;
	int32 count = 0 ;
	BView *bottom = FindView ("bottom") ;
	int32 d = fSettings.compare (s, true) ;
	
	if (d==0) return ;
	
	MoveTo (-2000, -2000) ; // move to the place the user cannot see.
	BeginViewTransaction () ;
	
	if (d==5) {
		// we must replace almost all MenuBars, all LayerItems, all LayerMenus.
		bottom->RemoveChild (fLayerBar) ;
		delete fLayerBar ;
		bottom->RemoveChild (fAppMenu) ;
		delete fAppMenu ;
		
		count = CountHandlers () ;
		for (int32 i=0 ; i<count ; i++) {
			info = cast_as (HandlerAt (i), LayerInfo) ;
			if (!info) continue ;
			
			info->Initialize () ;
			info->SetEnabled (true) ;
		}
		
		fSettings.set_to (*s) ;
		
		fLayerBar = new LayerBar () ;
		fAppMenu = new AppMenu () ;
		bottom->AddChild (fLayerBar) ;
		bottom->AddChild (fAppMenu) ;
		SetKeyMenuBar (fAppMenu) ;
		
	} else {
		fSettings.set_to (*s) ;
		
		fLayerBar->InvalidateLayout () ;
		fLayerBar->ResizeToPreferred () ;
		fAppMenu->InvalidateLayout () ;
		fAppMenu->ResizeToPreferred () ;
	}
	
	SetWorkspaces (fSettings.workspaces) ;
	
	info = FindLayer (fSettings.layers[current_workspace()]) ;
	fLayerBar->SetLayer (info) ;
	
	Reshape (!((fSettings.flags & FB_MINIMIZE) && !IsActive())) ;
	if (!IsActive()) Activate (true) ;
	
	EndViewTransaction () ;
	fAppMenu->Modify () ;
} ;

void BarWindow:: WindowActivated (bool active) {
	if (fSettings.flags & FB_MINIMIZE) {
		Reshape (active) ;
	}
	LayerManager::WindowActivated (active) ;
} ;

void BarWindow:: WorkspaceActivated (int32 workspace, bool active) {
	if (active) {
		bool maximize = (!(fSettings.flags & FB_MINIMIZE) || IsActive()) ;
		LayerInfo *info = FindLayer (fSettings.layers[workspace]) ;
		
		BeginViewTransaction () ;
		if (info!=fLayerBar->Layer()) fLayerBar->SetLayer (info) ;
		Reshape (maximize) ;
		EndViewTransaction () ;
		fAppMenu->Modify () ;
	}
} ;

void BarWindow:: ScreenChanged (BRect frame, color_space mode) {
	BRect rect = fAppMenu->Frame() ;
	if (fSettings.is_enabled_in (current_workspace())) {
		switch (fb_settings->location) {
			case FB_LEFT_TOP : 
				MoveTo (-rect.left, -rect.top) ;
				break ;
			case FB_RIGHT_TOP :
				MoveTo (frame.right - rect.right, -rect.top) ;
				break ;
			case FB_RIGHT_BOTTOM :
				MoveTo (frame.right - rect.right, frame.bottom - rect.bottom) ;
				break ;
			case FB_LEFT_BOTTOM :
				MoveTo (-rect.left, frame.bottom - rect.bottom) ;
				break ;
		}
	} else {
		rect = Bounds() ;
		MoveTo (-rect.Width() - 10, 0) ;
	}
	
	LayerManager::ScreenChanged (frame, mode) ;
} ;

void BarWindow:: Show () {
	LayerManager::Show () ;
	
	fChores = spawn_thread (Chores, "chores_slave", B_LOW_PRIORITY, this) ;
	if (fChores>=0) resume_thread (fChores) ;
} ;

status_t BarWindow:: Chores (void *data) {
	BarWindow* bar = (BarWindow*)data ;
	bar->ChoreTask () ;
	return B_OK ;
} ;

void BarWindow:: ChoreTask () {
	// send pulse, key polling, check layer items...
	thread_id thread = Thread() ;
	key_info kinfo ;
	bigtime_t intervalPulse = 1000*1000 ;	//	to update AppMenuItem's icon (but not supported currently)
	bigtime_t lastPulse = 0 ;
	bigtime_t intervalCheck = 2500*1000 ;	//	to check layer items.
	bigtime_t lastCheck = 0 ;
	LayerInfo *target = NULL ;
	int32 checkIndex = 0 ;
	bigtime_t now = 0 ;
	int32 wsCount = count_workspaces() ;	//	to adjust "Switch Workspace" command
	
	while (BLooper::LooperForThread (thread)) {
		now = system_time () ;
		
		/** CURRENTLY NOT USED 
		if (now - lastPulse > intervalPulse) {
			// send pulse.
			PostMessage (FB_UPDATE_ICON) ;
			lastPulse = now ;
		}**/
		
		if (wsCount != count_workspaces() && Lock()) {
			// if the count of workspaces has changed, tell it to the window.
			PostMessage (FB_MODIFY_ITEMS) ;
			wsCount = count_workspaces() ;
			Unlock() ;
		}
		
		if (now - lastCheck > intervalCheck && Lock()) {
			// check layer item
			if (fLayerBar) {
				target = fLayerBar->Layer() ;
				if (target && checkIndex >= target->CountItems()) checkIndex = 0 ;
			} else {
				target = NULL ;
			}
			
			if (target) {
				BMessage message (FB_CHECK_ITEMS) ;
				message.AddInt32 ("index", checkIndex) ;
				PostMessage (&message, target) ;
				checkIndex++ ;
			} else {
				checkIndex = 0 ;
			}
			lastCheck = now ;
			
			Unlock() ;
		}
		
		// key polling...
		if (get_key_info (&kinfo)==B_OK && be_app->Lock()) {
			FakeBar *app = cast_as (be_app, FakeBar) ;
			if (!(app->fHotKey)) {
				be_app->Unlock() ;
				goto Break ;
			}
			for (int32 i=0 ; i<16 ; i++) {
				if (kinfo.key_states[i]!=(app->fHotKey)[i]) {
					be_app->Unlock() ;
					goto Break ;
				}
			}
			be_app->Unlock() ;
			
			if (Lock()) {
				if (!IsHidden() && !IsActive()) Activate (true) ;
				Unlock() ;
			}
		}
		
		// rest
Break : 
		
		snooze (30*1000) ;
	}
} ;

bool BarWindow:: IsShowingShelf () const {
	return fShelfShown ;
} ;