//	FBBase.cpp
//
//	Tetsuo Yamada

#include	"FBBase.h"


//-----------------------------------------------------------
//	globals
//-----------------------------------------------------------
const char *fb_app_name = "FakeBar" ;
const char *fb_signature = "application/x-vnd.TY-FakeBar" ;
const char *fb_settings_name = "settings" ;
const char *fb_layers_name = "layers" ;
const char *fb_targets_name = "targets" ;
const char *fb_shelf_name = "shelf" ;

extern const settings* fb_settings ;

extern void write_layer_attr (BFile*, bool) ;

//===========================================================
//	struct settings
//===========================================================
settings:: settings () {
	location = FB_LEFT_BOTTOM ;
	direction = FB_BOTTOM_TO_TOP ;
	forInvoke = FB_SECONDARY_DOWN ;
	forExpand = FB_PRIMARY_DOWN ;
	flags = FB_DRAW_LABEL|FB_WORKSPACE_CMD|FB_DRAW_BORDER|FB_SHELF_BOTTOM ;
	which = B_MINI_ICON ;
	workspaces = B_ALL_WORKSPACES ;
	shelfSize = 1 ;
	for (int32 i=0 ; i<32 ; i++) {
		layers[i] = new char (strlen (B_EMPTY_STRING) + 1) ;
		strcpy (layers[i], B_EMPTY_STRING) ;
	}
} ;

settings:: ~settings () {
	for (int32 i=0 ; i<32 ; i++) {
		delete[] layers[i] ;
	}
} ;

status_t settings:: archive_to (BMessage *msg) const {
	status_t err = B_OK ;
	
	err = msg->AddInt32 ("location", location) ;
	if (err!=B_OK) return err ;
	
	err = msg->AddInt32 ("direction", direction) ;
	if (err!=B_OK) return err ;
	
	err = msg->AddInt32 ("forInvoke", forInvoke) ;
	if (err!=B_OK) return err ;
	
	err = msg->AddInt32 ("forExpand", forExpand) ;
	if (err!=B_OK) return err ;
	
	err = msg->AddInt32 ("flags", flags) ;
	if (err!=B_OK) return err ;
	
	err = msg->AddInt32 ("which", which) ;
	if (err!=B_OK) return err ;
	
	err = msg->AddInt32 ("workspaces", (int32)workspaces) ;
	if (err!=B_OK) return err ;
	
	err = msg->AddInt32 ("shelfSize", shelfSize) ;
	if (err!=B_OK) return err ;
	
	for (int32 i=0 ; i<32 ; i++) {
		err = msg->AddString ("layerName", layers[i]) ;
		if (err!=B_OK) return err ;
	}
	
	return B_OK ;
} ;

status_t settings:: set_to (BMessage *msg) {
	status_t err = B_OK ;
	char *string = NULL ;
	
	err = msg->FindInt32 ("location", (int32*)&location) ;
	if (err!=B_OK) return err ;
	
	err = msg->FindInt32 ("direction", (int32*)&direction) ;
	if (err!=B_OK) return err ;
	
	err = msg->FindInt32 ("forInvoke", (int32*)&forInvoke) ;
	if (err!=B_OK) return err ;
	
	err = msg->FindInt32 ("forExpand", (int32*)&forExpand) ;
	if (err!=B_OK) return err ;
	
	err = msg->FindInt32 ("flags", (int32*)&flags) ;
	if (err!=B_OK) return err ;
	
	err = msg->FindInt32 ("which", (int32*)&which) ;
	if (err!=B_OK) return err ;
	
	err = msg->FindInt32 ("workspaces", (int32*)&workspaces) ;
	if (err!=B_OK) return err ;
	
	err = msg->FindInt32 ("shelfSize", &shelfSize) ;
	if (err!=B_OK) shelfSize = 4 ;
	
	for (int32 i=0 ; i<32 ; i++) {
		err = msg->FindString ("layerName", i, &string) ;
		if (err==B_OK) {
			delete[] layers[i] ;
			layers[i] = new char [strlen (string) + 1] ;
			strcpy (layers[i], string) ;
		}
	}
	
	return B_OK ;
} ;

void settings:: set_to (const settings& src) {
	location = src.location ;
	direction = src.direction ;
	forInvoke = src.forInvoke ;
	forExpand = src.forExpand ;
	flags = src.flags ;
	which = src.which ;
	workspaces = src.workspaces ;
	shelfSize = src.shelfSize ;
	for (int32 i=0 ; i<32 ; i++) set_layer (src.layers[i], i) ;
} ;

int32 settings:: compare (const settings *alt, bool all) const {
	if (direction!=alt->direction) return 5 ;
	if (all) {
		if (workspaces!=alt->workspaces) return 4 ;
		for (int32 i=0 ; i<32 ; i++) {
			if (strcmp (layers[i], alt->layers[i])!=0) return 3 ;
		}
	} else {
		int32 cw = current_workspace () ;
		uint32 bit = 0x01 << cw ;
		if ((bit & workspaces)!=(bit & alt->workspaces)) return 4 ;
		if (strcmp (layers[cw], alt->layers[cw])!=0) return 3 ;
	}
	
	if (which!=alt->which) return 2 ;
	if ((flags & FB_DRAW_LABEL)!=(alt->flags & FB_DRAW_LABEL)) return 2 ;
	if ((flags & FB_ENABLE_SHELF)!=(alt->flags & FB_ENABLE_SHELF)) return 2 ;
	if (location!=alt->location) return 2 ;
	if (shelfSize!=alt->shelfSize) return 2 ;
	
	if (forInvoke!=alt->forInvoke) return 1 ;
	if (forExpand!=alt->forExpand) return 1 ;
	if (flags!=alt->flags) return 1 ;
	
	return 0 ;
} ;

//===========================================================
//	for alert
//===========================================================
void alert (int32 kind, const char *msg, status_t err) {
	beep () ;
	char *text = new char [strlen (msg) + 120] ;
	
	switch (kind) {
		case FB_READ_ALERT : 
			sprintf (text, "FakeBar Alert: \n\nFailed to read a file \"%s\".\n%s", 
					msg, strerror (err)) ;
			break ;
		case FB_WRITE_ALERT :
			sprintf (text, "FakeBar Alert: \n\nFailed to write a file \"%s\".\n%s", 
					msg, strerror (err)) ;
			break ;
		case FB_LAUNCH_ALERT : 
			sprintf (text, "FakeBar Alert: \n\nFailed to launch a file \"%s\".\n%s", 
					msg, strerror (err)) ;
			break ;
		case FB_READ_LAYER_ALERT : 
			sprintf (text, "FakeBar Alert: \n\nA layer file \"%s\" was ignored.", 
					msg) ;
			break ;
		case FB_INSTALL_ALERT : 
			sprintf (text, "FakeBar Alert: \n\nFailed to install an app \"%s\".\n%s", 
					msg, strerror (err)) ;
			break ;
		default : 
			sprintf (text, "FakeBar Alert: \n\n%s\n%s", msg, strerror (err)) ;
			break ;
	}
	
	BAlert *a = new BAlert ("FakeBar Alert", text, "OK", NULL, NULL, 
				B_WIDTH_AS_USUAL, B_WARNING_ALERT) ;
	a->Go() ;
} ;

//===========================================================
//	for settings files
//===========================================================
status_t get_settings (const char *name, entry_ref *ref) {
	status_t err = B_OK ;
	char path [B_PATH_NAME_LENGTH + 1] ;
	
	err = find_directory (B_USER_SETTINGS_DIRECTORY, dev_for_path ("/boot"), false, 
			path, B_PATH_NAME_LENGTH) ;
	if (err!=B_OK) return err ;
	
	strcat (path, "/") ;
	strcat (path, fb_app_name) ;
	if (strcmp (name, fb_app_name)==0) goto End ;
	
	strcat (path, "/") ;
	strcat (path, name) ;
	
End :
	err = get_ref_for_path (path, ref) ;
	if (err!=B_OK) return err ;
	
	return B_OK ;
} ;

void init_settings () {
	status_t err = B_OK ;
	BDirectory settings, fakebar, layers ;
	BFile targets ;
	char path [B_PATH_NAME_LENGTH + 1] ;
	BEntry entry ;
	
	// get settings directory
	err = find_directory (B_USER_SETTINGS_DIRECTORY, dev_for_path ("/boot"), 
			false, path, B_PATH_NAME_LENGTH) ;
	if (err!=B_OK) goto Err1 ;
	
	err = settings.SetTo (path) ;
	if (err!=B_OK) goto Err1 ;
	
	err = settings.InitCheck () ;
	if (err!=B_OK) goto Err1 ;
	
	// get settings/FakeBar directory
	err = settings.FindEntry (fb_app_name, &entry, true) ;
	if (err!=B_OK) {
		err = settings.CreateDirectory (fb_app_name, &fakebar) ;
	} else {
		err = fakebar.SetTo (&entry) ;
		if (err==B_OK) err = fakebar.InitCheck () ;
	}
	if (err!=B_OK) goto Err2 ;
	
	// get settings/FakeBar/layers directory
	err = fakebar.FindEntry (fb_layers_name, &entry, true) ;
	if (err!=B_OK) {
		err = fakebar.CreateDirectory (fb_layers_name, &layers) ;
	} else {
		err = layers.SetTo (&entry) ;
		if (err==B_OK) err = layers.InitCheck () ;
	}
	if (err!=B_OK) goto Err3 ;
	
	// get settings/FakeBar/targets file
	err = targets.SetTo (&fakebar, fb_targets_name, B_READ_WRITE|B_CREATE_FILE) ;
	if (err!=B_OK) goto Err4 ;
	
	err = targets.InitCheck () ;
	if (err!=B_OK) goto Err4 ;
	
	write_layer_attr (&targets, false) ;
	return ;
	
Err1 :
	sprintf (path, "FakeBar Error: \n\tFailed to get the user settings directory.\n\t%s", strerror (err)) ;
	goto End ;
Err2 :
	sprintf (path, "FakeBar Error: \n\tFailed to get a FakeBar settings directory.\n\t%s", strerror (err)) ;
	goto End ;
Err3 :
	sprintf (path, "FakeBar Error: \n\tFailed to get a FakeBar's Layers directory.\n\t%s", strerror (err)) ;
	goto End ;
Err4 :
	sprintf (path, "FakeBar Error: \n\tFailed to get a targets file.\n\t%s", strerror (err)) ;
	goto End ;
End : 
	BAlert *a = new BAlert ("FakeBar Error", path, "Exit", NULL, NULL, 
			B_WIDTH_AS_USUAL, B_STOP_ALERT) ;
	a->Go () ;
	exit (1) ;
} ;

status_t get_settings_node (BNode *node) {
	status_t err = B_OK ;
	entry_ref ref ;
	
	err = get_settings (fb_settings_name, &ref) ;
	if (err!=B_OK) return err ;
	
	err = node->SetTo (&ref) ;
	if (err!=B_OK) return err ;
	
	err = node->InitCheck () ;
	if (err!=B_OK) return err ;
	
	return B_OK ;
} ;