//	FBShelf.cpp
//
//	Tetsuo Yamada


#include	"FBShelf.h"

extern const settings*		fb_settings ;
extern const char*			fb_shelf_name ;
extern const char*			fb_app_name ;

//===================================================================
//	class FBShelf
//===================================================================
FBShelf::FBShelf (BView *view) 
			: BShelf (view) {
	// do nothing
} ;

FBShelf::FBShelf (const entry_ref *ref, BView *view) 
			: BShelf (ref, view) {
	// do nothing
} ;

FBShelf::FBShelf (BDataIO* stream, BView *view) 
			: BShelf (stream, view) {
	// do nothing
} ;

FBShelf::~FBShelf () {
	status_t err = B_OK ;
	
	if (IsDirty()) {
		entry_ref ref ;
		BDirectory dir ;
		BFile file ;
		
		err = get_settings (fb_app_name, &ref) ;
		if (err!=B_OK) goto Fail ;
		
		err = dir.SetTo (&ref) ;
		if (err!=B_OK) goto Fail ;
		
		err = dir.CreateFile (fb_shelf_name, &file, false) ;
		if (err==B_FILE_EXISTS) return ;
		if (err!=B_OK) goto Fail ;
		
		err = file.InitCheck () ;
		if (err!=B_OK) goto Fail ;
		
		err = SetSaveLocation (&file) ;
		if (err!=B_OK) goto Fail ;
		
		err = Save () ;
		if (err!=B_OK) goto Fail ;
		
		SetDirty (false) ;
	}
	
	return ;
	
Fail : 
	alert (FB_WRITE_ALERT, fb_shelf_name, err) ;
} ;

void FBShelf::MessageReceived (BMessage *msg) {
	int32 count = CountReplicants () ;
	BView *replicant = NULL ;
	
	switch (msg->what) {
		case FB_CLEAR_REPLICANTS : 
			for (int32 i = count - 1 ; i>=0 ; i--) {
				ReplicantAt (i, &replicant) ;
				DeleteReplicant (replicant) ;
			}
			break ;
		default : 
			BShelf::MessageReceived (msg) ;
			break ;
	}
} ;

//===================================================================
//	class ShelfBar
//===================================================================
ShelfBar:: ShelfBar () 
			: BMenuBar (BRect (0, 0, 10, 10), "shelf_bar", 
				B_FOLLOW_NONE, B_ITEMS_IN_ROW, false) {
	BMenuItem *item = NULL ;
	BMessage *msg = NULL ;
	
	fPopper = new BPopUpMenu ("popper", false, false, B_ITEMS_IN_COLUMN) ;
	
	// 0. hide shelf
	msg = new BMessage (FB_SWITCH_SHELF) ;
	msg->AddBool ("show", false) ;
	item = new BMenuItem ("Hide Shelf", msg) ;
	fPopper->AddItem (item) ;
	
	// 1. clear all replicants
	msg = new BMessage (FB_CLEAR_REPLICANTS) ;
	item = new BMenuItem ("Clear All Replicants", msg) ;
	fPopper->AddItem (item) ;
} ;

ShelfBar::~ShelfBar () {
	delete fPopper ;
} ;

void ShelfBar:: Draw (BRect update) {
	stroke_3d_rect (this, Bounds(), LowColor(), true) ;
} ;

void ShelfBar:: ResizeToPreferred () {
	float width, height, hor, ver ;
	fb_settings->get_item_size (&width, &height) ;
	fb_settings->get_item_margins (&hor, &ver) ;
	
	if (IS_COLUMN) {
		ResizeTo (width + 2*hor, (fb_settings->shelfSize)*(height + 2*ver + 1) - 1) ;
	} else {
		ResizeTo ((fb_settings->shelfSize)*(width + 2*hor + 1) - 1, height + 2*ver) ;
	}
} ;

void ShelfBar:: MouseDown (BPoint p) {
	uint32 buttons = 0, modifiers = 0 ;
	BRect openRect ;
	BMessage *msg = Looper()->CurrentMessage() ;
	
	msg->FindInt32 ("buttons", (int32*)&buttons) ;
	msg->FindInt32 ("modifiers", (int32*)&modifiers) ;
	
	if ((buttons & B_SECONDARY_MOUSE_BUTTON) || (modifiers & B_CONTROL_KEY)) {
		// popup
		fPopper->SetTargetForItems (Window()) ;
		openRect.Set (p.x, p.y, p.x, p.y) ;
		ConvertToScreen (&openRect) ;
		fPopper->Go (ConvertToScreen (p), true, true, openRect, false) ;
	} else {
		BMenuBar::MouseDown (p) ;
	}
} ;

void ShelfBar:: MouseMoved (BPoint p, uint32 t, const BMessage *msg) {
	if (	(fb_settings->flags & FB_AUTOFOCUS) || 
			(msg && (fb_settings->flags & FB_DROP_EXTENTION))	) {
		if (!(Window()->IsActive())) Window()->Activate (true) ;
	}
} ;