//	FakeBar.cpp
//
//	Tetsuo Yamada

#include	"FakeBar.h"


//==============================================================
//	main
//==============================================================
int main (int32 argc, char** argv) {
	bool option = false ;
	bool convert = false ;
	
	for (int32 i=1 ; i<argc ; i++) {
		if (strcmp (argv[i], "-hotkey")==0) {
			option = true ;
		} else if (strcmp (argv[i], "-convert")==0) {
			convert = true ;
		} else {
			printf ("To set a hot key, add option \"-hotkey\"\n"
					"To convert a settings for v1.0.4, add option \"-convert\"\n") ;
			return 0 ;
		}
	}
	
	FakeBar app (convert, option) ;
	app.Run () ;
	return 0 ;
} ;


IOSlave* fb_slave = NULL ;
extern const char* fb_signature ;
extern LayerManager* fb_manager ;

//==============================================================
//	class FakeBar
//==============================================================
FakeBar:: FakeBar (bool convert, bool option) 
		: BApplication (fb_signature) {
	init_settings () ;
	fHotKey = NULL ;
	fPrefs = NULL ;
	
	fSlave = new IOSlave () ;
	fSlave->Run () ;
	fb_slave = fSlave ;
	
	if (convert) ConvertSettings () ;
	
	if (option) {
		bool flag = false ;
		HotKeyWindow *hwin = new HotKeyWindow (&fHotKey, &flag) ;
		hwin->Show () ;
		while (!flag) snooze (20*1000) ;
	} else {
		status_t err = B_OK ;
		BNode node ;
		fHotKey = new uint8 [16] ;
		
		err = get_settings_node (&node) ;
		if (err!=B_OK) return ;
		
		err = node.ReadAttr ("hotKey", B_UINT8_TYPE, 0, fHotKey, 16) ;
		if (err<B_OK) {
			delete[] fHotKey ;
			fHotKey = NULL ;
		}
	}
} ;

void FakeBar::ReadyToRun () {
	BarWindow* window = new BarWindow () ;
	window->Show () ;
} ;

FakeBar:: ~FakeBar () {
	if (fHotKey) delete[] fHotKey ;
} ;

bool FakeBar:: QuitRequested () {
	if (fSlave) fSlave->PostMessage (B_QUIT_REQUESTED) ;
	return BApplication::QuitRequested () ;
} ;

void FakeBar:: AboutRequested () {
	BAlert *a = new BAlert ("about FakeBar", "FakeBar 1.1.3\n"
			"helps your life be easier.\n\n"
			"Author: Tetsuo Yamada\n"
			"Contact: ivy@ma.kcom.ne.jp\n"
			"Support: http://www.duelists.org/~ivy/", 
			"Easy ?", NULL, NULL, 
			B_WIDTH_FROM_LABEL, B_INFO_ALERT) ;
	
	BFont font ;
	rgb_color color = {0, 0, 125} ;
	
	a->TextView()->MakeSelectable (true) ;
	a->TextView()->SetStylable (true) ;
	
	// enlarge "FakeBar"
	a->TextView()->GetFontAndColor (0, &font) ;
	font.SetSize (font.Size() + 2) ;
	a->TextView()->SetFontAndColor (0, 13, &font, B_FONT_ALL, &color) ;
	
	font.SetSize (font.Size() - 2) ;
	
	// colour 'b' blue, 'e' red
	a->TextView()->SetFontAndColor (30, 31, &font, B_FONT_ALL, &color) ;
	color.red = 120 ;
	color.blue = 0 ;
	a->TextView()->SetFontAndColor (31, 32, &font, B_FONT_ALL, &color) ;
	
	a->ResizeBy (40, 20) ;
	a->Go () ;
} ;

void FakeBar:: MessageReceived (BMessage *msg) {
	switch (msg->what) {
		case FB_LOAD_PREFS : 
			if (fb_manager) {
				fb_manager->Lock () ;
				fb_manager->Quit () ;
				BarWindow* window = new BarWindow () ;
				window->Show () ;
			}
			break ;
		case FB_OPEN : 
			Open (msg) ;
			break ;
		case FB_OPEN_PARENT : 
			OpenParent (msg) ;
			break ;
		case FB_OPEN_WITH : 
			OpenWith (msg) ;
			break ;
		case FB_SHOW_PREFS : 
			if (fPrefs) {
				fPrefs->Activate (true) ;
			} else {
				fPrefs = new PrefsWindow () ;
				fPrefs->Show () ;
			}
			break ;
		case FB_PREFS_CLOSED : 
			fPrefs = NULL ;
			break ;
		case FB_SHOW_DIALOG : 
			ShowDialog (msg) ;
			break ;
		default : 
			BApplication::MessageReceived (msg) ;
			break ;
	}
} ;

status_t create_ref_item (const char *path, LayerInfo *info, RefRetriever** item) {
	const LayerManager* manager = info->Manager () ;
	list_which which = (strcmp (info->LayerName(), B_EMPTY_STRING)==0) 
			? manager->TargetsWhich() : manager->LayersWhich () ;
	
	status_t err = B_OK ;
	const char *name = ref_name (path) ;
	
	switch (which) {
		case FB_LAYER_ITEM_LIST : 
			*item = new LayerItem (path, info, is_layer_file (path) && 
						manager->FindLayer (name)) ;
			return B_OK ;
		case FB_LIST_ITEM_LIST : 
			*item = new FileListItem (path) ;
			return B_OK ;
		case FB_MENU_ITEM_LIST : {
			entry_ref ref ;
			get_ref_for_path (path, &ref) ;
			BMessage msg (FB_OPEN) ;
			msg.AddRef ("file", &ref) ;
			msg.AddRef ("entries", &ref) ;
			FileMenuItem *fitem = DirectoryMenu::StartHierarchy (&ref, &msg) ;
			if (!fitem) fitem = new FileMenuItem (path, new BMessage (msg)) ;
			fitem->SetTarget (be_app) ;
			*item = fitem ;
			return B_OK ;
		}
	}
	return B_ERROR ;
} ;

void FakeBar:: Open (BMessage *msg) const {
	
	status_t err = B_OK ;
	BMessage invoke (B_REFS_RECEIVED) ;
	entry_ref file, refs ;
	int32 index = 0 ;
	bool launched = false ;
	BEntry entry ;
	void* text = NULL ;
	char** argv = NULL ;
	type_code type ;
	ssize_t numBytes = 0 ;
	char signature [B_MIME_TYPE_LENGTH + 1] ;
	bool refIsExe = false ;
	team_id team = -1 ;
	
	// get application's signature
	err = msg->FindRef ("file", &file) ;
	if (err!=B_OK) goto Failure0 ;
	
	err = get_signature (&file, signature, &refIsExe) ;
	if (err!=B_OK) {
		strcpy (signature, B_EMPTY_STRING) ;
		entry_ref app ;
		err = be_roster->FindApp (&file, &app) ;
		if (err!=B_OK) goto Failure1 ;
		
		file = app ;
	}
	
	// add refs to the message, if needed.
	while (msg->FindRef ("entries", index++, &refs)==B_OK) {
		err = entry.SetTo (&refs, false) ;
		if (err!=B_OK) continue ;
		
		err = entry.InitCheck () ;
		if (err!=B_OK) continue ;
		
		if (!entry.Exists()) continue ;
		
		if (refs!=file || !refIsExe || entry.IsDirectory()) {
			invoke.AddRef ("refs", &refs) ;
		}
	}
	
	if (invoke.CountNames (B_REF_TYPE)>0) {
		// launch with B_REFS_RECEIVED ;
		if (strcmp (signature, B_EMPTY_STRING)!=0) {
			err = be_roster->Launch (signature, &invoke, &team) ;
			if (team<0) alert (FB_LAUNCH_ALERT, signature, err) ;
		} else {
			err = be_roster->Launch (&file, &invoke, &team) ;
			if (team<0) alert (FB_LAUNCH_ALERT, file.name, err) ;
		}
		launched = true ;
	}
	invoke.MakeEmpty () ;
	
	// get argv if requested.
	err = msg->GetInfo ("text/plain", &type, &index) ;
	if (err==B_OK && index>0) {
		argv = new char* [index] ;
		for (int32 i=0 ; i<index ; i++) {
			err = msg->FindData ("text/plain", type, i, &text, &numBytes) ;
			if (err!=B_OK) {
				argv[i] = new char [2] ;
				strcpy (argv[i], B_EMPTY_STRING) ;
				continue ;
			}
			
			argv[i] = new char [numBytes + 1] ;
			strncpy (argv[i], (char*)text, numBytes) ;
			argv[i][numBytes] = '\0' ;
		}
		
		if (strcmp (signature, B_EMPTY_STRING)!=0) {
			err = be_roster->Launch (signature, index, argv, &team) ;
			if (team<0) alert (FB_LAUNCH_ALERT, signature, err) ;
		} else {
			err = be_roster->Launch (&file, index, argv, &team) ;
			if (team<0) alert (FB_LAUNCH_ALERT, file.name, err) ;
		}
		launched = true ;
		for (int32 i=0 ; i<index ; i++) delete[] argv[i] ;
		delete[] argv ;
		argv = NULL ;
	}
	
	if (!launched) {
		// launch without B_REFS_RECEIVED, nor B_ARGV_RECEIVED
		if (strcmp (signature, B_EMPTY_STRING)!=0) {
			err = be_roster->Launch (signature, (BMessage*)NULL, &team) ;
			if (team<0) alert (FB_LAUNCH_ALERT, signature, err) ;
		} else {
			err = be_roster->Launch (&file, (BMessage*)NULL, &team) ;
			if (team<0) alert (FB_LAUNCH_ALERT, file.name, err) ;
		}
	}
	
	return ;
	
Failure0 : 
	// do nothing... why?
	return ;
Failure1 : 
	alert (FB_LAUNCH_ALERT, file.name, err) ;
	return ;
} ;

void FakeBar:: OpenParent (BMessage *msg) const {
	
	status_t err = B_OK ;
	entry_ref file, pref ;
	BEntry entry, parent ;
	BMessage invoke (B_REFS_RECEIVED) ;
	
	err = msg->FindRef ("file", &file) ;
	if (err!=B_OK) goto Failure0 ;
	
	err = entry.SetTo (&file, false) ;
	if (err!=B_OK) goto Failure1 ;
	
	err = entry.InitCheck () ;
	if (err!=B_OK) goto Failure1 ;
	
	if (!entry.Exists()) {
		err = B_ENTRY_NOT_FOUND ;
		goto Failure1 ;
	}
	
	err = entry.GetParent (&parent) ;
	if (err!=B_OK) goto Failure1 ;
	
	err = parent.InitCheck () ;
	if (err!=B_OK) goto Failure1 ;
	
	err = parent.GetRef (&pref) ;
	if (err!=B_OK) goto Failure1 ;
	
	invoke.AddRef ("refs", &pref) ;
	
	err = be_roster->Launch ("application/x-vnd.Be-TRAK", &invoke) ;
	if (err!=B_OK && err!=B_ALREADY_RUNNING) goto Failure1 ;
	
	return ;
	
Failure0 : 
	// what can i do?
	return ;
Failure1 : 
	alert (FB_LAUNCH_ALERT, file.name, err) ;
	return ;
} ;

void FakeBar:: OpenWith (BMessage *msg) const {
	
	status_t err = B_OK ;
	const char *signature = NULL ;
	entry_ref file ;
	int32 i = 0, j = 0 ;
	
	while (msg->FindString ("applications", i++, &signature)==B_OK) {
		BMessage invoke (B_REFS_RECEIVED) ;
		j = 0 ;
		
		while (msg->FindRef ("file", j++, &file)==B_OK) {
			invoke.AddRef ("refs", &file) ;
		}
		
		if (j==0) {
			// no refs. so launch application with no message.
			err = be_roster->Launch (signature) ;
		} else {
			err = be_roster->Launch (signature, &invoke) ;
		}
		if (err!=B_OK && err!=B_ALREADY_RUNNING) goto Failure1 ;
	}
	
	return ;
	
Failure0 : 
	// what can i do ???
	return ;
Failure1 : 
	alert (FB_LAUNCH_ALERT, file.name, err) ;
	return ;
} ;

void FakeBar:: ShowDialog (BMessage *msg) {
	LayerInfo *info = NULL ;
	char* path = NULL ;
	status_t err = B_OK ;
	char layerPath [B_PATH_NAME_LENGTH + 1] ;
	entry_ref ref ;
	char signature [B_MIME_TYPE_LENGTH + 1] ;
	BAlert *a = NULL ;
	char text [512] ;
	
	err = msg->FindString ("path", &path) ;
	if (err!=B_OK) goto Fail ;
	
	err = msg->FindPointer ("layer", (void **)&info) ;
	if (err!=B_OK) goto Fail ;
	
	err = find_directory (B_USER_SETTINGS_DIRECTORY, dev_for_path ("/boot"), false, 
			layerPath, B_PATH_NAME_LENGTH) ;
	if (err!=B_OK) goto Fail ;
	
	strcat (layerPath, "/") ;
	strcat (layerPath, fb_app_name) ;
	strcat (layerPath, "/") ;
	strcat (layerPath, fb_layers_name) ;
	strcat (layerPath, "/") ;
	
	if (info && info->Looper() && info->Looper()->Lock()) {
		strcat (layerPath, info->LayerName()) ;
		info->Looper()->Unlock () ;
	} else {
		goto Fail ;
	}
	
	err = get_ref_for_path (layerPath, &ref) ;
	if (err!=B_OK) goto Fail ;
	
	err = get_signature (&ref, signature) ;
	if (err!=B_OK) goto Fail ;
	
	sprintf (text, "The item \"%s\" was not found.\nEdit the layer \"%s\"?", 
			path, ref_name (layerPath)) ;
	a = new BAlert ("FakeBar Alert", text, 
			"Edit from Prefs", "Edit from TextEditor", "Don't Edit", 
			B_WIDTH_FROM_WIDEST, B_INFO_ALERT) ;
	
	beep () ;
	switch (a->Go()) {
		case 0 : // open preferences.
			if (fPrefs) {
				fPrefs->Activate (true) ;
			} else {
				fPrefs = new PrefsWindow () ;
				fPrefs->Show () ;
			}
			break ;
		case 1 : { // open layer from text editor.
			BMessage message (B_REFS_RECEIVED) ;
			message.AddRef ("refs", &ref) ;
			be_roster->Launch (signature, &message) ;
			break ;
		}
		default : // do nothing.
			break ;
	}
	
	return ;
	
Fail : 
	beep () ;
	return ;
} ;

void FakeBar:: ConvertSettings () const {
	status_t err = B_OK ;
	BFile file ;
	BMessage archive, message, invoke ;
	char path [B_PATH_NAME_LENGTH + 1] ;
	char *str = NULL ;
	int32 i = 0, j = 0 ;
	settings s ;
	bool flag = false ;
	entry_ref ref ;
	BAlert *a = NULL ;
	BEntry entry ;
	
	err = find_directory (B_USER_SETTINGS_DIRECTORY, dev_for_path ("/boot"), 
			false, 	path, B_PATH_NAME_LENGTH) ;
	if (err!=B_OK) goto Fail ;
	
	strcat (path, "/FakeBar_settings") ;
	
	err = entry.SetTo (path, false) ;
	if (err!=B_OK) goto Fail ;
	
	err = entry.InitCheck () ;
	if (err!=B_OK) goto Fail ;
	
	err = file.SetTo (&entry, B_READ_ONLY) ;
	if (err!=B_OK) goto Fail ;
	
	err = file.InitCheck () ;
	if (err!=B_OK) goto Fail ;
	
	err = archive.Unflatten (&file) ;
	if (err!=B_OK) goto Fail ;
	
	err = archive.FindMessage ("targets_", &message) ;
	if (err!=B_OK) goto Next1 ;
	
	// get targets
	invoke.what = FB_SAVE_LAYER ;
	err = message.FindString ("list_name", &str) ;
	if (err!=B_OK) goto Next1 ;
	
	invoke.AddString ("name", B_EMPTY_STRING) ;
	j = 0 ;
	while (message.FindString ("entry_item", j++, &str)==B_OK) {
		invoke.AddString ("paths", str) ;
	}
	fb_slave->PostMessage (&invoke) ;
	invoke.MakeEmpty () ;
	
Next1 : 
	// get layers
	i = 0 ;
	while (archive.FindMessage ("layer_at_", i++, &message)==B_OK) {
		invoke.what = FB_SAVE_LAYER ;
		err = message.FindString ("list_name", &str) ;
		if (err!=B_OK) continue ;
		
		invoke.AddString ("name", str) ;
		j = 0 ;
		while (message.FindString ("entry_item", j++, &str)==B_OK) {
			invoke.AddString ("paths", str) ;
		}
		fb_slave->PostMessage (&invoke) ;
		invoke.MakeEmpty () ;
	}
	
	// get settings
	err = archive.FindMessage ("fake_bar", &message) ;
	if (err!=B_OK) goto Fail ;
	
	message.FindInt32 ("location", (int32*)&(s.location)) ;
	message.FindInt32 ("direction", (int32*)&(s.direction)) ;
	message.FindInt32 ("forInvoke", (int32*)&(s.forInvoke)) ;
	message.FindInt32 ("forExpand", (int32*)&(s.forExpand)) ;
	if (message.FindBool ("minimize", &flag)==B_OK) 
			s.set_flags (FB_MINIMIZE, flag) ;
	if (message.FindBool ("autofocus", &flag)==B_OK) 
			s.set_flags (FB_AUTOFOCUS, flag) ;
	if (message.FindBool ("drawLabel", &flag)==B_OK) 
			s.set_flags (FB_DRAW_LABEL, flag) ;
	message.FindInt32 ("which", (int32*)&(s.which)) ;
	if (message.FindInt32 ("ws", &i)==B_OK) {
		switch (i) {
			case 0 : s.workspaces = current_workspace() ; break ;
			case 1 : s.workspaces = B_ALL_WORKSPACES ; break ;
			case 2 : s.workspaces = B_ALL_WORKSPACES ; break ;
		}
	}
	j = 0 ;
	while (message.FindString ("layerName", j++, &str)==B_OK) {
		s.set_layer (str, j - 1) ;
	}
	
	// save settings
	err = get_settings (fb_settings_name, &ref) ;
	if (err!=B_OK) goto Fail ;
	
	err = file.SetTo (&ref, B_WRITE_ONLY|B_CREATE_FILE) ;
	if (err!=B_OK) goto Fail ;
	
	err = file.InitCheck () ;
	if (err!=B_OK) goto Fail ;
	
	invoke.MakeEmpty () ;
	err = s.archive_to (&invoke) ;
	if (err!=B_OK) goto Fail ;
	
	err = invoke.Flatten (&file) ;
	if (err!=B_OK) goto Fail ;
	
	a = new BAlert ("FakeBar Convert", "Converting a settings file is successful.\n"
			"Remove an old settings file?", "Remove", "Don't Remove", NULL, 
			B_WIDTH_FROM_WIDEST) ;
	switch (a->Go()) {
		case 0 : {
			BDirectory trash ;
			
			err = find_directory (B_TRASH_DIRECTORY, dev_for_path ("/boot"), 
					false, path, B_PATH_NAME_LENGTH) ;
			if (err==B_OK) err = trash.SetTo (path) ;
			if (err==B_OK) err = trash.InitCheck () ;
			if (err==B_OK) err = entry.MoveTo (&trash, NULL, true) ;
			if (err!=B_OK) 
				alert (-1, "Failed to Remove an old settings file...", err) ;
			break ;
		}
	}
	
	return ;

Fail : 
	alert (-1, "Failed to convert a settings file.", err) ;
	return ;
} ;