//	ClickControl.cpp
//
//	Tetsuo Yamada

#include	"ClickControl.h"


//===================================================================
//	class ClickControl
//===================================================================
ClickControl:: ClickControl (BRect frame, 
				const char *name, 
				const char *label, 
				float imageWidth, 
				float imageHeight, 
				BMessage *message, 
				uint32 mode, 
				uint32 flags)
		: BControl (frame, name, label, message, mode, flags) {
	fImageRect.Set (0, 2, imageWidth, imageHeight + 2) ;
	SetTracking (true) ;
} ;

ClickControl:: ~ClickControl () {
	/* do nothing */
} ;

void ClickControl:: MouseDown (BPoint p) {
	if (!fImageRect.Contains (p)) return ;
	
	int32 value = 0 ;
	
	//	do asynchronous mouse tracking
	if (Window()->Flags() & B_ASYNCHRONOUS_CONTROLS) {
		//	we can handle asynchronous
		SetMouseEventMask (B_POINTER_EVENTS) ;
		value = ValueForPoint (p, fImageRect) ;
		if (value!=Value()) {
			SetValue (value) ;
			Invoke () ;
		}
		
		return ;
	}
	
	//	old style synchronous mouse tracking
	uint32 buttons = 0 ;
	BPoint lastLoc ;
	GetMouse (&p, &buttons) ;
	
	while (buttons) {
		if (p!=lastLoc && fImageRect.Contains (p)) {
			value = ValueForPoint (p, fImageRect) ;
			if (value!=Value()) {
				SetValue (value) ;
				Invoke () ;
			}
		}
		
		snooze (20*1000) ;
		lastLoc = p ;
		GetMouse (&p, &buttons) ;
	}
} ;

void ClickControl:: MouseMoved (BPoint p, uint32 t, const BMessage *msg) {
	uint32 buttons = 0 ;
	int32 value = 0 ;
	GetMouse (&p, &buttons) ;
	if (!buttons) return ;
	if (!Bounds().Contains (p)) return ;
	
	value = ValueForPoint (p, fImageRect) ;
	if (value!=Value()) {
		SetValue (value) ;
		Invoke () ;
	}
} ;

void ClickControl:: KeyDown (const char *bytes, int32 numBytes) {
	int32 value = 0 ;
	switch (bytes[0]) {
		case B_TAB :
			BControl::KeyDown (bytes, numBytes) ;
			break ;
		default :
			value = ValueForKey (bytes, numBytes) ;
			if (value!=Value()) {
				SetValue (value) ;
				Invoke () ;
			}
			break ;
	}
} ;

void ClickControl:: AttachedToWindow () {
	BControl::AttachedToWindow () ;
	
	BRect rect = Bounds() ;
	BFont font ;
	GetFont (&font) ;
	float maxh = max_c (fImageRect.bottom + 2, font.Size() + 7) ;
	if (rect.Height()<maxh) ResizeTo (rect.Width(), maxh) ;
} ;

void ClickControl:: GetPreferredSize (float *width, float *height) {
	BFont font ;
	GetFont (&font) ;
	*width = fImageRect.right + 6 + font.StringWidth (Label()) ;
	*height = max_c (fImageRect.bottom + 2, font.Size() + 7) ;
} ;

void ClickControl:: SetImageSize (float width, float height) {
	fImageRect.Set (0, 2, width, height + 2) ;
	Invalidate () ;
} ;

void ClickControl:: GetImageSize (float *width, float *height) const {
	*width = fImageRect.Width() ;
	*height = fImageRect.Height() ;
} ;

void ClickControl:: Draw (BRect update) {
	rgb_color high = HighColor() ;
	rgb_color low = LowColor() ;
	BPoint penStart (fImageRect.right + 6, fImageRect.bottom - 3) ;
	float stringEnd = StringWidth (Label()) ;
	
	//	draw label
	DrawString (Label(), penStart) ;
	
	//	draw navigation line if needed.
	if (IsFocus()) {
		SetHighColor (255, 255, 255) ;
		StrokeLine (BPoint (penStart.x - 1, fImageRect.bottom + 1), 
				BPoint (penStart.x + stringEnd + 1, fImageRect.bottom + 1)) ;
		SetHighColor (keyboard_navigation_color()) ;
		StrokeLine (BPoint (penStart.x - 1, fImageRect.bottom), 
				BPoint (penStart.x + stringEnd + 1, fImageRect.bottom)) ;
		SetHighColor (high) ;
	} else if (IsFocusChanging()) {
		StrokeRect (BRect (penStart.x - 1, fImageRect.bottom, 
				penStart.x + stringEnd + 1, fImageRect.bottom + 1), B_SOLID_LOW) ;
		StrokeLine (BPoint (penStart.x - 1, fImageRect.bottom + 1), 
				BPoint (penStart.x + stringEnd + 1, fImageRect.bottom + 1), B_SOLID_LOW) ;
	}
	
	//	vertual hook calling 
	if (update.Intersects (fImageRect)) {
		DrawImage (fImageRect) ;
	}
	
	//	mmm...
	if (!IsEnabled()) {
		SetDrawingMode (B_OP_BLEND) ;
		SetHighColor (ViewColor()) ;
		FillRect (update) ;
		FillRect (update) ;
	}
	
	SetHighColor (high) ;
	SetLowColor (low) ;
	SetDrawingMode (B_OP_COPY) ;
} ;