//	FileMenuItem.h
//
//	Tetsuo Yamada

#include	"FileMenuItem.h"

 
//==================================================================
//	class FileMenuItem
//==================================================================
FileMenuItem:: FileMenuItem (const entry_ref *ref, BMessage *msg) 
		: RefRetriever (ref),  BMenuItem (ref->name, msg){
	fIcon = NULL ;
	fEnabled = true ;
	RefUpdated () ;
} ;

FileMenuItem:: FileMenuItem (const entry_ref *ref, BMenu *submenu, BMessage *msg) 
		: RefRetriever (ref), BMenuItem (submenu, msg) {
	fIcon = NULL ;
	fEnabled = true ;
	RefUpdated () ;
	SetLabel (RefName()) ;
} ;

FileMenuItem:: FileMenuItem (const char *path, BMessage *msg) 
		: RefRetriever (path), BMenuItem (B_EMPTY_STRING, msg) {
	fIcon = NULL ;
	fEnabled = true ;
	RefUpdated () ;
	SetLabel (RefName()) ;
} ;

FileMenuItem:: FileMenuItem (const char *path, BMenu *submenu, BMessage *msg) 
		: RefRetriever (path), BMenuItem (submenu, msg) {
	fIcon = NULL ;
	fEnabled = true ;
	RefUpdated () ;
	SetLabel (RefName()) ;
} ;

FileMenuItem:: FileMenuItem (const RefRetriever *ref, BMessage *msg) 
		: RefRetriever (*ref), BMenuItem (ref->RefName(), msg) {
	fIcon = NULL ;
	fEnabled = true ;
	RefUpdated () ;
} ;

FileMenuItem:: FileMenuItem (const RefRetriever *ref, BMenu *submenu, BMessage *msg) 
		: RefRetriever (*ref), BMenuItem (submenu, msg) {
	fIcon = NULL ;
	fEnabled = true ;
	RefUpdated () ;
	SetLabel (RefName()) ;
} ;

FileMenuItem:: FileMenuItem (BMessage *archive) 
		: RefRetriever (), BMenuItem (archive) {
	fIcon = NULL ;
	fEnabled = true ;
	status_t err = B_OK ;
	const char *path = NULL ;
	
	err = archive->FindString ("p:path", &path) ;
	if (err!=B_OK) goto Fail ;
	
	err = SetPath (path) ;
	if (err!=B_OK) goto Fail ;
	
	return ;
Fail :
	return ;
} ;

FileMenuItem:: ~FileMenuItem () {
	if (fIcon) delete fIcon ;
	fIcon = NULL ;
} ;

status_t FileMenuItem:: RefUpdated () {
	status_t err = B_OK ;
	const char *name = RefName() ;
	
	err = Status() ;
	if (err!=B_OK) goto Fail ;
	
	if (!fIcon) fIcon = new BBitmap (P_MINI_RECT, B_CMAP8) ;
	err = GetIcon (fIcon, B_MINI_ICON) ;
	if (err!=B_OK) goto Fail ;
	
	if (!fEnabled) {
		fEnabled = true ;
		if (Menu() && Menu()->Window() && Menu()->Window()->Lock()) {
			Menu()->Invalidate (Frame()) ;
			Menu()->Window()->Unlock () ;
		}
	}
	return B_OK ;
	
Fail :
	if (fIcon) {
		delete fIcon ;
		fIcon = NULL ;
	}
	if (fEnabled) {
		fEnabled = false ;
		if (Menu() && Menu()->Window() && Menu()->Window()->Lock()) {
			Menu()->Invalidate (Frame()) ;
			Menu()->Window()->Unlock () ;
		}
	}
	
	return err ;
} ;

const BBitmap* FileMenuItem:: Icon () const {
	return fIcon ;
} ;

bool FileMenuItem:: IsSelected2 () const {
	return IsSelected () ;
} ;

bool FileMenuItem:: IsEnabled2 () const {
	return (fEnabled && IsEnabled()) ;
} ;

void FileMenuItem:: GetContentSize (float *width, float *height) {
	BMenuItem::GetContentSize (width, height) ;
	*width += 20.0 ; // allocate icon space
	if (*height<18) *height = 18.0 ;
} ;

void FileMenuItem:: DrawContent () {
	BMenu *menu = Menu () ;
	BPoint penLoc = menu->PenLocation () ;
	rgb_color high = menu->HighColor() ;
	
	if (fIcon) {
		menu->SetDrawingMode (B_OP_OVER) ;
		menu->DrawBitmap (fIcon, penLoc) ;
		menu->SetDrawingMode (B_OP_COPY) ;
	}
	
	if (!IsEnabled2() && IsEnabled()) 
		menu->SetHighColor (tint_color (high, B_LIGHTEN_1_TINT)) ;
	
	menu->MovePenTo (penLoc.x + 20, penLoc.y + 3) ;
	BMenuItem::DrawContent () ;
	
	menu->SetHighColor (high) ;
} ;

status_t FileMenuItem:: Invoke (BMessage *msg) {
	BMessage invoke ;
	BHandler *target = Target() ;
	BLooper *looper = (target) ? target->Looper() : NULL ;
	
	if (!IsEnabled2()) return B_OK ;
	if (msg) invoke = *msg ;
	else if (Message()) invoke = *(Message()) ;
	else return B_ERROR ;
	
	if (!looper) return BMenuItem::Invoke (msg) ;
	
	invoke.AddInt64 ("when", system_time()) ;
	invoke.AddPointer ("source", this) ;
	
	return looper->PostMessage (&invoke, target) ;
} ;

FileMenuItem* FileMenuItem:: Instantiate (BMessage *archive) {
	if (!validate_instantiation (archive, "FileMenuItem")) return NULL ;
	char *path = NULL ;
	if (archive->FindString ("p:path", &path)!=B_OK) return NULL ;
	return new FileMenuItem (archive) ;
} ;

status_t FileMenuItem:: Archive (BMessage *msg, bool deep) const {
	status_t err = B_OK ;
	
	err = BMenuItem::Archive (msg, deep) ;
	if (err!=B_OK) return err ;
	
	err = msg->AddString ("p:path", Path()) ;
	if (err!=B_OK) return err ;
	
	return B_OK ;
} ;