//	MatrixListItem.cpp
//
//	Tetsuo Yamada

#include	<time.h>
#include	<string.h>
#include	"MatrixListItem.h"
#include	"MatrixListView.h"

 
//===================================================================
//	class MatrixListItem
//===================================================================
MatrixListItem:: MatrixListItem () 
		: BListItem () {
	/* do nothing */
} ;

MatrixListItem:: ~MatrixListItem () {
	/* do nothing */
} ;

void MatrixListItem:: DrawItem (BView *owner, BRect itemRect, bool complete) {
	MatrixListView *view = cast_as (owner, MatrixListView) ;
	BRegion r ;
	BRegion itemClip ;
	ListColumn *column = NULL ;
	BRect columnRect ;
	int32 columnCount = view->CountColumns () ;
	float gap = view->Gap () ;
	rgb_color high = view->HighColor () ;
	rgb_color low = view->LowColor () ;
	
	if (complete && !IsSelected()) {
		view->FillRect (itemRect, B_SOLID_LOW) ;
	} else if (IsSelected()) {
		view->SetLowColor (tint_color (low, B_DARKEN_1_TINT)) ;
		view->FillRect (itemRect, B_SOLID_LOW) ;
	}
	
	view->GetClippingRegion (&r) ;
	for (int32 i=0 ; i<columnCount ; i++) {
		column = view->ColumnAt (i) ;
		columnRect = column->fFrame ;
		columnRect.InsetBy (gap, 0) ;
		columnRect = (columnRect & itemRect) ;
		if (r.Intersects (columnRect)) {
			view->SetHighColor (high) ;
			view->SetLowColor (low) ;
			itemClip.Set (columnRect) ;
			view->ConstrainClippingRegion (&itemClip) ;
			DrawColumn (view, column, columnRect) ;
		}
	}
	view->SetHighColor (high) ;
	view->SetLowColor (low) ;
	view->ConstrainClippingRegion (NULL) ;
} ;

float MatrixListItem:: MinimumColumnWidth (const ListColumn *column) const {
	return 0 ;
} ;

status_t MatrixListItem:: MessageForDrag (BMessage *msg) const {
	return B_OK ;
} ;

//===================================================================
//	class FileListItem
//===================================================================
FileListItem:: FileListItem (const entry_ref *ref) 
		: MatrixListItem (), RefRetriever (ref) {
	fIcon = NULL ;
	RefUpdated () ;
} ;

FileListItem:: FileListItem (const char *path) 
		: MatrixListItem (), RefRetriever (path) {
	fIcon = NULL ;
	RefUpdated () ;
} ;

FileListItem:: ~FileListItem () {
	delete fIcon ;
} ;

status_t FileListItem:: RefUpdated () {
	status_t err = B_OK ;
	BEntry entry ;
	entry_ref ref ;
	
	delete fIcon ;
	fIcon = NULL ;
	
	err = GetEntryRef (&ref) ;
	if (err!=B_OK) return err ;
	
	// get stat fStat
	err = entry.SetTo (&ref) ;
	if (err!=B_OK) return err ;
	
	err = entry.InitCheck () ;
	if (err!=B_OK) return err ;
	
	err = entry.GetStat (&fStat) ;
	if (err!=B_OK) return err ;
	
	// get bitmap fIcon
	err = entry.SetTo (&ref, true) ;
	if (err!=B_OK) return err ;
	
	err = entry.InitCheck () ;
	if (err!=B_OK) return err ;
	
	err = entry.GetRef (&ref) ;
	if (err!=B_OK) return err ;
	
	fIcon = new BBitmap (BRect (0, 0, 15, 15), B_CMAP8) ;
	err = GetIcon (fIcon, B_MINI_ICON) ;
	if (err!=B_OK) return err ;
	
	return B_OK ;
} ;

void FileListItem:: DrawColumn (MatrixListView *owner, 
				const ListColumn *column, 
				BRect columnRect) {
	
	char string [64] ;
	BPoint pen (columnRect.left, columnRect.bottom - 4) ;
	if (Status()!=B_OK) owner->SetHighColor (tint_color (owner->HighColor(), B_LIGHTEN_1_TINT)) ;
	switch (column->command) {
		case P_MLV_FILE_NAME :
			if (fIcon) {
				BPoint ip (pen.x, columnRect.top + (int)((columnRect.Height() - 16)/2)) ;
				owner->SetDrawingMode (B_OP_OVER) ;
				owner->DrawBitmap (fIcon, ip) ;
				owner->SetDrawingMode (B_OP_COPY) ;
			}
			pen.x += 20 ;
			owner->DrawString (RefName(), pen) ;
			break ;
		case P_MLV_FILE_PATH :
			owner->DrawString (Path(), pen) ;
			break ;
		case P_MLV_FILE_SIZE :
			if (S_ISDIR (fStat.st_mode)) {
//				owner->SetHighColor (selected_menu_gray) ;
				owner->DrawString ("folder", pen) ;
			} else if (S_ISLNK (fStat.st_mode)) {
//				owner->SetHighColor (selected_menu_gray) ;
				owner->DrawString ("symlink", pen) ;
			} else {
				if (fStat.st_blksize<1024) {
					sprintf (string, "%Ld bytes", fStat.st_blksize) ;
				} else if (fStat.st_blksize<1048576) {
					sprintf (string, "%Ld K", fStat.st_size/1024) ;
				} else {
					sprintf (string, "%Ld M", fStat.st_size/1048576) ;
				}
				owner->DrawString (string, pen) ;
			}
			break ;
		case P_MLV_LAST_MODIFIED :
			strncpy (string, ctime(&(fStat.st_mtime)), sizeof (string)) ;
			string[sizeof (string) - 1] = '\0' ;
			string[strlen (string) - 1] = '\0' ;
			owner->DrawString (string, pen) ;
			break ;
		case P_MLV_CREATED :
			strncpy (string, ctime(&(fStat.st_ctime)), sizeof (string)) ;
			string[sizeof (string) - 1] = '\0' ;
			string[strlen (string) - 1] = '\0' ;
			owner->DrawString (string, pen) ;
			break ;
	}
} ;

status_t FileListItem:: MessageForDrag (BMessage *msg) const {
	status_t err = B_OK ;
	
	err = msg->AddString ("paths", Path()) ;
	if (err!=B_OK) return err ;
	
	return B_OK ;
} ;

float FileListItem:: MinimumColumnWidth (const ListColumn *column) const {
	return 100 ;
} ;

void FileListItem:: Update (BView *owner, const BFont *font) {
	BListItem::Update (owner, font) ;
	if (Height()<18) SetHeight (18) ;
} ;