//	MatrixListItem.h
//
//	Tetsuo Yamada
//
//	MatrixListItem is an abstract ListItem for MatrixListView...
//	And its subclass FileListItem, which represents a file.


#ifndef		P_MATRIX_LIST_ITEM_h_
#define		P_MATRIX_LIST_ITEM_h_


#include	"GUIBase.h"
#include	"fs/RefRetriever.h"
#include	<interface/Bitmap.h>
#include	<interface/ListItem.h>
#include	<interface/Region.h>

class		MatrixListView ;
class		ListColumn ;


//===================================================================
//	class MatrixListItem
//===================================================================
//	the abstract class for the item of the MatrixListView
class MatrixListItem : public BListItem {
public :
							MatrixListItem () ;
	virtual					~MatrixListItem () ;
	
	virtual void			DrawItem (BView *owner, 
									BRect itemRect, 
									bool drawEverything = false) ;
	virtual void			DrawColumn (MatrixListView *owner, 
									const ListColumn *column, 
									BRect columnRect) = 0 ;
	virtual float			MinimumColumnWidth (const ListColumn *column) const ;
	virtual status_t		MessageForDrag (BMessage *msg) const ;
} ;

//===================================================================
//	class FileListItem
//===================================================================
#define		P_MLV_FILE_NAME		'PMfn'
#define		P_MLV_FILE_PATH		'PMfp'
#define		P_MLV_FILE_SIZE		'PMfs'
#define		P_MLV_LAST_MODIFIED	'PMmd'
#define		P_MLV_CREATED		'PMcr'

class FileListItem : public MatrixListItem, public RefRetriever {

/* variables */
private :
	BBitmap*				fIcon ;
	struct stat				fStat ;
	
/* methods */
public :
							FileListItem (const entry_ref *ref) ;
							FileListItem (const char *path) ;
	virtual					~FileListItem () ;
	
	virtual void			Update (BView *owner, const BFont *font) ;
	virtual	void			DrawColumn (MatrixListView *owner, 
									const ListColumn *column, 
									BRect columnRect) ;
	virtual float			MinimumColumnWidth (const ListColumn *column) const ;
	virtual status_t		MessageForDrag (BMessage *msg) const ;
	
protected :
	virtual status_t		RefUpdated () ;
} ;

#endif

