//	MatrixListView.h
//
//	Tetsuo Yamada
//
//	MatrixListView is ListView which lists lists items in both column and row.
//	This class invokes messages whenever its contents was changed.

#ifndef		P_MATRIX_LIST_VIEW_h_
#define		P_MATRIX_LIST_VIEW_h_

#include	"GUIBase.h"
#include	"MatrixListItem.h"
#include	<interface/ListView.h>
#include	<interface/ScrollBar.h>

class		MatrixListView ;


//===================================================================
//	class ListColumn
//===================================================================
class ListColumn : public BArchivable {
friend class				MatrixListView ;
friend class				MatrixListItem ;

/* variables */
public :
	uint32					command ;
	
private :
	char*					fName ;
	MatrixListView*			fOwner ;
	BRect					fFrame ;
	alignment				fAlign ;
//+	float					fMinimum ;
//+	bool					fMovable ;
//+	bool					fResizable ;
	
/* constructor & destructor */
public :
							ListColumn (uint32 command, 
									const char *name, 
									float width = 100) ;
	virtual					~ListColumn () ;
	
/* methods */
public :
	virtual float			PreferredWidth () const ;
	virtual void			ResizeToPreferred () ;
	virtual void			SetWidth (float width) ;
	
	BRect					Frame () const ;
	float					Width () const ;
	
	const char*				Name () const ;
	void					SetName (const char *name) ;
	
	alignment				Align () const ;
	void					SetAlign (alignment align) ;
} ;

//===================================================================
//	class MatrixListView
//===================================================================
/* inform message constants */
#define		P_MLV_ITEM_ADDED		1
#define		P_MLV_ITEM_REMOVED		2
#define		P_MLV_ITEM_MOVED		4
#define		P_MLV_COLUMN_ADDED		8
#define		P_MLV_COLUMN_REMOVED	16
#define		P_MLV_COLUMN_MOVED		32
#define		P_MLV_COLUMN_RESIZED	64
#define		P_MLV_MSG_DROPPED		128

#define		P_MLV_ITEM_CHANGED		7
#define		P_MLV_COLUMN_CHANGED	56


class MatrixListView : public BListView {

/* variables */
private :
	BList					fColumns ;
	float					fGap ;
	int32					fUnderlined ;
	int32					fFlags ;
	
/* constructor & destructor */
public :
							MatrixListView (BRect frame, 
									const char *name, 
									uint32 informFlags = P_MLV_ITEM_CHANGED|P_MLV_COLUMN_CHANGED, 
									list_view_type type = B_SINGLE_SELECTION_LIST, 
									uint32 mode = B_FOLLOW_LEFT|B_FOLLOW_TOP, 
									uint32 flags = B_WILL_DRAW|B_NAVIGABLE|B_FRAME_EVENTS) ;
	virtual					~MatrixListView () ;
	
/* re-written hooks */
public :
	virtual void			AttachedToWindow () ;
	virtual void			FrameResized (float width, float height) ;
	virtual void			GetPreferredSize (float *width, float *height) ;
	virtual void			MouseMoved (BPoint p, uint32 t, const BMessage *msg) ;
	virtual void			MessageReceived (BMessage *msg) ;
	virtual void			ScrollTo (BPoint p) ;
	void					ScrollTo (float x, float y) ;
	
	virtual bool			InitiateDrag (BPoint p, int32 index, bool selected) ;
	
	virtual bool			AddItem (BListItem *item, int32 index) ;
	virtual bool			AddItem (BListItem *item) ;
	virtual BListItem*		RemoveItem (int32 index) ;
	virtual bool			RemoveItem (BListItem *item) ;
	
/* newly defined methods */
public :
	bool					AddColumn (ListColumn *column, int32 index = -1) ;
	bool					RemoveColumn (ListColumn *column) ;
	ListColumn*				RemoveColumn (int32 index) ;
	int32					CountColumns () const ;
	ListColumn*				ColumnAt (int32 index) const ;
	ListColumn*				ColumnAt (BPoint p) const ;
	int32					ColumnIndexOf (ListColumn *column) const ;
	bool					MoveColumn (int32 from, int32 to) ;
	
	virtual void			SetInformFlags (uint32 flags) ;
	uint32					InformFlags () const ;
	
	virtual void			SetGap (float gap) ;
	float					Gap () const ;
	
private :
	void					UpdateUnderline (int32 index) ;
	void					HandleItemDropping (BMessage *msg) ;
	void					SetScrollBars () ;
} ;


inline void MatrixListView::ScrollTo (float x, float y) {
	MatrixListView::ScrollTo (BPoint (x, y)) ;
} ;

#endif

