//	OpenWithMenu.cpp
//
//	Tetsuo Yamada

#include	"OpenWithMenu.h"


//===========================================================
//	class OpenWithMenu
//===========================================================
OpenWithMenu:: OpenWithMenu (const entry_ref *ref, BMessage *msg, menu_layout layout) 
				: BMenu (ref->name), BInvoker (msg, NULL, NULL) {
	fRef = *ref ;
	fInfo.MakeEmpty () ;
	
	if (msg) {
		// add dummy data.
		msg->AddString ("applications", B_EMPTY_STRING) ;
		msg->AddRef ("refs", &fRef) ;
	}
} ;

OpenWithMenu:: ~OpenWithMenu () {
	// do nothing
} ;

status_t OpenWithMenu:: SetEntryRef (const entry_ref *ref) {
	status_t err = B_OK ;
	if (Window()) return B_INTERRUPTED ;
	if (fInfo.HasData ("supporting_apps", B_OBJECT_TYPE)) {
		fRef = *ref ;
		fInfo.MakeEmpty () ;
		RemoveItems (0, CountItems(), true) ;
		
		err = find_supporting_apps (&fRef, &fInfo) ;
		if (err!=B_OK) {
			BMenuItem *item = new BMenuItem ("Internal Error Occurred", (BMessage*)NULL) ;
			item->SetEnabled (false) ;
			AddItem (item) ;
		}
	} else {
		fRef = *ref ;
	}
	return B_OK ;
} ;

const entry_ref* OpenWithMenu:: EntryRef () const {
	return &fRef ;
} ;

bool OpenWithMenu:: AddDynamicItem (add_state state) {
	status_t err = B_OK ;
	BMenuItem *item = NULL ;
	int32 count = CountItems() ;
	
	switch (state) {
		case B_INITIAL_ADD : 
			RemoveItems (0, count, true) ;
			fInfo.MakeEmpty () ;
			
			err = find_supporting_apps (&fRef, &fInfo) ;
			if (err!=B_OK) {
				fInfo.MakeEmpty () ;
				item = new BMenuItem ("Internal Error Occurred", (BMessage*)NULL) ;
				item->SetEnabled (false) ;
				AddItem (item) ;
				return false ;
			}
			
			return true ;
			
		case B_PROCESSING : {
			int32 index = 0 ;
			support_info *info = NULL, *prev = NULL ;
			ssize_t num ;
			BMessage *invoke = NULL ;
			
			while (fInfo.FindData ("supporting_apps", B_OBJECT_TYPE, index++, 
					(void **)&info, &num)==B_OK) {
				
				if (prev && info->state!=prev->state) AddSeparatorItem () ;
				
				if (Message()) {
					invoke = new BMessage (Message()) ;
					invoke->ReplaceString ("applications", info->signature) ;
					invoke->ReplaceRef ("refs", &(info->ref)) ;
					item = new FileMenuItem (&(info->ref), invoke) ;
				} else {
					item = new FileMenuItem (&(info->ref), (BMessage*)NULL) ;
				}
				AddItem (item) ;
				item->SetTarget (Target()) ;
				
				prev = info ;
			}
			
			if (!prev) {
				// No supporting apps... really ???
				item = new BMenuItem ("No Apps", (BMessage*)NULL) ;
				item->SetEnabled (false) ;
				AddItem (item) ;
			}
			
			fInfo.MakeEmpty () ;
			return false ;
		}
		
		case B_ABORT : 
			fInfo.MakeEmpty () ;
			RemoveItems (0, count, true) ;
			return false ;
	}
	
	return false ;
} ;

status_t OpenWithMenu:: Invoke (BMessage *msg) {
	return B_OK ;
} ;

status_t OpenWithMenu:: SetMessage (BMessage *message) {
	if (message) {
		// add dummy data.
		message->AddString ("applications", B_EMPTY_STRING) ;
		message->AddRef ("refs", &fRef) ;
	}
	return BInvoker::SetMessage (message) ;
} ;

void OpenWithMenu:: AttachedToWindow () {
	SetTarget (Superitem()->Target()) ;
	BMenu::AttachedToWindow () ;
} ;