//	TextDialog.cpp
//
//	Tetsuo Yamada

#include	"TextDialog.h"

#define		P_TEXT_DIALOG_MSG	'pTDM'


filter_result text_dialog_filter (BMessage *msg, BHandler **target, 
									BMessageFilter *filter) {
	// filters B_KEY_DOWN only
	if (msg->what!=B_KEY_DOWN) return B_DISPATCH_MESSAGE ;
	
	const char *bytes = NULL ;
	uint32 modifiers = 0  ; // not used...
	TextDialog *owner = cast_as (filter->Looper(), TextDialog) ;
	
	// can filter TextDialog's messages
	if (!owner) return B_DISPATCH_MESSAGE ;
	
	msg->FindInt32 ("modifiers", (int32*)&modifiers) ;
	msg->FindString ("bytes", &bytes) ;
	
	switch (bytes[0]) {
		case B_ENTER : // invoke
			*target = owner->fOk ;
			break ;
		case B_ESCAPE : // cancel
			owner->fCancel->SetValue (1) ;
			owner->PostMessage (B_QUIT_REQUESTED) ;
			return B_SKIP_MESSAGE ;
	}
	
	return B_DISPATCH_MESSAGE ;
} ;

TextDialog:: TextDialog (const char *label, const char *oldText, 
				confirm_func func, bool allowsSame, void* extraData) 
			: BWindow (BRect (0, 0, 240, 80), "Text Dialog", 
				B_MODAL_WINDOW_LOOK, B_MODAL_APP_WINDOW_FEEL, 
				B_NOT_RESIZABLE|B_NOT_CLOSABLE|B_NOT_ZOOMABLE|B_NOT_MINIMIZABLE) {
	
	fOldText = new char [strlen (oldText) + 1] ;
	strcpy (fOldText, oldText) ;
	fAllowsSame = allowsSame ;
	fNewTextBuf = NULL ;
	fConfirm = func ;
	fResult = NULL ;
	fExtraData = extraData ;
	
	BRect rect = BuildViews (label) ;
	ResizeTo (rect.Width(), rect.Height()) ;
	fOk->SetResizingMode (B_FOLLOW_RIGHT|B_FOLLOW_BOTTOM) ;
	fCancel->SetResizingMode (B_FOLLOW_RIGHT|B_FOLLOW_BOTTOM) ;
	
	fInput->MakeFocus (true) ;
	fInput->TextView()->SelectAll () ;
	
	BMessageFilter *filter = new BMessageFilter (B_KEY_DOWN, text_dialog_filter) ;
	AddCommonFilter (filter) ;
} ;

TextDialog:: ~TextDialog () {
	delete[] fOldText ;
	fOldText = NULL ;
} ;

void TextDialog:: Show () {
	BRect rect = Bounds() ;
	BRect frame = BScreen (this).Frame() ;
	
	MoveTo ((frame.Width() - rect.Width())*0.5, 
		(frame.Height() - rect.Height())*0.33) ;
	
	BWindow::Show () ;
} ;

void TextDialog:: MessageReceived (BMessage *msg) {
	switch (msg->what) {
		case P_TEXT_DIALOG_MSG : 
			if (strcmp (fOldText, fInput->Text())!=0) {
				// a new text was set.
				if (fConfirm (fInput->Text(), fExtraData)) {
					strcpy (fNewTextBuf, fInput->Text()) ;
					*fResult = true ;
					PostMessage (B_QUIT_REQUESTED) ;
				} else {
					goto ShowAlert ;
				}
			} else if (fAllowsSame) {
				// a same text is allowed.
				*fResult = true ;
				strcpy (fNewTextBuf, fInput->Text()) ;
				PostMessage (B_QUIT_REQUESTED) ;
			} else {
				// a same text is not allowed.
				goto ShowAlert ;
			}
			break ;
		default : 
			BWindow::MessageReceived (msg) ;
			break ;
	}
	
	return ;
	
ShowAlert : 
	char *text = new char [strlen (fInput->Text()) + 60] ;
	sprintf (text, "That name (\"%s\") is not permitted.\nPlease input another one.", 
			fInput->Text()) ;
	
	BAlert *a = new BAlert ("Text Dialog: Alert", text, "OK") ;
	a->Go () ;
	delete[] text ;
	
	fInput->SetText (fOldText) ;
	fInput->MakeFocus (true) ;
	fInput->TextView()->SelectAll () ;
	fOk->SetValue (0) ;
} ;

BRect TextDialog:: BuildViews (const char *label) {
	BRect bounds = Bounds() ;
	BRect rect ;
	float width = 0, height = 0 ;
	float bw = 60 ; // button width
	
	// bottom view
	BView *bottom = new BView (bounds, "bottom", B_FOLLOW_ALL, B_WILL_DRAW) ;
	bottom->SetViewColor (235, 235, 235) ;
	AddChild (bottom) ;
	
	// BStringView showing label
	bounds.InsetBy (5, 5) ;
	rect = bounds ;
	rect.bottom = rect.top ;
	BStringView *strView = new BStringView (rect, "label_view", label, 
			B_FOLLOW_TOP|B_FOLLOW_LEFT_RIGHT) ;
	strView->SetFont (be_bold_font) ;
	bottom->AddChild (strView) ;
	height = be_bold_font->Size() + 2 ;
	strView->ResizeTo (rect.Width(), height) ;
	
	// TextControl to input
	rect.OffsetBy (0, height + 5) ;
	fInput = new BTextControl (rect, "input", B_EMPTY_STRING, fOldText, 
			new BMessage (P_TEXT_DIALOG_MSG), B_FOLLOW_LEFT_RIGHT|B_FOLLOW_TOP) ;
	fInput->SetDivider (0) ;
	bottom->AddChild (fInput) ;
	fInput->SetTarget (this) ;
	fInput->GetPreferredSize (&width, &height) ;
	fInput->ResizeTo (rect.Width(), height) ;
	
	// cancel button
	rect.OffsetBy (0, height + 5) ;
	rect.right -= bw + 5 ;
	rect.left = rect.right - bw ;
	fCancel = new BButton (rect, "cancel", "Cancel", 
			new BMessage (B_QUIT_REQUESTED), B_FOLLOW_NONE) ;
	bottom->AddChild (fCancel) ;
	fCancel->SetTarget (this) ;
	fCancel->GetPreferredSize (&width, &height) ;
	fCancel->ResizeTo (bw, height) ;
	
	// ok button
	rect.OffsetBy (bw + 5, 0) ;
	fOk = new BButton (rect, "ok", "OK", new BMessage (P_TEXT_DIALOG_MSG), 
			B_FOLLOW_NONE) ;
	bottom->AddChild (fOk) ;
	fOk->SetTarget (this) ;
	fOk->ResizeTo (rect.Width(), height) ;
	
	bounds.right = rect.right ;
	bounds.bottom = rect.top + height ;
	bounds.InsetBy (-5, -5) ;
	return bounds ;
} ;

BStringView* TextDialog:: StringView () const {
	return cast_as (FindView ("label_view"), BStringView) ;
} ;

BTextView* TextDialog:: TextView () const {
	return fInput->TextView() ;
} ;

BButton* TextDialog:: CancelButton () const {
	return fCancel ;
} ;

BButton* TextDialog:: OKButton () const {
	return fOk ;
} ;

bool TextDialog:: Go (char *newTextBuf) {
	bool result = false ;
	BLooper *looper = BLooper::LooperForThread (find_thread (NULL)) ;
	BWindow *window = (looper) ? cast_as (looper, BWindow) : NULL ;
	
	fNewTextBuf = newTextBuf ;
	fResult = &result ;
	Show () ;
	
	while (fOldText) {
		if (window) window->UpdateIfNeeded () ;
		snooze (20*1000) ;
	}
	
	return result ;
} ;