//	TextDialog.h
//
//	Tetsuo Yamada
//
//	The modal dialog to input a text (such as name).

#ifndef		P_TEXT_DIALOG_h_
#define		P_TEXT_DIALOG_h_


#include	"GUIBase.h"
#include	<interface/Alert.h>
#include	<interface/Window.h>
#include	<interface/TextControl.h>
#include	<interface/StringView.h>
#include	<interface/Button.h>
#include	<app/MessageFilter.h>


//	return true if the newly inputed text is legal.
typedef bool				(*confirm_func) (const char *newText, 
									void* extraData) ;

//	filter_hook
filter_result				text_dialog_filter (BMessage *msg, 
									BHandler **target, 
									BMessageFilter *msgFilter) ;

//==============================================================
//	class TextDialog
//==============================================================
class TextDialog : public BWindow {
//	friends
friend filter_result		text_dialog_filter (BMessage*, BHandler**, 
									BMessageFilter*) ;
	
//	fields
private : 
	char*					fOldText ;
	bool					fAllowsSame ;
	char*					fNewTextBuf ;
	confirm_func			fConfirm ;
	bool*					fResult ;
	void*					fExtraData ;
	
	//	children
	BTextControl*			fInput ;
	BButton*				fOk ;
	BButton*				fCancel ;
	
//	methods
public : 
							TextDialog (const char *label, 
									const char *oldText, 
									confirm_func func, 
									bool allowsSameText = true, 
									void* extraData = NULL) ;
	virtual					~TextDialog () ;
	
	virtual void			MessageReceived (BMessage *msg) ;
	
	//	call this method to display TextDialog.
	//	this method returns true if a new text is set, and fills newTextBuf with it.
	//	Otherwise returns false (without filling newTextBuf).
	bool					Go (char *newTextBuf) ;
	
	BStringView*			StringView () const ;
	BTextView*				TextView () const ;
	BButton*				CancelButton () const ;
	BButton*				OKButton () const ;
	
protected : 
	virtual void			Show () ;
	
private : 
	BRect					BuildViews (const char *label) ;
} ;

#endif

