//	HotKeyWindow.cpp
//
//	Tetsuo Yamada

#include	"HotKeyWindow.h"


//==============================================================
//	function key_filter
//==============================================================
filter_result key_filter (BMessage *message, BHandler **target, BMessageFilter *filter) {
	uint32 modifiers = 0 ;
	message->FindInt32 ("modifiers", (int32*)&modifiers) ;
	message->ReplaceInt32 ("modifiers", 0) ;
	message->AddInt32 ("modifiers2", modifiers) ;
	return B_DISPATCH_MESSAGE ;
}

//==============================================================
//	class KeyView
//==============================================================
KeyView:: KeyView (BRect frame, uint8 **states) 
			: BView (frame, "key_view", B_FOLLOW_NONE, B_WILL_DRAW|B_NAVIGABLE) {
	
	fLabel = new char [strlen ("Don't Use") + 1] ;
	strcpy (fLabel, "Don't Use") ;
	fStates = states ;
} ;

KeyView:: ~KeyView () {
	delete[] fLabel ;
} ;

void KeyView:: KeyDown (const char *bytes, int32 numBytes) {
	BMessage *msg = Window()->CurrentMessage() ;
	uint8 *states = NULL ;
	status_t err = B_OK ;
	ssize_t num = 0 ;
	uint32 modifiers = 0 ;
	char label [128] ;
	
	err = msg->FindData ("states", B_UINT8_TYPE, (void **)&states, &num) ;
	if (err!=B_OK) goto Fail ;
	
	err = msg->FindInt32 ("modifiers2", (int32*)&modifiers) ;
	if (err!=B_OK) goto Fail ;
	
	if (!(*fStates)) {
		*fStates = new uint8 [16] ;
	}
	for (int32 i=0 ; i<16 ; i++) (*fStates)[i] = states[i] ;
	
	strcpy (label, B_EMPTY_STRING) ;
	if (modifiers & B_SHIFT_KEY) Connect (label, "Shift") ;
	if (modifiers & B_COMMAND_KEY) Connect (label, "Command") ;
	if (modifiers & B_CONTROL_KEY) Connect (label, "Control") ;
	if (modifiers & B_OPTION_KEY) Connect (label, "Option") ;
	Connect (label, "'") ;
	strcat (label, bytes) ;
	strcat (label, "'") ;
	
	delete[] fLabel ;
	fLabel = new char [strlen (label) + 1] ;
	strcpy (fLabel, label) ;
	
	Invalidate () ;
	return ;
	
Fail : 
	beep () ;
} ;

void KeyView:: AttachedToWindow () {
	BView::AttachedToWindow () ;
	
	SetFont (be_plain_font) ;
	SetViewColor (255, 255, 255) ;
} ;

void KeyView:: Draw (BRect) {
	BRect bounds = Bounds() ;
	BPoint p ;
	
	stroke_3d_rect (this, bounds, ui_color2 (P_STANDARD_GRAY), false) ;
	bounds.InsetBy (1, 1) ;
	
	p.x = (bounds.Width() - StringWidth (fLabel))/2 ;
	p.y = 3*bounds.Height()/4 ;
	DrawString (fLabel, p) ;
} ;

//==============================================================
//	class HotKeyWindow
//==============================================================
HotKeyWindow::HotKeyWindow (uint8 **states, bool *endFlag) 
			: BWindow (BRect (0, 0, 240, 78), "Press Hot Key", 
			B_MODAL_WINDOW_LOOK, B_MODAL_APP_WINDOW_FEEL, 
			B_NOT_RESIZABLE|B_NOT_CLOSABLE|B_NOT_ZOOMABLE|B_NOT_MINIMIZABLE) {
	fEndFlag = endFlag ;
	
	BRect bounds (0, 0, 240, 78) ;
	BRect rect ;
	BRect frame = BScreen (this).Frame() ;
	
	MoveTo ((frame.Width() - bounds.Width())/2, (frame.Height() - bounds.Height())/3) ;
	
	BView *bottom = new BView (bounds, "bottom", B_FOLLOW_ALL, B_WILL_DRAW) ;
	bottom->SetViewColor (ui_color2 (P_STANDARD_GRAY)) ;
	AddChild (bottom) ;
	
	rect = bounds ;
	rect.InsetBy (SMALL_SPACE, SMALL_SPACE) ;
	rect.bottom = rect.top + 14 ;
	BStringView *msgView = new BStringView (rect, "msg_view", "Press Hot Key !!") ;
	msgView->SetAlignment (B_ALIGN_CENTER) ;
	bottom->AddChild (msgView) ;
	
	rect.OffsetBy (0, 14 + SMALL_SPACE) ;
	rect.bottom = rect.top + 20 ;
	fKeyView = new KeyView (rect, states) ;
	bottom->AddChild (fKeyView) ;
	
	rect.left = rect.right - SMALL_SPACE - 2*60 ;
	rect.top = rect.bottom + SMALL_SPACE ;
	rect.bottom = rect.top + 20 ;
	rect.right = rect.left + 60 ;
	fUnset = new BButton (rect, "unset", "Unset", new BMessage (B_QUIT_REQUESTED)) ;
	bottom->AddChild (fUnset) ;
	
	rect.OffsetBy (60 + SMALL_SPACE, 0) ;
	fSet = new BButton (rect, "set", "Set", new BMessage (B_QUIT_REQUESTED)) ;
	bottom->AddChild (fSet) ;
	
	AddCommonFilter (new BMessageFilter (B_ANY_DELIVERY, B_ANY_SOURCE, B_KEY_DOWN, 
			key_filter)) ;
	fKeyView->MakeFocus (true) ;
} ;

HotKeyWindow:: ~HotKeyWindow () {
	*fEndFlag = true ;
} ;

bool HotKeyWindow::QuitRequested () {
	BButton *invoker = NULL ;
	BNode node ;
	
	get_settings_node (&node) ;
	
	CurrentMessage()->FindPointer ("source", (void **)&invoker) ;
	if (invoker==fUnset || strcmp (fKeyView->fLabel, "Don't Use")==0) {
		delete[] *(fKeyView->fStates) ;
		*(fKeyView->fStates) = NULL ;
		node.RemoveAttr ("hotKey") ;
		printf ("Your Hot Key was Cleared...\n") ;
	} else {
		node.WriteAttr ("hotKey", B_UINT8_TYPE, 0, *(fKeyView->fStates), 16) ;
		printf ("Your Hot Key was Set...\n") ;
	}
	
	return true ;
} ;