//	IOSlave.cpp
//
//	Tetsuo Yamada

#include	"IOSlave.h"


//==============================================================
//	class IOSlave
//==============================================================
IOSlave:: IOSlave () 
			: BLooper ("io_slave", 10, 50) {
	
	status_t err = B_OK ;
	entry_ref ref ;
	
	err = get_settings (fb_layers_name, &ref) ;
	if (err==B_OK) fDir.SetTo (&ref) ;
} ;

IOSlave:: ~IOSlave () {
	// do nothing
} ;

void IOSlave:: MessageReceived (BMessage *msg) {
	
	switch (msg->what) {
		case FB_SAVE_LAYER : 
			SaveLayer (msg) ;
			break ;
		case FB_LOAD_LAYER : 
			LoadLayer (msg) ;
			break ;
		case FB_STOP_LOADING : 
			// do nothing. go to line 129 and see CheckQueue().
			break ;
		case FB_DELETE_LAYER : 
			DeleteLayer (msg) ;
			break ;
		default :
			BLooper::MessageReceived (msg) ;
			break ;
	}
} ;

void IOSlave:: LoadLayer (BMessage *msg) {
	
	status_t err = B_OK ;
	RefRetriever* item = NULL ;
	LayerInfo *info = NULL ;
	BLooper *looper = NULL ;
	int32 start = 0 ;
	int32 blocksize = 0 ;
	BFile file ;
	char type [B_MIME_TYPE_LENGTH + 1] ;
	char *buf = NULL ;
	char *ptr = NULL ;
	off_t size = 0 ;
	entry_ref ref ;
	int32 index = 0 ;
	BMessage message (FB_ITEMS_FOUND) ;
	
	msg->FindPointer ("layer", (void **)&info) ;
	msg->FindPointer ("looper", (void **)&looper) ;
	msg->FindInt32 ("blocksize", &blocksize) ;
	msg->FindInt32 ("start", &start) ;
	if (info) {
		blocksize = max_c (blocksize, 1) ;
		start = max_c (start, 0) ;
	} else {
		return ;
	}
	
	// lock
	if (looper->Lock()) {
		if (looper->IndexOf (info)<0) {
			looper->Unlock () ;
			goto Failure1 ;
		}
	} else {
		goto Failure1 ;
	}
	
	if (strcmp (info->LayerName(), B_EMPTY_STRING)==0) {
		// layer to be read is targets
		err = get_settings (fb_targets_name, &ref) ;
		if (err==B_OK) err = file.SetTo (&ref, B_READ_ONLY) ;
	} else {
		// layer to be read is ordinary one.
		BEntry entry ;
		
		err = fDir.FindEntry (info->LayerName(), &entry, true) ;
		if (err==B_OK) err = file.SetTo (&entry, B_READ_ONLY) ;
	}
	
	if (err==B_OK) err = file.InitCheck () ;
	if (err!=B_OK) goto Error3 ;
	
	err = file.Lock () ;
	if (err!=B_OK) goto Error3 ;
	
	// is its filetype all right?
	err = file.ReadAttr ("BEOS:TYPE", B_MIME_TYPE, 0, type, B_MIME_TYPE_LENGTH) ;
	if (err<B_OK) goto Error2 ;
	
	if (strcmp (type, "text/plain")!=0) {
		err = B_BAD_VALUE ;
		goto Error2 ;
	}
	
	err = file.GetSize (&size) ;
	if (err!=B_OK) goto Error2 ;
	
	buf = new char [size + 1] ;
	
	err = file.Read (buf, size) ;
	if (err<B_OK) goto Error1 ;
	
	file.Unlock () ;
	
	buf[size] = '\0' ;
	buf[err] = '\0' ;
	
	// read line loooooooop
	ptr = strtok (buf, "\n") ;
	while (ptr) {
		if (!CheckQueue(info)) {
			looper->Unlock () ;
			delete[] buf ;
			return ;
		}
		
		if (ptr[0]=='#') {
			// comment
			ptr = strtok (NULL, "\n") ;
			continue ;
		}
		
		if (start>index) {
			// skip
			ptr = strtok (NULL, "\n") ;
			index++ ;
			continue ;
		}
		
		err = create_ref_item (ptr, info, &item) ;
		if (err!=B_OK) {
			// failed to create item ;
			ptr = strtok (NULL, "\n") ;
			item = NULL ;
			continue ;
		}
		
		message.AddPointer ("items", item) ;
		message.AddInt32 ("index", index) ;
		index++ ;
		
		if (index%blocksize==blocksize - 1) ProcessMessage (&message, info) ;
		message.what = FB_ITEMS_FOUND ;
		item = NULL ;
		ptr = strtok (NULL, "\n") ;
		
		// unlock -> rest -> lock
		info->Looper()->Unlock() ;
		snooze (20*1000) ;
		if (looper->Lock()) {
			if (looper->IndexOf (info)<0) {
				looper->Unlock () ;
				goto Failure2 ;
			}
		} else {
			goto Failure2 ;
		}
	}
	
	message.what = FB_ITEMS_FOUND ;
	message.AddBool ("finish", true) ;
	ProcessMessage (&message, info) ;
	
	delete[] buf ;
	looper->Unlock() ;
	return ;
	
Error1 : 
	delete[] buf ;
Error2 : 
	file.Unlock () ;
Error3 : 
	if (strcmp (info->LayerName(), B_EMPTY_STRING)==0) {
		alert (FB_READ_ALERT, fb_targets_name, err) ;
	} else {
		alert (FB_READ_ALERT, info->LayerName(), err) ;
	}
	looper->Unlock() ;
	
	return ;

Failure2 : 
	delete[] buf ;
	file.Unlock () ;
	ProcessMessage (&message, NULL) ;
Failure1 : 
	return ;
} ;

void IOSlave:: ProcessMessage (BMessage *message, LayerInfo *target) {
	
//	status_t err = B_OK ;
	RefRetriever *item = NULL ;
	int32 index = 0 ;
	
	if (	!target || !(target->Looper()) || 
			target->Looper()->PostMessage (message, target)!=B_OK	) {
		while (message->FindPointer ("items", index++, (void **)&item)==B_OK) delete item ;
	}
	message->RemoveName ("items") ;
} ;

bool IOSlave:: CheckQueue (LayerInfo *info) {
	
	BMessageQueue *q = MessageQueue() ;
	BMessage *msg = NULL ;
	int32 count = q->CountMessages() ;
	LayerInfo *alt = NULL ;
	
	for (int32 i=0 ; i<count ; i++) {
		msg = q->FindMessage (i) ;
		if (msg->what!=FB_STOP_LOADING) continue ;
		
		if (msg->FindPointer ("layer", (void **)&alt)==B_OK) {
			if (alt==info) {
				q->RemoveMessage (msg) ;
				return false ;
			}
		}
	}
	return true ;
} ;

void IOSlave:: SaveLayer (BMessage *msg) {
	status_t err = B_OK ;
	BFile file ;
	entry_ref ref ;
	int32 index = 0 ;
	const char* path = NULL ;
	const char* name = NULL ;
	off_t written = 0 ;
	bool icon = false ;
	
	msg->FindBool ("icon", &icon) ;
	
	err = msg->FindString ("name", &name) ;
	if (err!=B_OK) goto Error2 ;
	
	if (strcmp (name, B_EMPTY_STRING)==0) {
		// layer to be written is targets
		err = get_settings (fb_targets_name, &ref) ;
		if (err==B_OK) err = file.SetTo (&ref, B_WRITE_ONLY|B_CREATE_FILE) ;
	} else {
		// layer to be written is ordinary one.
		BEntry entry ;
		
		err = fDir.FindEntry (name, &entry, true) ;
		if (err==B_OK) err = file.SetTo (&entry, B_WRITE_ONLY|B_CREATE_FILE) ;
		if (err==B_ENTRY_NOT_FOUND) {
			err = fDir.CreateFile (name, &file) ;
			write_layer_attr (&file, icon) ;
		}
	}
	
	if (err!=B_OK) goto Error2 ;
	
	err = file.InitCheck () ;
	if (err!=B_OK) goto Error2 ;
	
	err = file.Lock () ;
	if (err!=B_OK) goto Error2 ;
	
	while (msg->FindString ("paths", index++, &path)==B_OK) {
		err = file.WriteAt (written, path, strlen (path)) ;
		if (err<B_OK) goto Error1 ;
		
		written += err ;
		
		err = file.WriteAt (written, "\n", 1) ;
		if (err<B_OK) goto Error1 ;
		
		written += err ;
	}
	
	file.SetSize (written) ;
	file.Unlock () ;
	
	return ;
	
Error1 : 
	file.Unlock () ;
Error2 : 
	if (!name) return ;
	if (strcmp (name, B_EMPTY_STRING)==0) {
		alert (FB_WRITE_ALERT, fb_targets_name, err) ;
	} else {
		alert (FB_WRITE_ALERT, name, err) ;
	}
	return ;
} ;

void IOSlave:: DeleteLayer (BMessage *msg) {
	
	status_t err = B_OK ;
	BEntry entry ;
	int32 number = 1 ;
	char path [B_PATH_NAME_LENGTH + 1] ;
	BDirectory trash ;
	char* name = NULL ;
	
	err = msg->FindString ("name", &name) ;
	if (err!=B_OK) goto Failure0 ;
	
	err = fDir.FindEntry (name, &entry, false) ;
	if (err!=B_OK) goto Failure0 ;
	
	err = find_directory (B_TRASH_DIRECTORY, dev_for_path ("/boot"), false, 
			path, B_PATH_NAME_LENGTH) ;
	if (err!=B_OK) goto Failure1 ;
	
	err = trash.SetTo (path) ;
	if (err!=B_OK) goto Failure1 ;
	
	err = trash.InitCheck () ;
	if (err!=B_OK) goto Failure1 ;
	
	err = entry.MoveTo (&trash) ;
	while (err==B_FILE_EXISTS) {
		// rename to move
		sprintf (path, "%s %d", name, number++) ;
		err = entry.MoveTo (&trash, path, false) ;
	}
	if (err!=B_OK) goto Failure1 ;
	
	return ;
	
Failure0 : 
	// no way to solve...
	return ;
	
Failure1 : 
	entry.Remove () ;
} ;