//	LayerBar.cpp
//
//	Tetsuo Yamada

#include	"LayerBar.h"
#include	"BarWindow.h"


extern const settings* fb_settings ;
extern LayerManager* fb_manager ;


//==============================================================
//	class LayerBar
//==============================================================
LayerBar:: LayerBar () 
		: BMenuBar (BRect (0, 0, 1, 1), "layer_bar", B_FOLLOW_LEFT|B_FOLLOW_TOP, 
				(IS_COLUMN) ? B_ITEMS_IN_COLUMN : B_ITEMS_IN_ROW, false) {
	fCurrent = FB_NO_LIST ;
	fLayer = NULL ;
	fSelection = -1 ;
	fTimeForTrack = 0 ;
	
	SetTriggersEnabled (false) ;
	SetRadioMode (false) ;
	SetLabelFromMarked (false) ;
	SetFont (be_plain_font) ;
} ;

LayerBar:: ~LayerBar () {
	RemoveItems (0, CountItems(), false) ;
} ;

void LayerBar:: AttachedToWindow () {
	if (fCurrent==FB_NO_LIST && fLayer) AttachLayer () ;
	ResizeToPreferred () ;
	BMenuBar::AttachedToWindow () ;
} ;

void LayerBar:: AllDetached () {
	if (fCurrent==FB_LAYER_ITEM_LIST && fLayer) DetachLayer () ;
	BMenuBar::AllDetached () ;
} ;

void LayerBar:: MouseDown (BPoint p) {
	
	BWindow *window = Window () ;
	BMessage *msg = NULL ;
	uint32 modifiers = 0, buttons = 0, clicks = 0 ;
	mouse_operation mop = FB_NO_USE ; // mouse operation 
	fSelection = IndexAt (p) ;
	
	if (!(window->IsActive())) window->Activate (true) ;
	
	msg = window->CurrentMessage () ;
	msg->FindInt32 ("modifiers", (int32*)&modifiers) ;
	msg->FindInt32 ("buttons", (int32*)&buttons) ;
	msg->FindInt32 ("clicks", (int32*)&clicks) ;
	
	if (clicks >= 2) {
		//	double click
		mop = FB_DOUBLE_CLICK ;
	} else if (buttons==B_SECONDARY_MOUSE_BUTTON || (modifiers & B_CONTROL_KEY)) {
		//	secondary mouse button down
		mop = FB_SECONDARY_DOWN ;
	} else if (buttons==B_PRIMARY_MOUSE_BUTTON) {
		//	primary mouse button down
		mop = FB_PRIMARY_DOWN ;
	}
	if (mop==fb_settings->forInvoke) {
		if (fSelection!=-1) {
			LayerItem *item = LayerItemAt (fSelection) ;
			if (item) LaunchLayerItem (item) ;
		}
	} else if (mop==fb_settings->forExpand) {
		BMenuBar::MouseDown (p) ;
	}
} ;

int32 LayerBar:: IndexAt (BPoint p) const {
	if (!(Bounds().Contains (p))) return -1 ;
	int32 index = -1 ;
	float hor, ver, width, height ;
	fb_settings->get_item_margins (&hor, &ver) ;
	fb_settings->get_item_size (&width, &height) ;
	
	if (IS_COLUMN) {
		if (p.x<0 || p.x>width + 2*hor) return -1 ;
		index = (int)(p.y/(2*ver + height + 1)) ;
	} else {
		if (p.y<0 || p.y>height + 2*ver) return -1 ;
		index = (int)(p.x/(2*hor + width + 1)) ;
	}
	if (index>=CountItems() || index<0) return -1 ;
	return index ;
} ;

LayerItem* LayerBar:: LayerItemAt (int32 index) const {
	BMenuItem *item = ItemAt (index) ;
	return (item) ? cast_as (item, LayerItem) : NULL ;
} ;

LayerInfo* LayerBar:: Layer () const {
	return fLayer ;
} ;

void LayerBar:: SetLayer (LayerInfo *info) {
	if (info && info->Manager()->LayersWhich()!=FB_LAYER_ITEM_LIST) return ;
	if (info && strcmp (info->LayerName(), B_EMPTY_STRING)==0) info = NULL ;
	
	if (fLayer && fCurrent==FB_LAYER_ITEM_LIST) DetachLayer () ;
	fLayer = info ;
	if (fLayer && Window()) AttachLayer () ;
} ;

void LayerBar:: AttachLayer () {
	// already filled
	if (fCurrent!=FB_NO_LIST) return ;
	// there's no layer to attach to
	if (!fLayer) return ;
	
	LayerItem *item = NULL ;
	int32 count = 0 ;
	BList *items = fLayer->SetLock (true, this) ;
	
	// layer is not available
	if (!items) return ;
	
	count = items->CountItems () ;
	for (int32 i=0 ; i<count ; i++) {
		item = cast_as ((RefRetriever*)(items->ItemAt (i)), LayerItem) ;
		item->CheckNode (true) ; // this line slows up layer switching:-( but...
		AddItem (item, (IS_POSITIVE) ? i : 0) ;
	}
	
	fCurrent = FB_LAYER_ITEM_LIST ;
	fSelection = -1 ;
} ;

void LayerBar:: DetachLayer () {
	RemoveItems (0, CountItems(), false) ;
	
	if (fLayer) fLayer->SetLock (false, this) ;
	fCurrent = FB_NO_LIST ;
	fSelection = -1 ;
} ;

void LayerBar:: LaunchLayerItem (LayerItem *item) {
	Window()->UpdateIfNeeded () ;
	BRegion r ;
	BRect frame = item->Frame() ;
	
	for (int32 i=1 ; i<9 ; i++) {
		item->fSelected = i%2 ;
		switch (item->fSelected) {
			case 1 :
				item->Draw () ;
				break ;
			default :
				FillRect (frame, B_SOLID_LOW) ;
				item->Draw () ;
		}
		Flush () ;
		snooze (50*1000) ;
	}
	item->Invoke () ;
	Invalidate (frame) ;
} ;

void LayerBar:: RescanItems () {
	// there's no layer to rescan
	if (!fLayer) return ;
	
	int32 iCount = CountItems() ;
	int32 oCount = 0 ;
	LayerItem *item = NULL ;
	BList* items = fLayer->SetLock (true, this) ;
	
	// layer is not available
	if (!items) return ;
	
	fCurrent = FB_LAYER_ITEM_LIST ;
	oCount = items->CountItems () ;
	
	// no item to add
	if (iCount>=oCount) return ;
	
	for (int32 i=iCount ; i<oCount ; i++) {
		item = cast_as ((RefRetriever*)(items->ItemAt (i)), LayerItem) ;
		AddItem (item, (IS_POSITIVE) ? i : 0) ;
	}
} ;

void LayerBar:: ResizeToPreferred () {
	int32 count = CountItems () ;
	float hor, ver, width, height ;
	fb_settings->get_item_margins (&hor, &ver) ;
	fb_settings->get_item_size (&width, &height) ;
	
	SetItemMargins (hor, ver, hor, ver) ;
	SetMaxContentWidth (width) ;
	switch (Layout()) {
		case B_ITEMS_IN_COLUMN :
			ResizeTo (width + 2*hor, count*(height + 2*ver + 1) - 1) ;
			break ;
		case B_ITEMS_IN_ROW :
			ResizeTo (count*(width + 2*hor + 1) - 1, height + 2*ver) ;
			break ;
	}
} ;

void LayerBar:: Draw (BRect update) {
	
	FillRect (update, B_SOLID_LOW) ;
	int32 count = CountItems () ;
	LayerItem *item = NULL ;
	rgb_color high = HighColor () ;
	rgb_color low = LowColor() ;
	
	for (int32 i=count - 1 ; i>=0 ; i--) {
		item = LayerItemAt (i) ;
		if (update.Intersects (item->Frame())) {
			SetHighColor (high) ;
			SetLowColor (low) ;
			item->Draw () ;
		}
	}
	
	if (!(fb_settings->flags & FB_DRAW_BORDER)) {
		stroke_3d_rect (this, Bounds(), low, true) ;
	}
} ;

void LayerBar:: KeyDown (const char *bytes, int32 numBytes) {
	
	//	because of BMenu's bug, we must care item selection.
	switch (bytes[0]) {
		case B_ESCAPE : // unselect item...
			fSelection = -1 ;
			break ;
		case B_ENTER : 
		case B_SPACE : 
		case B_UP_ARROW : 
		case B_DOWN_ARROW : 
		case B_LEFT_ARROW : 
		case B_RIGHT_ARROW : 
			for (int32 i=CountItems() - 1 ; i>=0 ; i--) {
				if (LayerItemAt (i)->IsSelected2()) {
					fSelection = i ;
					break ;
				}
			}
			break ;
	}
	
	BMenuBar::KeyDown (bytes, numBytes) ;
} ;

int32 LayerBar:: Selection () const {
	return fSelection ;
} ;

void LayerBar:: MouseMoved (BPoint p, uint32 t, const BMessage *msg) {
	if ((fb_settings->flags & FB_AUTOFOCUS) && !(Window()->IsActive())) {
		BPoint cp ;
		uint32 buttons ;
		GetMouse (&cp, &buttons, false) ;
		if (Bounds().Contains (cp)) Window()->Activate (true) ;
		return ;
	}
	
	/* this block is NOT USED because FB_DROP_EXTENTION is not implemented
	if (msg && (fb_settings->flags & FB_DROP_EXTENTION)) {
		if (Window()->IsActive()) {
			DropTrack (p, t) ;
		} else {
			Window()->Activate (true) ;
		}
		return ;
	}*/
	
	if (msg) {
		int32 index = -1 ;
		LayerItem *item = NULL ;
		BRect rect ;
		
		index = IndexAt (p) ;
		if (t==B_EXITED_VIEW) index = -1 ;
		if (index!=fSelection) {
			// unselect item
			item = LayerItemAt (fSelection) ;
			if (item) {
				item->fSelected = false ;
				rect = item->Frame() ;
				FillRect (rect, B_SOLID_LOW) ;
				item->Draw () ;
				Flush () ;
			}
			// select item
			fSelection = index ;
			item = LayerItemAt (fSelection) ;
			if (item) {
				item->fSelected = true ;
				item->Draw () ;
				Flush () ;
			}
		}
	}
} ;

void LayerBar:: MessageReceived (BMessage *msg) {
	if (msg->WasDropped()) {
		BRect rect ;
		BPoint p = msg->DropPoint () ;
		ConvertFromScreen (&p) ;
		LayerItem *item = LayerItemAt (IndexAt (p)) ;
		if (item) item->MessageDropped (msg) ;
		
		int32 count = CountItems () ;
		for (int32 i=0 ; i<count ; i++) {
			item = LayerItemAt (i) ;
			if (item->fSelected) {
				item->fSelected = false ;
				rect = item->Frame() ;
				FillRect (rect, B_SOLID_LOW) ;
				item->Draw () ;
			}
		}
		fSelection = -1 ;
		Flush () ;
	} else {
		BMenuBar::MessageReceived (msg) ;
	}
} ;

/*void LayerBar:: DropTrack (BPoint p, uint32 t) {
	int32 index = -1 ;
	LayerItem *item = NULL ;
	LayerMenu *menu = NULL ;
	uint32 buttons = 0 ;
	BRect rect ;
	BMessage *msg = Window()->CurrentMessage() ;
	int32 anchor = 0 ;
	
	if (t==B_EXITED_VIEW) {
		// mouse is now exited...
		item = LayerItemAt (fSelection) ;
		if (item) menu = cast_as (item->Submenu(), LayerMenu) ;
		
		if (menu && menu->Window()) {
			// menu has been displayed.
			if (menu->Window()->Lock()) {
				rect = menu->Bounds() ;
				menu->ConvertToScreen (&rect) ;
				menu->Window()->Unlock () ;
				
				while (system_time() - fTimeForTrack < OB_INTERVAL && buttons) {
					ConvertToScreen (&p) ;
					if (rect.Contains (p)) {
						// mouse enters submenu.
						return ;
					}
					
					GetMouse (&p, &buttons, false) ;
				}
			}
		}
				
		// i wait long enough or failed to lock submenu's Window() 
		// or no submenu's Window()
		anchor = 1 ;
		goto UnselectItem ;
Anchor1 : 
		fSelection = -1 ;
		fTimeForTrack = system_time() ;
		return ;
	}
	
	// mouse is moving within this menu.
	index = IndexAt (p) ;
	
	// item selection is changed.
	if (fSelection!=index) {
		if (system_time() - fTimeForTrack > SELECT_INTERVAL) {
			item = LayerItemAt (fSelection) ;
			if (item) menu = cast_as (item->Submenu(), LayerMenu) ;
			anchor = 2 ;
			goto UnselectItem ;
Anchor2 : 
			fSelection = index ;
			item = LayerItemAt (fSelection) ;
			if (item) menu = cast_as (item->Submenu(), LayerMenu) ;
			anchor = 3 ;
			goto SelectItem ;
Anchor3 : 
			fTimeForTrack = system_time () ;
		}
		return ;
	}
	
	// same item is selected.
	item = LayerItemAt (fSelection) ;
	if (item) menu = cast_as (item->Submenu(), LayerMenu) ;
	if (menu && menu->Window()) {
		fTimeForTrack = system_time() ;
		return ;
	}
	
	if (menu && menu->ExpandLayer()) {
		if (system_time() - fTimeForTrack > SHOW_INTERVAL) {
			anchor = 4 ;
			goto ShowSubmenu ;
Anchor4 : 
			fTimeForTrack = system_time () ;
			return ;
		}
	}
	
	return ;
	
//---------------------------
UnselectItem : 
	if (menu && menu->Window()) menu->Hide() ;
	if (item) {
		item->fSelected = false ;
		rect = item->Frame() ;
		rect.InsetBy (1, 1) ;
		FillRect (rect, B_SOLID_LOW) ;
		item->Draw () ;
		Flush () ;
	}
	goto GoBack ;
	
SelectItem : 
	if (item) {
		item->fSelected = true ;
		rect = item->Frame() ;
		rect.InsetBy (1, 1) ;
		item->Draw () ;
		Flush () ;
	}
	goto GoBack ;
	
ShowSubmenu : 
	if (menu && menu->ExpandLayer() && !(menu->Window())) {
		menu->Show () ;
		BMessage msg ;
		menu->Window()->Archive (&msg, false) ;
		msg.PrintToStream () ;
	}
	goto GoBack ;
	
GoBack : 
	switch (anchor) {
		case 1 : goto Anchor1 ;
		case 2 : goto Anchor2 ;
		case 3 : goto Anchor3 ;
		case 4 : goto Anchor4 ;
	}
} ;*/