//	LayerInfo.h
//
//	Tetsuo Yamada
//
//	LayerInfo does not represent layer's infomation but layer itself.
//	LayerManager is a window which manages LayerInfo.

#ifndef		FB_LAYER_INFO_h_
#define		FB_LAYER_INFO_h_

#include	"FBBase.h"
#include	"fs/RefRetriever.h"
#include	<app/Handler.h>
#include	<interface/Window.h>
#include	<support/List.h>


class		LayerInfo ;
class		LayerManager ;
class		IOSlave ;


typedef enum list_which {
	FB_NO_LIST = -1, 
	FB_MENU_ITEM_LIST, 
	FB_LAYER_ITEM_LIST, 
	FB_LIST_ITEM_LIST, 
	FB_REF_RETRIEVER_LIST
} ;

//==============================================================
//	globals for layer
//==============================================================
//	returns true, if a passed file is a layer file (includes targets)
bool						is_layer_file (const entry_ref *ref) ;
bool						is_layer_file (const char *path) ;

void						write_layer_attr (BFile *file, bool icon = false) ;

//==============================================================
//	class LayerInfo
//==============================================================
//	this is one of the most important class in FakeBar.
//	this class (and only this class) can handle layers.
class LayerInfo : public BHandler {
// friends
friend class				LayerManager ;

// fields
public : 
	int32					fData ; // extra field. LayerInfo won't touch this field.
	
private :
	char*					fName ; // layer name
	void*					fLocker ; // who is using thie layer?
	bool					fCompleted ; // all items are completely added ?
	bool					fWithFile ; // this layer has a targeting file ?
	BList					fItems ;
	
// methods
public : 
							LayerInfo (const char *layerName, 
									bool withFile = true) ;
							~LayerInfo () ;
	
	virtual void			MessageReceived (BMessage *msg) ;
	
	// ask.
	const char*				LayerName () const ;
	bool					IsCompleted () const ;
	int32					CountItems () const ;
	void*					Locker () const ;
	LayerManager*			Manager () const ; // do inline
	bool					IsEnabled () const ; // do inline
	
	//	returns 0 if this object holds items which is same as info.
	int32					CompareItems (const LayerInfo *info) const ;
	
	//	returns NULL if locking failed.
	BList*					SetLock (bool lock, void* locker) ;
	
	void					SetLayerName (const char *name) ;
	
	void					Initialize () ;
	
	void					ItemsFound (BMessage *msg) ;
	
	void					CheckAllItems (bool deep) ;
	
	void					PrintToStream () const ;
	
	void					SetEnabled (bool enabled) ;
} ;

//==============================================================
//	class LayerManager
//==============================================================
class LayerManager : public BWindow {
// friends
friend class				LayerInfo ;

// fields
private : 
	
protected : 
	settings				fSettings ;
	list_which				fLayersWhich ;
	list_which				fTargetsWhich ;
	
// methods
public : 
	virtual void			Show () ;
	virtual void			Quit () ;
	virtual void			MessageReceived (BMessage *msg) ;
	
	// newly defined hooks
	virtual void			UpdateSettings (settings *s) ;
	virtual void			SetLayerEnabled (bool enabled) ;
	virtual status_t		LoadLayers () ;
	
	status_t				GetLayerList (BMessage *container) const ;
	status_t				GetLayerPointer (BMessage *container) const ;
	
	LayerInfo*				AddLayer (const char* name, bool withFile = true) ;
	LayerInfo*				RemoveLayer (const char *name) ;
	
	LayerInfo*				FindLayer (const char *name) const ;
	LayerInfo*				FindNextLayer (LayerInfo* start = NULL, 
									int32 increment = 1) const ;
	
	list_which				LayersWhich () const ;
	list_which				TargetsWhich () const ;
	
protected : 
							LayerManager (BRect frame, 
									const char *title, 
									uint32 flags, 
									list_which layersWhich, 
									list_which targetsWhich) ;
	virtual					~LayerManager () ;
	
	void					UpdateSettings (BMessage *msg = NULL, 
									bool callVirtual = false) ;
} ;

//==============================================================
//	LayerInfo inline implementations
//==============================================================
inline LayerManager* LayerInfo:: Manager () const {
	return cast_as (Looper(), LayerManager) ;
} ;

inline bool LayerInfo:: IsEnabled () const {
	return (Locker()!=&fData) ;
} ;

#endif

