//	LayerItem.cpp
//
//	Tetsuo Yamada

#include	"LayerItem.h"
#include	"LayerBar.h"


extern const settings* fb_settings ;
extern LayerManager* fb_manager ;

//==============================================================
//	class LayerItem
//==============================================================
LayerItem:: LayerItem (const char *path, LayerInfo *owner, bool expandLayer) 
		: BMenuItem (new LayerMenu (expandLayer)), RefRetriever (path) {
	fSelected = false ;
	fIcon = NULL ;
	fEnabled = true ;
	fOwner = owner ;
	RefUpdated () ;
	SetLabel (RefName()) ;
	SetTarget (be_app) ;
} ;

LayerItem:: ~LayerItem () {
	if (fIcon) delete fIcon ;
} ;

status_t LayerItem:: RefUpdated () {
	status_t err = Status() ;
	
	BEntry entry ;
	entry_ref ref ;
	LayerMenu* submenu = cast_as (Submenu(), LayerMenu) ;
	BMessage *msg = NULL ;
	const char *name = RefName() ;
	BRect iconRect ;
	
	err = Status() ;
	if (err!=B_OK) goto Fail ;
	
	err = entry.SetTo (Path(), true) ;
	if (err!=B_OK) goto Fail ;
	
	err = entry.InitCheck () ;
	if (err!=B_OK) goto Fail ;
	
	if (!entry.Exists()) {
		err = B_ENTRY_NOT_FOUND ;
		goto Fail ;
	}
	
	iconRect = (fb_settings->which==B_LARGE_ICON) ? P_LARGE_RECT : P_MINI_RECT ;
	if (fIcon && iconRect!=fIcon->Bounds()) {
		delete fIcon ;
		fIcon = NULL ;
	}
	
	if (!fIcon) fIcon = new BBitmap (iconRect, B_CMAP8) ;
	err = GetIcon (fIcon, fb_settings->which) ;
	if (err<B_OK) {
		delete fIcon ;
		fIcon = NULL ;
	}
	
	err = GetEntryRef (&ref) ;
	if (err!=B_OK) goto Fail ;
	
	msg = new BMessage (FB_OPEN) ;
	msg->AddRef ("file", &ref) ;
	msg->AddRef ("entries", &ref) ;
	SetMessage (msg) ;
	
	if (!(submenu->ExpandLayer()) && (!(submenu->Window()) || submenu->Window()->IsHidden())) 
		submenu->RebuildRoutines (&ref) ;
	
	fEnabled = true ;
	
	if (Menu() && Menu()->Window()) Menu()->Invalidate (Frame()) ;
	return B_OK ;
	
Fail :
	if (fIcon) {
		delete fIcon ;
		fIcon = NULL ;
	}
	
	fEnabled = false ;
	
	msg = new BMessage (FB_SHOW_DIALOG) ;
	msg->AddPointer ("layer", fOwner) ;
	msg->AddString ("path", Path()) ;
	SetMessage (msg) ;
	
	if (!(submenu->ExpandLayer()) && (!(submenu->Window()) || submenu->Window()->IsHidden())) 
		submenu->RebuildRoutines (NULL) ;
	
	if (Menu() && Menu()->Window()) Menu()->Invalidate (Frame()) ;
	return err ;
} ;

bool LayerItem:: IsSelected2 () const {
	return (IsSelected() || fSelected) ;
} ;

bool LayerItem:: IsEnabled2 () const {
	return (IsEnabled() && fEnabled) ;
} ;

bool LayerItem:: IsAvailable () const {
	LayerMenu* submenu = cast_as (Submenu(), LayerMenu) ;
	LayerInfo* target = submenu->fLayer ;
	const LayerItem* super = this ;
	BMenuItem* sitem = NULL ;
	while (super) {
		if (super->fOwner==target) return false ;
		sitem = super->Menu()->Superitem() ;
		super = (sitem) ? cast_as (sitem, LayerItem) : NULL ;
	}
	return true ;
} ;

void LayerItem:: GetContentSize (float *width, float *height) {
	fb_settings->get_item_size (width, height) ;
} ;

void LayerItem:: Draw () {
	
	BRect frame = Frame() ;
	BMenu *menu = Menu() ;
	rgb_color high = menu->HighColor() ;
	rgb_color low = menu->LowColor() ;
	BFont font ;
	float hor, ver ;
	BPoint ip ;
	
	menu->GetFont (&font) ;
	fb_settings->get_item_margins (&hor, &ver) ;
	
	// draw background
	if (IsSelected2()) {
		menu->SetLowColor (tint_color (low, B_HIGHLIGHT_BACKGROUND_TINT)) ;
		menu->FillRect (frame, B_SOLID_LOW) ;
	} else if (fb_settings->flags & FB_DRAW_BORDER) {
		stroke_3d_rect (menu, frame, low, true) ;
	}
	
	// draw "Expand Layer" mark
	if (cast_as (Submenu(), LayerMenu)->ExpandLayer()) {
		BRect rect = frame ;
		rect.OffsetBy (-1, -1) ;
		rect.top = rect.bottom - 3 ;
		rect.left = rect.right - 3 ;
		if (IsAvailable()) menu->SetHighColor (0, 0, 135) ;
		else menu->SetHighColor (100, 100, 155) ;
		menu->FillTriangle (rect.RightBottom(), rect.RightTop(), 
					rect.LeftBottom(), rect) ;
	}
	
	// draw icon and label
	if (fb_settings->flags & FB_DRAW_LABEL) {
		BPoint sp ;
		char label [32] ;
		switch (fb_settings->which) {
			case B_LARGE_ICON : 
				ip.x = ceil(frame.left + (frame.Width() - B_LARGE_ICON)/2) ;
				ip.y = ceil(frame.top + 2) ;
				TruncateLabel (frame.Width() - 2*hor, label) ;
				sp.x = frame.left + (frame.Width() - font.StringWidth (label))/2 ;
				sp.y = ip.y + B_LARGE_ICON + font.Size() ;
				break ;
			case B_MINI_ICON : 
				ip.x = ceil(hor) ;
				ip.y = ceil(frame.top + (frame.Height() - B_MINI_ICON)/2) ;
				TruncateLabel (frame.Width() - 2*hor - 23, label) ;
				sp.x = frame.left + hor + 23 ;
				sp.y = frame.top + (frame.Height() + font.Size())/2 ;
				break ;
		}
		
		if (!IsEnabled2() && IsEnabled()) {
			menu->SetHighColor (tint_color (high, B_LIGHTEN_1_TINT)) ;
		} else {
			menu->SetHighColor (0, 0, 0) ;
		}
		menu->DrawString (label, sp) ;
	} else {
		ip.x = ceil(frame.left + (frame.Width() - fb_settings->which)/2) ;
		ip.y = ceil(frame.top + (frame.Height() - fb_settings->which)/2) ;
	}
	if (fIcon) {
		menu->SetDrawingMode (B_OP_OVER) ;
		menu->DrawBitmap (fIcon, ip) ;
		menu->SetDrawingMode (B_OP_COPY) ;
	} else {
		stroke_3d_rect (menu, 
			BRect (ip.x, ip.y, 
				ip.x + fb_settings->which - 1, 
				ip.y + fb_settings->which - 1
			), 
			menu->LowColor(), false
		) ;
	}
	
	// finish
	menu->SetHighColor (high) ;
	menu->SetLowColor (low) ;
	menu->SetDrawingMode (B_OP_COPY) ;
} ;

void LayerItem:: MessageDropped (const BMessage *msg) {
	if (!fEnabled) {
		beep () ;
		return ;
	}
	
	status_t err = B_OK ;
	BMessage message (FB_OPEN) ;
	entry_ref ref ;
	int32 index = 0 ;
	void* text = NULL ;
	ssize_t size = 0 ;
	char path [B_PATH_NAME_LENGTH + 1] ;
	
	err = GetEntryRef (&ref) ;
	if (err!=B_OK) goto Fail1 ;
	
	message.AddRef ("file", &ref) ;
	
	while (msg->FindRef ("refs", index++, &ref)==B_OK) {
		message.AddRef ("entries", &ref) ;
	}
	
	index = 0 ;
	while (msg->FindData ("text/plain", B_MIME_TYPE, index++, &text, &size)==B_OK) {
		message.AddData ("text/plain", B_MIME_TYPE, text, size, true, 1) ;
	}
	
	err = be_app->PostMessage (&message) ;
	if (err!=B_OK) goto Fail2 ;
	
	return ;
	
Fail1 : 
	alert (FB_LAUNCH_ALERT, B_EMPTY_STRING, err) ;
	return ;
Fail2 : 
	alert (FB_LAUNCH_ALERT, ref.name, err) ;
	return ;
} ;

status_t LayerItem:: Invoke (BMessage *msg) {
	status_t err = B_OK ;
	BMessage invoke ;
	BHandler *target = Target() ;
	BLooper *looper = (target) ? target->Looper() : fb_manager ;
	LayerBar *bar = cast_as (Menu(), LayerBar) ;
	
	//	because of BMenu's bug, we must add these lines...
	if (bar) {
		if (bar->Selection()!=bar->IndexOf (this)) return B_OK ;
		if (bar->IndexOf (this)==-1) return B_OK ;
	} else {
		fb_manager->Lock() ;
		bar = cast_as (fb_manager->FindView ("layer_bar"), LayerBar) ;
		fb_manager->Unlock () ;
		if (bar->Selection()<0) return B_OK ;
	}
	
	if (msg) invoke = *msg ;
	else if (Message()) invoke = *(Message()) ;
	else return B_ERROR ;
	
	invoke.AddInt64 ("when", system_time()) ;
	invoke.AddPointer ("source", this) ;
	
	return looper->PostMessage (&invoke, target) ;
} ;

//==============================================================
//	class LayerMenu
//==============================================================
LayerMenu:: LayerMenu (bool expand) 
			: BMenu (B_EMPTY_STRING, 
					(expand && !IS_COLUMN) ? B_ITEMS_IN_ROW : B_ITEMS_IN_COLUMN) {
	fCurrent = FB_NO_LIST ;
	fLayer = NULL ;
	fExpandLayer = expand ;
	
	SetRadioMode (false) ;
	SetLabelFromMarked (false) ;
	SetTriggersEnabled (!expand) ;
	SetFont (be_plain_font) ;
} ;

LayerMenu:: ~LayerMenu () {
	// do nothing
} ;

void LayerMenu:: AttachedToWindow () {
	if (!fLayer) {
		// find my owner
		fb_manager->Lock () ;
		if (fExpandLayer) 
			fLayer = fb_manager->FindLayer (SuperLayerItem()->RefName()) ;
		else 
			fLayer = fb_manager->FindLayer (B_EMPTY_STRING) ;
		fb_manager->Unlock () ;
	}
	if (fExpandLayer) {
		float hor, ver ;
		fb_settings->get_item_margins (&hor, &ver) ;
		SetItemMargins (hor, ver, hor, ver) ;
	}
	
	if (fExpandLayer) {
		AttachLayer () ;
	} else {
		AttachTargets () ;
	}
	
	BMenu::AttachedToWindow () ;
} ;

void LayerMenu:: GetSize (float *width, float *height) {
	float hor, ver ;
	int32 count = CountItems () ;
	fb_settings->get_item_size (width, height) ;
	fb_settings->get_item_margins (&hor, &ver) ;
	
	if (count==0) {
		*width = 2*hor + 4 ;
		*height = 2*ver + 4 ;
		return ;
	}
	
	*width += 2*hor ;
	*height += 2*ver ;
	switch (Layout()) {
		case B_ITEMS_IN_ROW : 
			*width = (*width + 1)*count ;
			break ;
		case B_ITEMS_IN_COLUMN : 
			*height = (*height + 1)*count ;
			break ;
	}
} ;

void LayerMenu:: AllDetached () {
	BMenu::AllDetached () ;
	
	if (fExpandLayer) DetachLayer () ;
	else DetachTargets () ;
} ;

list_which LayerMenu:: CurrentList () const {
	return fCurrent ;
} ;

LayerItem* LayerMenu:: LayerItemAt (int32 index) const {
	BMenuItem *item = ItemAt (index) ;
	return (item) ? cast_as (item, LayerItem) : NULL ;
} ;

LayerItem* LayerMenu:: SuperLayerItem () const {
	return cast_as (Superitem(), LayerItem) ;
} ;

int32 LayerMenu:: IndexAt (BPoint p, bool screen) {
	if (screen) ConvertFromScreen (&p) ;
	
	int32 index = -1 ;
	float hor, ver, width, height ;
	fb_settings->get_item_margins (&hor, &ver) ;
	fb_settings->get_item_size (&width, &height) ;
	
	switch (Layout()) {
		case B_ITEMS_IN_COLUMN : 
			if (p.x<0 || p.x>width + 2*hor) return -1 ;
			index = (int)(p.y/(2*ver + height + 1)) ;
			break ;
		case B_ITEMS_IN_ROW :
			if (p.y<0 || p.y>height + 2*ver) return -1 ;
			index = (int)(p.x/(2*hor + width + 1)) ;
			break ;
	}
	if (index>=CountItems() || index<0) return -1 ;
	return index ;
} ;

bool LayerMenu:: ExpandLayer () const {
	return fExpandLayer ;
} ;

BPoint LayerMenu:: ScreenLocation () {
	if (!fExpandLayer) return BMenu::ScreenLocation () ;
	
	BPoint p = BMenu::ScreenLocation () ;
	float width, height ;
	BRect super = Supermenu()->ConvertToScreen (Superitem()->Frame()) ;
	GetSize (&width, &height) ;
	width += 1 ;
	height += 1 ;
	
	switch (fb_settings->direction) {
		case FB_LEFT_TO_RIGHT :
		case FB_TOP_TO_BOTTOM :
			break ;
		case FB_RIGHT_TO_LEFT :
			p.x = max_c (super.right - width, 0) ;
			break ;
		case FB_BOTTOM_TO_TOP :
			p.y = max_c (super.bottom - height, 0) ;
			break ;
	}
	return p ;
} ;

void LayerMenu:: AttachLayer () {
	// already filled with items
	if (fCurrent!=FB_NO_LIST) return ;
	// layer is not available
	if (!SuperLayerItem()->fEnabled) return ;
	
	LayerItem *item = NULL ;
	int32 count = 0 ;
	BList *items = fLayer->SetLock (true, this) ;
	
	// layer is not available
	if (!items) {
		BMenuItem *mitem = new BMenuItem ("Layer in Use", (BMessage*)NULL) ;
		mitem->SetEnabled (false) ;
		AddItem (mitem) ;
		return ;
	}
	
	count = items->CountItems() ;
	for (int32 i=0 ; i<count ; i++) {
		item = cast_as ((RefRetriever*)(items->ItemAt (i)), LayerItem) ;
		item->CheckNode (true) ;
		AddItem (item, (IS_POSITIVE) ? i : 0) ;
	}
	
	fCurrent = FB_LAYER_ITEM_LIST ;
} ;

void LayerMenu:: AttachTargets () {
	if (fCurrent!=FB_NO_LIST) return ;
	if (!(SuperLayerItem()->fEnabled)) return ;
	
	FileMenuItem *item = NULL ;
	int32 count = 0 ;
	entry_ref ref ;
	BList *items = NULL ;
	
	// if item is illegal, return
	if (SuperLayerItem()->GetEntryRef (&ref)!=B_OK) return ;
	
	items = fLayer->SetLock (true, this) ;
	
	// layer is not available
	if (!items) return ;
	
	count = items->CountItems () ;
	
	if (count) AddSeparatorItem () ;
	for (int32 i=0 ; i<count ; i++) {
		item = cast_as ((RefRetriever*)(items->ItemAt (i)), FileMenuItem) ;
		item->Message()->ReplaceRef ("file", &ref) ;
		item->CheckNode (true) ;
		AddItem (item) ;
	}
	InvalidateLayout () ;
	
	fCurrent = FB_MENU_ITEM_LIST ;
} ;

void LayerMenu:: DetachLayer () {
	RemoveItems (0, CountItems(), fCurrent!=FB_LAYER_ITEM_LIST) ;
	
	fLayer->SetLock (false, this) ;
	fCurrent = FB_NO_LIST ;
} ;

void LayerMenu:: DetachTargets () {
	if (fCurrent!=FB_MENU_ITEM_LIST) return ;
	
	int32 count = CountItems () ;
	FileMenuItem *item = NULL ;
	for (int32 i=count - 1 ; i>=0 ; i--) {
		item = cast_as (ItemAt (i), FileMenuItem) ;
		if (item) {
			RemoveItem (i) ;
		} else {
			if (cast_as (ItemAt (i), BSeparatorItem)) {
				// this item is a separator item
				delete RemoveItem (i) ;
			}
			break ;
		}
	}
	
	fLayer->SetLock (false, this) ;
	fCurrent = FB_NO_LIST ;
} ;

void LayerMenu:: RebuildRoutines (const entry_ref *ref) {
	status_t err = B_OK ;
	BMenuItem *mitem = NULL ;
	BMessage *msg = NULL ;
	BMessage message ;
	int32 count = CountItems () ;
	BMenu* submenu = NULL ;
	
	for (int32 i=count - 1 ; i>=0 ; i--) {
		delete RemoveItem (i) ;
	}
	
	if (!ref) {
		// super item is disabled. so...
		mitem = new BMenuItem (SuperLayerItem()->Path(), (BMessage*)NULL) ;
		mitem->SetEnabled (false) ;
		AddItem (mitem, 0) ;
		return ;
	}
	
	// add items in reverse order
	
	// 3. open with
	msg = new BMessage (FB_OPEN_WITH) ;
	msg->AddRef ("file", ref) ;
	submenu = new OpenWithMenu (ref, msg, B_ITEMS_IN_COLUMN) ;
	submenu->SetFont (be_plain_font) ;
	mitem = new BMenuItem (submenu) ;
	mitem->SetLabel ("Open With") ;
	AddItem (mitem, 0) ;
	mitem->SetTarget (be_app) ;
	
	// 2. open parent
	msg = new BMessage (FB_OPEN_PARENT) ;
	msg->AddRef ("file", ref) ;
	mitem = new BMenuItem ("Open Parent", msg) ;
	mitem->SetTarget (be_app) ;
	AddItem (mitem, 0) ;
	
	// 1. separator
	AddItem (mitem = new BSeparatorItem(), 0) ;
	
	// 0. file menu item
	message.what = FB_OPEN ;
	message.AddRef ("entries", ref) ;
	message.AddRef ("file", ref) ;
	mitem = DirectoryMenu::StartHierarchy (ref, &message) ;
	mitem->SetTarget (be_app) ;
	AddItem (mitem, 0) ;
	msg->what = FB_OPEN_PARENT ;
} ;

/*void LayerMenu:: PrintInfo () const {
	int32 count = CountItems () ;
	BMenuItem *item = NULL ;
	BRect *bounds = NULL ;
	
	printf ("archivable=%d, invoker=%d, menuitem=%d, messenger=%d\n", 
		sizeof (BArchivable), sizeof (BInvoker), sizeof (BMenuItem), sizeof (BMessenger)) ;
	
	for (int32 i=0 ; i<count ; i++) {
		item = ItemAt (i) ;
		bounds = (BRect*)((char*)item + 80) ;
		printf ("item[%d] \"%s\"\n", i, item->Label()) ;
		bounds->PrintToStream () ;
	}
} ;*/