//	LayerItem.h
//
//	Tetsuo Yamada
//
//	LayerItem represents an item which is registered to the layer.
//	LayerMenu is a submenu of LayerItem.

#ifndef		FB_LAYER_ITEM_h_
#define		FB_LAYER_ITEM_h_

#include	"FBBase.h"
#include	"LayerInfo.h"
#include	"GUI/FileMenuItem.h"
#include	"GUI/DirectoryMenu.h"
#include	"GUI/OpenWithMenu.h"
#include	"GUI/GUIBase.h"
#include	<interface/Menu.h>
#include	<interface/Bitmap.h>
#include	<interface/MenuItem.h>


class		LayerMenu ;
class		LayerItem ;
class		LayerBar ;


//==============================================================
//	class LayerMenu
//==============================================================
class LayerMenu : public BMenu {
// friends
friend class				LayerItem ;
friend class				LayerBar ;

// fields
private :
	list_which				fCurrent ; // current list state
	LayerInfo*				fLayer ; // layer to attach
	bool					fExpandLayer ;
	
// methods
public :
							LayerMenu (bool expandLayer) ;
							~LayerMenu () ;
	
	virtual void			AttachedToWindow () ;
	virtual void			AllDetached () ;
	
	list_which				CurrentList () const ;
	LayerItem*				LayerItemAt (int32 index) const ;
	LayerItem*				SuperLayerItem () const ;
	int32					IndexAt (BPoint p, bool screen = false) ;
	bool					ExpandLayer () const ;
	
protected :
	virtual BPoint			ScreenLocation () ;
	
private : 
	void					AttachLayer () ;
	void					DetachLayer () ;
	void					AttachTargets () ;
	void					DetachTargets () ;
	
	//	build routine items.
	void					RebuildRoutines (const entry_ref *ref = NULL) ;
	
	void					GetSize (float *width, float *height) ;
	
//	void					PrintInfo () const ;
} ;

//==============================================================
//	class LayerItem
//==============================================================
class LayerItem : public BMenuItem, public RefRetriever {
// friends
friend class				LayerMenu ;
friend class				LayerBar ;

// fiends
private :
	bool					fSelected ;
	bool					fEnabled ;
	LayerInfo*				fOwner ;
	BBitmap*				fIcon ;
	
// methods
public :
							LayerItem (const char *path, 
									LayerInfo *owner, 
									bool expandLayer) ;
							~LayerItem () ;
	
	bool					IsSelected2 () const ;
	bool					IsEnabled2 () const ;
	bool					IsAvailable () const ;
	
protected :
	virtual void			Draw () ;
	virtual void			GetContentSize (float *width, float *height) ;
	virtual status_t		RefUpdated () ;
	virtual status_t		Invoke (BMessage *msg = NULL) ;
	
private :
	void					MessageDropped (const BMessage *msg) ;
} ;

#endif

