//	LayerSet.cpp
//
//	Tetsuo Yamada

#include	"LayerSet.h"
#include	"PrefsWindow.h"


//==========================================================
//	class LayerSet
//==========================================================
LayerSet:: LayerSet () 
			: BView (BRect (0, 0, 300, 300), "layer_set", B_FOLLOW_ALL, 
			B_WILL_DRAW) {
	fCrtLayer = NULL ;
	fChanged = false ;
	
	SetViewColor (235, 235, 235) ;
	SetFont (be_plain_font) ;
	
	float bw = StringWidth ("Remove Item") + 16 ;
	float bh = 24 ;
	char label [64] ;
	BMessage *msg = NULL ;
	BRect rect ;
	BRect bounds = Bounds() ;
	bounds.InsetBy (SMALL_SPACE, SMALL_SPACE) ;
	
	// MenuField to select layer
	rect = bounds ;
	rect.right = bounds.right - bw - SMALL_SPACE ;
	rect.bottom = rect.top + bh ;
	strcpy (label, "Select Layer: ") ;
	fLayerField = new BMenuField (rect, "layer_field", label, new BMenu ("menu")) ;
	AddChild (fLayerField) ;
	fLayerField->SetFont (be_bold_font) ;
	fLayerField->SetDivider (be_bold_font->StringWidth (label) + 5) ;
	
	// "New Layer" button
	rect.OffsetBy (rect.Width() + SMALL_SPACE, 0) ;
	rect.right = bounds.right ;
	fNewLayer = new BButton (rect, "new_layer", "New Layer", 
			new BMessage (FB_LAYER_NEW_LAYER), B_FOLLOW_RIGHT|B_FOLLOW_TOP) ;
	AddChild (fNewLayer) ;
	
	// remove, rename buttons and StringView
	msg = new BMessage (FB_HANDLE_LAYER) ;
	
	rect.OffsetBy (0, rect.Height() + BIG_SPACE) ;
	fRemoveLayer = new BButton (rect, "remove", "Remove", 
			new BMessage (FB_LAYER_REMOVE_LAYER), B_FOLLOW_RIGHT|B_FOLLOW_TOP) ;
	
	rect.OffsetBy (- rect.Width() - SMALL_SPACE, 0) ;
	fRenameLayer = new BButton (rect, "rename", "Rename", 
			new BMessage (FB_LAYER_RENAME_LAYER), B_FOLLOW_RIGHT|B_FOLLOW_TOP) ;
	
	rect.right = rect.left - SMALL_SPACE ;
	rect.left = bounds.left ;
	fLayerName = new BStringView (rect, "layer_name", B_EMPTY_STRING, 
			B_FOLLOW_LEFT_RIGHT|B_FOLLOW_TOP) ;
	fLayerName->SetFont (be_bold_font) ;
	
	AddChild (fLayerName) ;
	AddChild (fRenameLayer) ;
	AddChild (fRemoveLayer) ;
	
	rect.OffsetBy (0, rect.Height() + SMALL_SPACE) ;
	rect.right = bounds.right - B_V_SCROLL_BAR_WIDTH ;
	rect.bottom = bounds.bottom - bh - SMALL_SPACE - B_H_SCROLL_BAR_HEIGHT ;
	rect.InsetBy (2, 2) ;
	fListView = new MatrixListView (rect, "list", 
			P_MLV_ITEM_CHANGED|P_MLV_MSG_DROPPED, B_MULTIPLE_SELECTION_LIST, 
			B_FOLLOW_ALL) ;
	fListView->AddColumn (new ListColumn (P_MLV_FILE_NAME, "Name", 140)) ;
	fListView->AddColumn (new ListColumn (P_MLV_FILE_PATH, "Path", 500)) ;
	fListView->SetSelectionMessage (new BMessage (FB_SELECTION_CHANGED)) ;
	BScrollView *scr = new BScrollView ("scr", fListView, B_FOLLOW_ALL, 0, 
			true, true) ;
	AddChild (scr) ;
	
	// add item button, remove item button
	rect.bottom = bounds.bottom ;
	rect.right = bounds.right ;
	rect.top = rect.bottom - bh ;
	rect.left = rect.right - bw ;
	fRemoveItem = new BButton (rect, "remove_item", "Remove Item", 
			new BMessage (FB_LAYER_REMOVE_ITEM), B_FOLLOW_RIGHT|B_FOLLOW_BOTTOM) ;
	
	rect.OffsetBy (- rect.Width() - SMALL_SPACE, 0) ;
	fAddItem = new BButton (rect, "add_item", "Add Item", 
			new BMessage (FB_SHOW_OPEN_PANEL), B_FOLLOW_RIGHT|B_FOLLOW_BOTTOM) ;
	
	AddChild (fAddItem) ;
	AddChild (fRemoveItem) ;
} ;

LayerSet:: ~LayerSet () {
	// do nothing
} ;

void LayerSet:: BuildMenu () const {
	BMessage container ;
	int32 index = 0 ;
	const char *name = NULL ;
	BMenu* menu = fLayerField->Menu() ;
	BMenuItem *item = NULL ;
	
	index = menu->CountItems() ;
	for (int32 i=index - 1 ; i>=0 ; i--) delete (BMenuItem*)(menu->RemoveItem(i)) ;
	
	menu->SetLabelFromMarked (true) ;
	menu->SetRadioMode (true) ;
	
	index = 0 ;
	cast_as (Window(), LayerManager)->GetLayerList (&container) ;
	while (container.FindString ("name", index++, &name)==B_OK) {
		if (strcmp (name, B_EMPTY_STRING)!=0) {
			item = new BMenuItem (name, new BMessage (FB_LAYER_SELECT_LAYER)) ;
			menu->AddItem (item) ;
			if (Window()) item->SetTarget (this) ;
		}
	}
} ;

void LayerSet:: SelectLayer (const char *name) {
	BList *items = NULL ;
	FileListItem *item = NULL ;
	int32 count = 0 ;
	LayerManager *manager = cast_as (Window(), LayerManager) ;
	bool enabled = false ;
	uint32 flags = fListView->InformFlags () ;
	
	if (manager) manager->DisableUpdates () ;
	
	if (fCrtLayer) {
		// unselect layer
		fListView->SetInformFlags (0) ;
		count = fListView->CountItems () ;
		for (int32 i=count - 1 ; i>=0 ; i--) fListView->RemoveItem (i) ;
		fCrtLayer->SetLock (false, this) ;
		fCrtLayer = NULL ;
		
		fLayerName->SetText (B_EMPTY_STRING) ;
		fLayerField->MenuBar()->ItemAt (0)->SetLabel (B_EMPTY_STRING) ;
		fListView->SetInformFlags (flags) ;
	}
	
	if (name && manager) fCrtLayer = manager->FindLayer (name) ;
	else fCrtLayer = NULL ;
	if (fCrtLayer) {
		// select layer
		items = fCrtLayer->SetLock (true, this) ;
		if (!items) goto Finish ;
		
		fListView->SetInformFlags (0) ;
		count = items->CountItems() ;
		for (int32 i=0 ; i<count ; i++) {
			item = cast_as ((RefRetriever*)(items->ItemAt (i)), FileListItem) ;
			fListView->AddItem (item) ;
		}
		
		fLayerName->SetText (name) ;
		fLayerField->Menu()->FindItem (name)->SetMarked (true) ;
		fListView->SetInformFlags (flags) ;
		
		enabled = true ;
	}
	
Finish : 
	fAddItem->SetEnabled (enabled) ;
	fRemoveItem->SetEnabled (false) ;
	fRenameLayer->SetEnabled (enabled) ;
	fRemoveLayer->SetEnabled (enabled) ;
	fListView->ScrollTo (0, 0) ;
	
	if (manager) manager->EnableUpdates() ;
	if (manager) manager->UpdateIfNeeded () ;
} ;

void LayerSet:: AttachedToWindow () {
	BuildMenu () ;
	if (!fCrtLayer) {
		BMenuItem *item = fLayerField->Menu()->ItemAt (0) ;
		if (item) {
			SelectLayer (item->Label()) ;
		} else {
			SelectLayer (NULL) ;
		}
	}
	
	fListView->SetTarget (this) ;
	fAddItem->SetTarget (Window()) ;
	fRemoveItem->SetTarget (this) ;
	fRenameLayer->SetTarget (this) ;
	fRemoveLayer->SetTarget (this) ;
	fNewLayer->SetTarget (this) ;
	fLayerField->Menu()->SetTargetForItems (this) ;
	
	BView::AttachedToWindow () ;
} ;

void LayerSet:: RescanItems () {
	if (!fCrtLayer) return ;
	
	int32 iCount = fListView->CountItems () ;
	int32 oCount = fCrtLayer->CountItems () ;
	
	if (oCount>iCount) {
		FileListItem *item = NULL ;
		BList *items = fCrtLayer->SetLock (true, this) ;
		uint32 flags = fListView->InformFlags () ;
		if (!items) return ;
		
		fListView->SetInformFlags (0) ;
		for (int32 i=iCount ; i<oCount ; i++) {
			item = cast_as ((RefRetriever*)(items->ItemAt (i)), FileListItem) ;
			fListView->AddItem (item) ;
		}
		fListView->SetInformFlags (flags) ;
	}
} ;

bool confirm_entry (const char *name, void* extra) {
	if (!name) return false ;
	
	for (int32 i=strlen (name) - 1 ; i>=0 ; i--) if (name[i]=='/') return false ;
	if (strcmp (name, B_EMPTY_STRING)==0) return false ;
	if (!extra) return false ;
	
	const char *layers = NULL ;
	BMessage *msg = (BMessage*)extra ;
	int32 index = 0 ;
	
	while (msg->FindString ("name", index++, &layers)==B_OK) {
		if (strcmp (layers, name)==0) return false ;
	}
	return true ;
} ;

void LayerSet:: MessageReceived (BMessage *msg) {
	
	TextDialog *dialog = NULL ;
	char name [B_FILE_NAME_LENGTH + 1] ;
	int32 selection = -1 ;
	PrefsWindow *window = cast_as (Window(), PrefsWindow) ;
	BMessage container ;
	
	switch (msg->what) {
		case FB_LAYER_SELECT_LAYER : { // select layer...
			BMenuItem *item = NULL ;
			msg->FindPointer ("source", (void **)&item) ;
			SelectLayer (item->Label()) ;
			break ;
		}
		case FB_LAYER_RENAME_LAYER : // rename current layer...
			window->GetLayerList (&container) ;
			dialog = new TextDialog ("Input A New Name: ", 
					fCrtLayer->LayerName(), confirm_entry, false, &container) ;
			if (dialog->Go (name)) {
				// name is changed.
				fCrtLayer->SetLayerName (name) ;
				fLayerName->SetText (name) ;
				fLayerField->Menu()->FindMarked ()->SetLabel (name) ;
				fLayerField->Menu()->Superitem()->SetLabel (name) ;
				fChanged = true ;
			}
			break ;
		case FB_LAYER_NEW_LAYER : // create a new layer...
			window->GetLayerList (&container) ;
			strcpy (name, "/") ;
			for (int32 i=1 ; !confirm_entry (name, &container) ; i++) {
				sprintf (name, "layer_%d", i) ;
			}
			dialog = new TextDialog ("Input A New Layer's Name: ", 
					name, confirm_entry, true, &container) ;
			if (dialog->Go (name)) {
				// newly created.
				LayerInfo* info = window->CreateLayer (name) ;
				if (info) {
					BuildMenu () ;
					fChanged = true ;
					SelectLayer (name) ;
				} else {
					beep () ;
				}
			}
			break ;
		case FB_LAYER_REMOVE_LAYER : {
			const char *layerName = fCrtLayer->LayerName() ;
			LayerInfo *info = window->FindNextLayer (fCrtLayer) ;
			if (info && strcmp (info->LayerName(), B_EMPTY_STRING)==0) 
				info = window->FindNextLayer (info) ;
			if (info && strcmp (info->LayerName(), B_EMPTY_STRING)==0) 
				info = NULL ;
			if (info && strcmp (layerName, info->LayerName())!=0) {
				SelectLayer (info->LayerName()) ;
			} else {
				SelectLayer (NULL) ;
			}
			window->DeleteLayer (layerName) ;
			BuildMenu () ;
			fChanged = true ;
			break ;
		}
		case FB_LAYER_REMOVE_ITEM : 
			RemoveItem () ;
			break ;
		case P_MLV_ITEM_ADDED : 
			ItemAdded (msg) ;
			break ;
		case P_MLV_ITEM_MOVED : 
			ItemMoved (msg) ;
			break ;
		case P_MLV_MSG_DROPPED : 
			ItemDropped (msg) ;
			break ;
		case B_REFS_RECEIVED : 
			RefsReceived (msg) ;
			break ;
		case FB_SELECTION_CHANGED : 
			break ;
		default : 
			BView::MessageReceived (msg) ;
			break ;
	}
	
	selection = -1 ;
	selection = fListView->CurrentSelection () ;
	fRemoveItem->SetEnabled (selection>=0) ;
	fRenameLayer->SetEnabled (fCrtLayer) ;
	fRemoveLayer->SetEnabled (fCrtLayer) ;
} ;

void LayerSet:: RemoveItem () {
	
	fChanged = true ;
	
	uint32 flags = fListView->InformFlags() ;
	int32 index = 0 ;
	BList *items = fCrtLayer->SetLock (true, this) ;
	FileListItem *item = NULL ;
	
	while ((index = fListView->CurrentSelection (0))>=0) {
		fListView->RemoveItem (index) ;
		item = cast_as ((RefRetriever*)(items->RemoveItem (index)), FileListItem) ;
		delete item ;
	}
	
	fListView->SetInformFlags (flags) ;
} ;

void LayerSet:: ItemAdded (BMessage *msg) {
	
	int32 index = -1 ;
	BList *items = fCrtLayer->SetLock (true, this) ;
	FileListItem *item = NULL ;
	
	if (!items) return ;
	
	if (msg->FindInt32 ("index", &index)==B_OK) {
		item = cast_as (fListView->ItemAt (index), FileListItem) ;
		items->AddItem ((RefRetriever*)item, index) ;
	}
	fChanged = true ;
} ;

void LayerSet:: ItemMoved (BMessage *msg) {
	int32 from = -1, to = -1 ;
	BList *items = fCrtLayer->SetLock (true, this) ;
	if (!items) return ;
	if (msg->FindInt32 ("from", &from)==B_OK && msg->FindInt32 ("to", &to)==B_OK) {
		RefRetriever *item = (RefRetriever*)(items->RemoveItem (from)) ;
		items->AddItem (item, to) ;
	}
	fChanged = true ;
} ;

void LayerSet:: ItemDropped (BMessage *msg) {
	if (!fCrtLayer) {
		beep () ;
		return ;
	}
	
	status_t err = B_OK ;
	int32 i = 0, j = 0 ;
	BMessage dropped ;
	entry_ref ref ;
	
	err = msg->FindInt32 ("dropped_index", &i) ;
	if (err==B_OK) err = msg->FindMessage ("dropped_msg", &dropped) ;
	if (err==B_OK) {
		while (dropped.FindRef ("refs", j++, &ref)==B_OK) {
			if (i>=0) {
				fListView->AddItem (new FileListItem (&ref), i++) ;
			} else {
				fListView->AddItem (new FileListItem (&ref)) ;
			}
		}
	}
} ;

void LayerSet:: RefsReceived (BMessage *msg) {
	if (!fCrtLayer) {
		beep () ;
		return ;
	}
	
	int32 index = 0 ;
	entry_ref ref, old ;
	
	while (msg->FindRef ("refs", index++, &ref)==B_OK) {
		if (ref!=old) fListView->AddItem (new FileListItem (&ref)) ;
		old = ref ;
	}
} ;