//	MiscSet.cpp
//
//	Tetsuo Yamada

#include	"MiscSet.h"


//==========================================================
//	class MiscSet
//==========================================================
MiscSet:: MiscSet (settings *crt) 
		: BView (BRect (0, 0, 200, 200), "misc_set", B_FOLLOW_ALL, B_WILL_DRAW) {
	fCrt = crt ;
	
	SetFont (be_plain_font) ;
	SetViewColor (235, 235, 235) ;
	
	BRect rect = Bounds() ;
	
	float width, height ;
	BMessage *msg = NULL ;
	BMenu *menu = NULL ;
	
	// 0. Enable Switch Workspace Command
	rect.InsetBy (SMALL_SPACE, SMALL_SPACE) ;
	msg = new BMessage (FB_MISC_MSG) ;
	msg->AddInt32 ("id", 0) ;
	fWsCmd = new BCheckBox (rect, "ws_cmd", "Enable \"Switch Workspace\" Command", 
			msg) ;
	AddChild (fWsCmd) ;
	fWsCmd->GetPreferredSize (&width, &height) ;
	fWsCmd->ResizeTo (width, height) ;
	
	// 1. Use a peculiar icon for the new layer
	rect.OffsetBy (0, height + SMALL_SPACE) ;
	msg = new BMessage (FB_MISC_MSG) ;
	msg->AddInt32 ("id", 1) ;
	fOrigIcon = new BCheckBox (rect, "orig_icon", 
			"Use Peculiar Icons For New Layer", msg) ;
	AddChild (fOrigIcon) ;
	fOrigIcon->GetPreferredSize (&width, &height) ;
	fOrigIcon->ResizeTo (width, height) ;
	
	// 2. Enable Shelf
	rect.OffsetBy (0, height + SMALL_SPACE) ;
	msg = new BMessage (FB_MISC_MSG) ;
	msg->AddInt32 ("id", 2) ;
	fShelf = new BCheckBox (rect, "shelf", 
			"Enable Shelf", msg) ;
	AddChild (fShelf) ;
	fShelf->GetPreferredSize (&width, &height) ;
	fShelf->ResizeTo (width, height) ;
	
	// 3. shelf size
	rect.OffsetBy (BIG_SPACE*2, height + 1) ;
	rect.bottom = rect.top + be_plain_font->Size() + 10 ;
	fShelfSize = new BMenuField (rect, "shelf_size", "Shelf Size:", 
		(menu = new BMenu("disabled"))) ;
	menu->SetLabelFromMarked (true) ;
	menu->SetRadioMode (true) ;
	for (int32 i=1 ; i<=10 ; i++) {
		char label [3] ;
		sprintf (label, "%d", i) ;
		msg = new BMessage (FB_MISC_MSG) ;
		msg->AddInt32 ("id", 3) ;
		msg->AddInt32 ("size", i) ;
		menu->AddItem (new BMenuItem (label, msg)) ;
	}
	AddChild (fShelfSize) ;
	fShelfSize->SetDivider (StringWidth ("Shelf Size:") + 5) ;
	fShelfSize->GetPreferredSize (&width, &height) ;
	fShelfSize->ResizeTo (width, height) ;
	
	// 4. Install Shelf on the Tail... tail???
	rect.OffsetBy (0, height + SMALL_SPACE) ;
	msg = new BMessage (FB_MISC_MSG) ;
	msg->AddInt32 ("id", 4) ;
	fLocation = new BCheckBox (rect, "shelf_loc", 
			"Install Shelf on the Tail", msg) ;
	AddChild (fLocation) ;
	fLocation->GetPreferredSize (&width, &height) ;
	fLocation->ResizeTo (width, height) ;
} ;

MiscSet:: ~MiscSet () {
	// do nothing
} ;

void MiscSet:: SettingsToViews () {
	bool shelfEnabled = (fCrt->flags & FB_ENABLE_SHELF) ;
	fWsCmd->SetValue (fCrt->flags & FB_WORKSPACE_CMD) ;
	fOrigIcon->SetValue (fCrt->flags & FB_ORIGINAL_ICON) ;
	fShelf->SetValue (shelfEnabled) ;
	fShelfSize->Menu()->ItemAt (fCrt->shelfSize - 1)->SetMarked (true) ;
	fLocation->SetValue (fCrt->flags & FB_SHELF_BOTTOM) ;
	
	fShelfSize->SetEnabled (shelfEnabled) ;
	fLocation->SetEnabled (shelfEnabled) ;
} ;

void MiscSet:: AttachedToWindow () {
	fWsCmd->SetTarget (this) ;
	fOrigIcon->SetTarget (this) ;
	fShelfSize->Menu()->SetTargetForItems (this) ;
	fShelf->SetTarget (this) ;
	fLocation->SetTarget (this) ;
	
	SettingsToViews () ;
	BView::AttachedToWindow () ;
} ;

void MiscSet:: MessageReceived (BMessage *msg) {
	if (msg->what==FB_MISC_MSG) {
		int32 id = -1 ;
		msg->FindInt32 ("id", &id) ;
		
		switch (id) {
			case 0 : 
				fCrt->set_flags (FB_WORKSPACE_CMD, fWsCmd->Value()) ;
				break ;
			case 1 : 
				fCrt->set_flags (FB_ORIGINAL_ICON, fOrigIcon->Value()) ;
				break ;
			case 2 : 
				fCrt->set_flags (FB_ENABLE_SHELF, fShelf->Value()) ;
				fShelfSize->SetEnabled (fShelf->Value()) ;
				fLocation->SetEnabled (fShelf->Value()) ;
				break ;
			case 3 : 
				msg->FindInt32 ("size", &id) ;
				fCrt->shelfSize = id ;
				break ;
			case 4 : 
				fCrt->set_flags (FB_SHELF_BOTTOM, fLocation->Value()) ;
				break ;
		}
	} else {
		BView::MessageReceived (msg) ;
	}
} ;