//	PrefsWindow.cpp
//
//	Tetsuo Yamada

#include	"PrefsWindow.h"


extern IOSlave* fb_slave ;
extern LayerManager* fb_manager ;
extern status_t create_ref_item (const char*, LayerInfo*, RefRetriever**) ;

//================================================================
//	class PrefsWindow
//================================================================
PrefsWindow:: PrefsWindow () 
			: LayerManager (BRect (0, 0, 500, 500), "FakeBar Preferences", 
					B_NOT_ZOOMABLE|B_ASYNCHRONOUS_CONTROLS, 
					FB_LIST_ITEM_LIST, FB_LIST_ITEM_LIST) {
	
	fPanel = NULL ;
	fPanelDir.device = -1 ;
	fMessenger = new BMessenger (this) ;
	
	SetLook (B_TITLED_WINDOW_LOOK) ;
	SetFeel (B_NORMAL_WINDOW_FEEL) ;
	fTmpSettings.set_to (fSettings) ;
	
	status_t err = B_OK ;
	BRect frame = BScreen (this).Frame() ;
	BRect rect ;
	int32 index = 0 ;
	BNode node ;
	
	SetSizeLimits (PREFS_MIN_WIDTH, frame.right, PREFS_MIN_HEIGHT, frame.bottom) ;
	
	get_settings_node (&node) ;
	
	err = node.ReadAttr ("prefs_frame", B_RECT_TYPE, 0, &rect, sizeof (rect)) ;
	if (err<B_OK) {
		// No prefs archive
		rect.left = (frame.Width() - PREFS_MIN_WIDTH)/2 ;
		rect.top = (frame.Height() - PREFS_MIN_HEIGHT)/2 ;
		rect.right = rect.left + PREFS_MIN_WIDTH ;
		rect.bottom = rect.top + PREFS_MIN_HEIGHT ;
	} else if (!frame.Contains (rect.LeftTop())) {
		// prefs window bounds rectangle is out of screen bounds...
		rect.left = max_c((frame.Width() - rect.Width())/2, 50) ;
		rect.top = max_c((frame.Height() - rect.Height())/2, 50) ;
	}
	ResizeTo (rect.Width(), rect.Height()) ;
	MoveTo (rect.left, rect.top) ;
	
	BuildChildren (rect.Width(), rect.Height()) ;
	
	err = node.ReadAttr ("prefs_index", B_INT32_TYPE, 0, &index, 4) ;
	if (err<B_OK || index<0) index = 0 ;
	SetControl (index) ;
	fList->MakeFocus (true) ;
	fList->SetSelectionMessage (new BMessage (FB_SELECTION_CHANGED)) ;
	fRemember->SetValue (err>=B_OK) ;
} ;

PrefsWindow:: ~PrefsWindow () {
	delete fMessenger ;
} ;

void PrefsWindow:: BuildChildren (float width, float height) {
	
	const BRect leftside (SMALL_SPACE, SMALL_SPACE, 100, height - SMALL_SPACE) ;
	const BRect rightside (leftside.right + BIG_SPACE, SMALL_SPACE, width - SMALL_SPACE, height - SMALL_SPACE) ;
	float bw = leftside.Width() ;
	BRect rect ;
	
	// bottom view
	BView *bottom = new BView (Bounds(), "bottom", B_FOLLOW_ALL, B_WILL_DRAW) ;
	bottom->SetViewColor (235, 235, 235) ;
	AddChild (bottom) ;
	
	// index list view.
	rect = leftside ;
	rect.bottom = 100 ;
	rect.right -= B_V_SCROLL_BAR_WIDTH ;
	rect.InsetBy (2, 2) ;
	fList = new BListView (rect, "list", B_SINGLE_SELECTION_LIST, B_FOLLOW_ALL) ;
	fList->AddItem (new BStringItem ("Window")) ;
	fList->AddItem (new BStringItem ("Workspaces")) ;
	fList->AddItem (new BStringItem ("Layers")) ;
	fList->AddItem (new BStringItem ("Targets")) ;
	fList->AddItem (new BStringItem ("Misc")) ;
	
	BScrollView *scr = new BScrollView ("list_scr", fList, B_FOLLOW_LEFT|B_FOLLOW_TOP, 
			B_NAVIGABLE, false, true) ;
	bottom->AddChild (scr) ;
	
	// remember ?
	rect = scr->Frame() ;
	rect.OffsetBy (0, rect.Height() + BIG_SPACE) ;
	rect.bottom = rect.top + be_plain_font->Size() + 8 ;
	fRemember = new BCheckBox (rect, "remember", "Don't forget", NULL) ;
	bottom->AddChild (fRemember) ;
	
	rect.left += 18 ;
	rect.OffsetBy (0, rect.Height()) ;
	BStringView *r1 = new BStringView (rect, "remember1", "this window's") ;
	bottom->AddChild (r1) ;
	rect.OffsetBy (0, rect.Height()) ;
	BStringView *r2 = new BStringView (rect, "remember2", "status.") ;
	bottom->AddChild (r2) ;
	
	// command buttons
	fRevert = new BButton (BRect (0, 0, bw, 1), "revert", "Revert", 
			new BMessage (FB_REVERT_PREFS), B_FOLLOW_LEFT|B_FOLLOW_BOTTOM) ;
	bottom->AddChild (fRevert) ;
	
	fCancel = new BButton (BRect (0, 0, bw, 1), "cancel", "Cancel", 
			new BMessage (B_QUIT_REQUESTED), B_FOLLOW_LEFT|B_FOLLOW_BOTTOM) ;
	bottom->AddChild (fCancel) ;
	
	fSet = new BButton (BRect (0, 0, bw, 1), "set", "Set", 
			new BMessage (FB_SET_PREFS), B_FOLLOW_LEFT|B_FOLLOW_BOTTOM) ;
	bottom->AddChild (fSet) ;
	
	rect = fSet->Bounds() ;
	fSet->MoveTo (leftside.left + (leftside.Width() - rect.Width())/2, 
			leftside.bottom - rect.Height()) ;
	fCancel->MoveTo (leftside.left + (leftside.Width() - rect.Width())/2, 
			leftside.bottom - 2*rect.Height() - BIG_SPACE) ;
	fRevert->MoveTo (leftside.left + (leftside.Width() - rect.Width())/2, 
			leftside.bottom - 3*rect.Height() - 2*BIG_SPACE) ;
	SetDefaultButton (fSet) ;
	
	// main controls box
	rect = rightside ;
	fBox = new BBox (rect, NULL, B_FOLLOW_ALL) ;
	bottom->AddChild (fBox) ;
	
	// controls
	fWindow = new WindowSet (&fSettings) ;
	fWorkspace = new WorkspaceSet (&fSettings) ;
	fLayer = new LayerSet () ;
	fTarget = new TargetSet (this) ;
	fMisc = new MiscSet (&fSettings) ;
} ;

void PrefsWindow:: SetControl (int32 index) {
	
	BRect rect ;
	BView *oldView = NULL ;
	BView *newView = NULL ;
	
	if (fPanel) {
		delete fPanel ;
		fPanel = NULL ;
	}
	
	// first of all, remove a current control from view hierarchy.
	if (fWindow->Parent()) {
		if (index==0) return ;
		oldView = fWindow ;
	} else if (fWorkspace->Parent()) {
		if (index==1) return ;
		oldView = fWorkspace ;
	} else if (fLayer->Parent()) {
		if (index==2) return ;
		oldView = fLayer ;
	} else if (fTarget->Parent()) {
		if (index==3) return ;
		oldView = fTarget ;
	} else if (fMisc->Parent()) {
		if (index==4) return ;
		oldView = fMisc ;
	} else {
		if (index<0) return ;
		oldView = NULL ;
	}
	
	// then, add new control to the view
	switch (index) {
		case 0 : // add WindowSet.
			newView = fWindow ;
			fBox->SetLabel ("Window") ;
			break ;
		case 1 : // add WorkspaceSet.
			newView = fWorkspace ;
			fBox->SetLabel ("Workspaces") ;
			break ;
		case 2 : // add LayerSet.
			newView = fLayer ;
			fBox->SetLabel ("Layers") ;
			break ;
		case 3 : // add TargetSet.
			newView = fTarget ;
			fBox->SetLabel ("Targets") ;
			break ;
		case 4 : // add MiscSet.
			newView = fMisc ;
			fBox->SetLabel ("Misc") ;
			break ;
		default : // unselect current selection.
			newView = NULL ;
			fBox->SetLabel (B_EMPTY_STRING) ;
			break ;
	}
	
	DisableUpdates () ;
	if (oldView) oldView->RemoveSelf () ;
	
	if (newView) {
		rect = fBox->Bounds() ;
		rect.InsetBy (SMALL_SPACE, SMALL_SPACE) ;
		rect.top += BIG_SPACE ;
		fList->Select (index) ;
		fBox->AddChild (newView) ;
		newView->MoveTo (rect.left, rect.top) ;
		newView->ResizeTo (rect.Width(), rect.Height()) ;
	}
	
	EnableUpdates () ;
	UpdateIfNeeded () ;
} ;

void PrefsWindow:: DispatchMessage (BMessage *msg, BHandler *handler) {
	LayerManager::DispatchMessage (msg, handler) ;
	if ((msg->what>='fb\0\0' && msg->what<='fb\255\255') || msg->what<=128) {
		Lock () ;
		bool rescan = (msg->what==FB_ITEMS_FOUND) ;
		bool changed = (fSettings.compare (&fTmpSettings, true)!=0) ;
		bool finish = false ;
		
		if (!changed) changed = fLayer->fChanged ;
		if (!changed) changed = fTarget->fChanged ;
		
		fRevert->SetEnabled (changed) ;
		fSet->SetEnabled (changed) ;
		if (rescan && fLayer->Parent() && 
					fLayer->fCrtLayer && fLayer->fCrtLayer==handler) {
			msg->FindBool ("finish", &finish) ;
			if (!finish) fLayer->RescanItems () ;
			else fLayer->SelectLayer (fLayer->fCrtLayer->LayerName()) ;
		} else if (rescan && fTarget->Parent() && fTarget->fTargets==handler) {
			msg->FindBool ("finish", &finish) ;
			if (!finish) fTarget->RescanItems() ;
		}
		Unlock () ;
	}
} ;

bool PrefsWindow:: QuitRequested () {
	BNode node ;
	get_settings_node (&node) ;
	
	if (fRemember->Value()) {
		// if "remember prefs window's status" checked
		BRect frame = Frame() ;
		int32 index = fList->CurrentSelection () ;
		node.WriteAttr ("prefs_frame", B_RECT_TYPE, 0, &frame, sizeof (frame)) ;
		node.WriteAttr ("prefs_index", B_INT32_TYPE, 0, &index, 4) ;
	} else {
		node.RemoveAttr ("prefs_frame") ;
		node.RemoveAttr ("prefs_index") ;
	}
	
	be_app->PostMessage (FB_PREFS_CLOSED) ;
	return LayerManager::QuitRequested () ;
} ;

void PrefsWindow:: MessageReceived (BMessage *msg) {
	int32 index = 0 ;
	switch (msg->what) {
		case FB_REVERT_PREFS : 
			Revert () ;
			break ;
		case FB_SET_PREFS : 
			Set () ;
			break ;
		case FB_SELECTION_CHANGED : 
			index = fList->CurrentSelection () ;
			if (index>=0) SetControl (index) ;
			break ;
		case FB_SHOW_OPEN_PANEL : 
			if (fPanel) {
				fPanel->Window()->Activate (true) ;
			} else {
				fPanel = new BFilePanel (B_OPEN_PANEL, fMessenger, NULL, 
						B_ANY_NODE, true, NULL, NULL, false, true) ;
				if (fPanelDir.device>=0) fPanel->SetPanelDirectory (&fPanelDir) ;
				fPanel->Show () ;
			}
			break ;
		case B_REFS_RECEIVED : 
			if (fLayer->Parent()) fLayer->MessageReceived (msg) ;
			else if (fTarget->Parent()) fTarget->MessageReceived (msg) ;
			fPanel->GetPanelDirectory (&fPanelDir) ;
			break ;
		case B_CANCEL : 
			fPanel->GetPanelDirectory (&fPanelDir) ;
			delete fPanel ;
			fPanel = NULL ;
			break ;
		default : 
			LayerManager::MessageReceived (msg) ;
			break ;
	}
} ;

void PrefsWindow:: Quit () {
	if (!(fWindow->Parent())) delete fWindow ;
	if (!(fWorkspace->Parent())) delete fWorkspace ;
	if (!(fLayer->Parent())) delete fLayer ;
	if (!(fTarget->Parent())) delete fTarget ;
	if (!(fMisc->Parent())) delete fMisc ;
	
	LayerManager::Quit () ;
} ;

void PrefsWindow:: Revert () {
	int32 selection = fList->CurrentSelection () ;
	// before revert...
	if (fLayer->fCrtLayer) fLayer->SelectLayer (NULL) ;
	SetControl (-1) ;
	
	// reverting...
	fSettings.set_to (fTmpSettings) ;
	LoadLayers () ;
	
	// after revert...
	fLayer->fChanged = false ;
	fTarget->fChanged = false ;
	SetControl (selection) ;
} ;

void PrefsWindow:: Set () {
	/*** HOW TO UPDATE PREFERENCES ***
		if the layers (includes targets) have changed (even if it is only one), 
		save new preferences. Then, the main window reload them.
		Otherwise (changes are only settings), 
		save a new settings and send its archive to the main window.
	****/
	
	// unlock all layers
	fLayer->SelectLayer (NULL) ;
	if (fTarget->Parent()) fTarget->RemoveSelf() ;
	
	entry_ref ref ;
	BFile file ;
	status_t err = B_OK ;
	LayerInfo *info = NULL ;
	LayerInfo *orig = NULL ;
	int32 count = CountHandlers () ;
	bool reloadNeeded = false ; // if this flag is true, the main window must reload all settings.
	BMessage message (FB_UPDATE_SETTINGS) ;
	
	PostMessage (B_QUIT_REQUESTED) ;
	
	// update settings file
	fSettings.archive_to (&message) ;
	
	err = get_settings (fb_settings_name, &ref) ;
	if (err!=B_OK) return ;
	
	err = file.SetTo (&ref, B_WRITE_ONLY|B_CREATE_FILE) ;
	if (err!=B_OK) return ;
	
	err = file.InitCheck () ;
	if (err!=B_OK) return ;
	
	err = message.Flatten (&file) ;
	if (err!=B_OK) return ;
	
	// update layers...
	if (!(fb_manager->Lock())) return ;
	for (int32 i=0 ; i<count ; i++) {
		info = cast_as (HandlerAt (i), LayerInfo) ;
		if (!info) continue ;
		
		orig = fb_manager->FindLayer (info->LayerName()) ;
		
		if (orig) {
			orig->fData |= 1 ; // checked flag
			if (orig->CompareItems(info)==0) continue ;
		}
		reloadNeeded = true ;
		
		// send message to update the layer file.
		BMessage saveMsg (FB_SAVE_LAYER) ;
		saveMsg.AddBool ("icon", (fSettings.flags & FB_ORIGINAL_ICON)) ;
		BList *items = info->SetLock (true, this) ;
		int32 icount = 0 ;
		RefRetriever* item = NULL ;
		
		if (!items) continue ;
		
		icount = items->CountItems() ;
		for (int32 j=0 ; j<icount ; j++) {
			item = (RefRetriever*)(items->ItemAt (j)) ;
			saveMsg.AddString ("paths", item->Path()) ;
		}
		saveMsg.AddString ("name", info->LayerName()) ;
		fb_slave->PostMessage (&saveMsg) ;
	}
	
	// find removed layer and tell fb_slave to delete it
	for (int32 i=0 ; i<count ; i++) {
		orig = cast_as (fb_manager->HandlerAt (i), LayerInfo) ;
		if (!orig) continue ;
		if (orig->fData & 0x1) {
			orig->fData = (orig->fData & ~0x1) ;
			continue ;
		}
		
		BMessage msg (FB_DELETE_LAYER) ;
		msg.AddString ("name", orig->LayerName()) ;
		fb_slave->PostMessage (&msg) ;
		reloadNeeded = true ;
	}
	
	fb_manager->Unlock () ;
	
	//	the main window reloads settings, shelf, layers, targets.
	if (reloadNeeded) be_app->PostMessage (FB_LOAD_PREFS) ;
	//	the main window updates its settings.
	else fb_manager->PostMessage (&message) ;
} ;

LayerInfo* PrefsWindow:: CreateLayer (const char *name) {
	return AddLayer (name, false) ;
} ;

bool PrefsWindow:: DeleteLayer (const char *name) {
	LayerInfo *info = RemoveLayer (name) ;
	if (info) {
		delete info ;
		return true ;
	}
	return false ;
} ;