//	TargetSet.cpp
//
//	Tetsuo Yamada

#include	"TargetSet.h"


//==========================================================
//	class TargetSet
//==========================================================
TargetSet:: TargetSet (LayerManager *manager) 
		: BView (BRect (0, 0, 300, 300), "target_set", B_FOLLOW_ALL, B_WILL_DRAW) {
	
	fTargets = manager->FindLayer (B_EMPTY_STRING) ;
	fChanged = false ;
	
	SetViewColor (235, 235, 235) ;
	SetFont (be_plain_font) ;
	
	float bw = StringWidth ("Remove Item") + 16 ;
	float bh = 24 ;
	BRect rect ;
	BRect bounds = Bounds() ;
	bounds.InsetBy (SMALL_SPACE, SMALL_SPACE) ;
	
	// MatrixListView and it's ScrollView
	rect = bounds ;
	rect.right -= B_V_SCROLL_BAR_WIDTH ;
	rect.bottom -= B_H_SCROLL_BAR_HEIGHT + bh + SMALL_SPACE ;
	rect.InsetBy (2, 2) ;
	fListView = new MatrixListView (rect, "list", 
			P_MLV_ITEM_CHANGED|P_MLV_MSG_DROPPED, B_MULTIPLE_SELECTION_LIST, 
			B_FOLLOW_ALL) ;
	fListView->AddColumn (new ListColumn (P_MLV_FILE_NAME, "Name", 140)) ;
	fListView->AddColumn (new ListColumn (P_MLV_FILE_PATH, "Path", 500)) ;
	fListView->SetSelectionMessage (new BMessage (FB_SELECTION_CHANGED)) ;
	BScrollView *scr = new BScrollView ("scr", fListView, B_FOLLOW_ALL, 0, 
			true, true) ;
	AddChild (scr) ;
	
	// buttons
	rect = bounds ;
	rect.top = rect.bottom - bh ;
	rect.left = rect.right - bw ;
	fRemoveItem = new BButton (rect, "remove_item", "Remove Item", 
			new BMessage (FB_TARGET_REMOVE_ITEM), B_FOLLOW_RIGHT|B_FOLLOW_BOTTOM) ;
	
	rect.OffsetBy (- rect.Width() - SMALL_SPACE, 0) ;
	fAddItem = new BButton (rect, "add item", "Add Item", 
			new BMessage (FB_SHOW_OPEN_PANEL), B_FOLLOW_RIGHT|B_FOLLOW_BOTTOM) ;
	
	AddChild (fAddItem) ;
	AddChild (fRemoveItem) ;
} ;

TargetSet:: ~TargetSet () {
	// do nothing
} ;

void TargetSet:: AttachedToWindow () {
	fListView->SetTarget (this) ;
	fAddItem->SetTarget (Window()) ;
	fRemoveItem->SetTarget (this) ;
	
	uint32 flags = fListView->InformFlags() ;
	BList* items = fTargets->SetLock (true, this) ;
	FileListItem *item = NULL ;
	int32 count = 0 ;
	if (!items) {
		/*printf ("---------- targets locked!!!\n") ;
		exit (1) ;*/
		return ;
	}
	
	fListView->SetInformFlags (0) ;
	count = items->CountItems () ;
	for (int32 i=0 ; i<count ; i++) {
		item = cast_as ((RefRetriever*)(items->ItemAt (i)), FileListItem) ;
		fListView->AddItem (item) ;
	}
	fListView->SetInformFlags (flags) ;
	
	BView::AttachedToWindow () ;
	
	int32 selection = fListView->CurrentSelection () ;
	fRemoveItem->SetEnabled (selection>=0) ;
} ;

void TargetSet:: DetachedFromWindow () {
	int32 count = fListView->CountItems() ;
	uint32 flags = fListView->InformFlags () ;
	fListView->SetInformFlags (0) ;
	for (int32 i=count - 1 ; i>=0 ; i--) fListView->RemoveItem (i) ;
	fTargets->SetLock (false, this) ;
	fListView->SetInformFlags (flags) ;
	
	BView::DetachedFromWindow() ;
} ;

void TargetSet:: RescanItems () {
	BList *items = fTargets->SetLock (true, this) ;
	if (!items) return ;
	int32 iCount = fListView->CountItems () ;
	int32 oCount = fTargets->CountItems () ;
	
	if (oCount>iCount) {
		FileListItem *item = NULL ;
		uint32 flags = fListView->InformFlags () ;
		
		fListView->SetInformFlags (0) ;
		for (int32 i=iCount ; i<oCount ; i++) {
			item = cast_as ((RefRetriever*)(items->ItemAt (i)), FileListItem) ;
			fListView->AddItem (item) ;
		}
		fListView->SetInformFlags (flags) ;
	}
} ;

void TargetSet:: MessageReceived (BMessage *msg) {
	switch (msg->what) {
		case FB_TARGET_REMOVE_ITEM : 
			RemoveItem () ;
			break ;
		case P_MLV_ITEM_ADDED : 
			ItemAdded (msg) ;
			break ;
		case P_MLV_ITEM_MOVED : 
			ItemMoved (msg) ;
			break ;
		case P_MLV_MSG_DROPPED : 
			ItemDropped (msg) ;
			break ;
		case B_REFS_RECEIVED : 
			RefsReceived (msg) ;
			break ;
		case FB_SELECTION_CHANGED : 
			break ;
		default : 
			BView::MessageReceived (msg) ;
			break ;
	}
	
	int32 selection = fListView->CurrentSelection () ;
	fRemoveItem->SetEnabled (selection>=0) ;
} ;

void TargetSet:: RemoveItem () {
	uint32 flags = fListView->InformFlags() ;
	int32 index = 0 ;
	BList *items = fTargets->SetLock (true, this) ;
	FileListItem *item = NULL ;
	
	while ((index = fListView->CurrentSelection (0))>=0) {
		fListView->RemoveItem (index) ;
		item = cast_as ((RefRetriever*)(items->RemoveItem (index)), FileListItem) ;
		delete item ;
		fChanged = true ;
	}
	
	fListView->SetInformFlags (flags) ;
} ;

void TargetSet:: ItemAdded (BMessage *msg) {
	int32 index = -1 ;
	BList *items = fTargets->SetLock (true, this) ;
	FileListItem *item = NULL ;
	
	if (!items) return ;
	
	if (msg->FindInt32 ("index", &index)==B_OK) {
		item = cast_as (fListView->ItemAt (index), FileListItem) ;
		items->AddItem ((RefRetriever*)item, index) ;
		fChanged = true ;
	}
} ;

void TargetSet:: ItemMoved (BMessage *msg) {
	int32 from = -1, to = -1 ;
	BList *items = fTargets->SetLock (true, this) ;
	if (!items) return ;
	if (msg->FindInt32 ("from", &from)==B_OK && msg->FindInt32 ("to", &to)==B_OK) {
		RefRetriever *item = (RefRetriever*)(items->RemoveItem (from)) ;
		items->AddItem (item, to) ;
		fChanged = true ;
	}
} ;

void TargetSet:: ItemDropped (BMessage *msg) {
	status_t err = B_OK ;
	int32 i = 0, j = 0 ;
	BMessage dropped ;
	entry_ref ref ;
	
	err = msg->FindInt32 ("dropped_index", &i) ;
	if (err==B_OK) err = msg->FindMessage ("dropped_msg", &dropped) ;
	if (err==B_OK) {
		while (dropped.FindRef ("refs", j++, &ref)==B_OK) {
			if (i>=0) {
				fListView->AddItem (new FileListItem (&ref), i++) ;
			} else {
				fListView->AddItem (new FileListItem (&ref)) ;
			}
		}
	}
} ;

void TargetSet:: RefsReceived (BMessage *msg) {
	int32 index = 0 ;
	entry_ref ref, old ;
	
	while (msg->FindRef ("refs", index++, &ref)==B_OK) {
		if (ref!=old) fListView->AddItem (new FileListItem (&ref)) ;
		old = ref ;
	}
} ;