//	WindowSet.cpp
//
//	Tetsuo Yamada


#include	"WindowSet.h"


#define		FB_HALF_WIDTH	3
#define		FB_HALF_HEIGHT	8
#define		FB_SIDE_LENGTH	5

//==========================================================
//	class DirectionControl
//==========================================================
DirectionControl:: DirectionControl (BRect frame, const char *name, 
									BMessage *message, uint32 mode) 
			: ClickControl (frame, name, "Direction", 24, 24, message, 
									mode, B_WILL_DRAW|B_NAVIGABLE) {
	// do nothing
} ;

DirectionControl:: ~DirectionControl () {
	// do nothing
} ;

int32 DirectionControl:: ValueForPoint (BPoint p, BRect imageRect) const {
	BPoint c (imageRect.left + imageRect.Width()/2, imageRect.top + imageRect.Height()/2) ;
	p -= c ;
	if (p.x - p.y>=0) { // up or right
		if (p.x + p.y>=0) return FB_LEFT_TO_RIGHT ;
		return FB_BOTTOM_TO_TOP ;
	} else { // left or down
		if (p.x + p.y>=0) return FB_TOP_TO_BOTTOM ;
		return FB_RIGHT_TO_LEFT ;
	}
} ;

int32 DirectionControl:: ValueForKey (const char *bytes, int32 num) const {
	switch (bytes[0]) {
		case B_LEFT_ARROW : 
			return FB_RIGHT_TO_LEFT ;
		case B_UP_ARROW : 
			return FB_BOTTOM_TO_TOP ;
		case B_RIGHT_ARROW : 
			return FB_LEFT_TO_RIGHT ;
		case B_DOWN_ARROW : 
			return FB_TOP_TO_BOTTOM ;
		default : 
			return (Value() + 5)%4 ;
	}
} ;

void DirectionControl:: DrawImage (BRect imageRect) {
	rgb_color low = LowColor () ;
	rgb_color high = HighColor () ;
	BPoint head, side1, side2 ;
	BPoint c (imageRect.left + imageRect.Width()/2, imageRect.top + imageRect.Height()/2) ;
	BRect rect (c, c) ;
	
	// draw bounds and background
	stroke_3d_rect (this, imageRect, low, false) ;
	imageRect.InsetBy (1, 1) ;
	SetHighColor (tint_color (ui_color2 (P_STANDARD_GRAY), B_DARKEN_2_TINT)) ;
	stroke_3d_rect (this, imageRect, HighColor(), false) ;
	imageRect.InsetBy (1, 1) ;
	SetHighColor (255, 255, 255) ;
	FillRect (imageRect) ;
	
	// draw bounds to click
	SetHighColor (ui_color2 (P_STANDARD_GRAY)) ;
	StrokeLine (imageRect.LeftTop(), imageRect.RightBottom()) ;
	StrokeLine (imageRect.LeftBottom(), imageRect.RightTop()) ;
	
	// draw arrow
	SetHighColor (ui_color2 (B_KEYBOARD_NAVIGATION_COLOR)) ;
	switch (Value()) {
		case FB_LEFT_TO_RIGHT : 
			rect.InsetBy (-FB_HALF_HEIGHT, -FB_HALF_WIDTH) ;
			head.Set (rect.right, c.y) ;
			side1.Set (head.x - 2*FB_HALF_WIDTH, c.y - 2*FB_HALF_WIDTH) ;
			side2.Set (head.x - 2*FB_HALF_WIDTH, c.y + 2*FB_HALF_WIDTH) ;
			rect.right = side1.x ;
			break ;
		case FB_RIGHT_TO_LEFT : 
			rect.InsetBy (-FB_HALF_HEIGHT, -FB_HALF_WIDTH) ;
			head.Set (rect.left, c.y) ;
			side1.Set (head.x + 2*FB_HALF_WIDTH, c.y - 2*FB_HALF_WIDTH) ;
			side2.Set (head.x + 2*FB_HALF_WIDTH, c.y + 2*FB_HALF_WIDTH) ;
			rect.left = side1.x ;
			break ;
		case FB_BOTTOM_TO_TOP : 
			rect.InsetBy (-FB_HALF_WIDTH, -FB_HALF_HEIGHT) ;
			head.Set (c.x, rect.top) ;
			side1.Set (c.x - 2*FB_HALF_WIDTH, rect.top + 2*FB_HALF_WIDTH) ;
			side2.Set (c.x + 2*FB_HALF_WIDTH, rect.top + 2*FB_HALF_WIDTH) ;
			rect.top = side1.y ;
			break ;
		case FB_TOP_TO_BOTTOM : 
			rect.InsetBy (-FB_HALF_WIDTH, -FB_HALF_HEIGHT) ;
			head.Set (c.x, rect.bottom) ;
			side1.Set (c.x - 2*FB_HALF_WIDTH, rect.bottom - 2*FB_HALF_WIDTH) ;
			side2.Set (c.x + 2*FB_HALF_WIDTH, rect.bottom - 2*FB_HALF_WIDTH) ;
			rect.bottom = side1.y ;
			break ;
	}
	FillRect (rect) ;
	FillTriangle (head, side2, side1) ;
	
	// finishing
	SetHighColor (high) ;
	SetLowColor (low) ;
} ;

//==========================================================
//	class LocationControl
//==========================================================
LocationControl:: LocationControl (BRect frame, const char *name, 
									BMessage *message, uint32 mode) 
			: ClickControl (frame, name, "Location", 24, 24, message, 
									mode, B_WILL_DRAW|B_NAVIGABLE) {
	// do nothing
} ;

LocationControl:: ~LocationControl () {
	// do nothing
} ;

int32 LocationControl:: ValueForPoint (BPoint p, BRect imageRect) const {
	BPoint c (imageRect.left + imageRect.Width()/2, imageRect.top + imageRect.Height()/2) ;
	if (p.y<c.y) {
		if (p.x<c.x) return FB_LEFT_TOP ;
		else return FB_RIGHT_TOP ;
	} else {
		if (p.x>=c.x) return FB_RIGHT_BOTTOM ;
		else return FB_LEFT_BOTTOM ;
	}
} ;

int32 LocationControl:: ValueForKey (const char *bytes, int32 numBytes) const {
	int32 value = Value() ;
	switch (bytes[0]) {
		case B_LEFT_ARROW : 
			return (value + 3)%4 ;
		default : 
			return (value + 1)%4 ;
	}
} ;

void LocationControl:: DrawImage (BRect imageRect) {
	rgb_color low = LowColor () ;
	rgb_color high = HighColor () ;
	BPoint head, side1, side2 ;
	BPoint c (imageRect.left + imageRect.Width()/2, imageRect.top + imageRect.Height()/2) ;
	
	// draw bounds and background
	stroke_3d_rect (this, imageRect, low, false) ;
	imageRect.InsetBy (1, 1) ;
	SetHighColor (tint_color (ui_color2 (P_STANDARD_GRAY), B_DARKEN_2_TINT)) ;
	stroke_3d_rect (this, imageRect, HighColor(), false) ;
	imageRect.InsetBy (1, 1) ;
	SetHighColor (255, 255, 255) ;
	FillRect (imageRect) ;
	
	// draw bounds to click
	SetHighColor (ui_color2 (P_STANDARD_GRAY)) ;
	StrokeLine (BPoint (imageRect.left, c.y), BPoint (imageRect.right, c.y)) ;
	StrokeLine (BPoint (c.x, imageRect.top), BPoint (c.x, imageRect.bottom)) ;
	
	// draw pointer
	SetHighColor (ui_color2 (B_KEYBOARD_NAVIGATION_COLOR)) ;
	switch (Value()) {
		case FB_LEFT_TOP : 
			head = imageRect.LeftTop () ;
			side1.Set (head.x, head.y + FB_SIDE_LENGTH) ;
			side2.Set (head.x + FB_SIDE_LENGTH, head.y) ;
			break ;
		case FB_RIGHT_TOP : 
			head = imageRect.RightTop () ;
			side1.Set (head.x, head.y + FB_SIDE_LENGTH) ;
			side2.Set (head.x - FB_SIDE_LENGTH, head.y) ;
			break ;
		case FB_RIGHT_BOTTOM : 
			head = imageRect.RightBottom () ;
			side1.Set (head.x, head.y - FB_SIDE_LENGTH) ;
			side2.Set (head.x - FB_SIDE_LENGTH, head.y) ;
			break ;
		case FB_LEFT_BOTTOM : 
			head = imageRect.LeftBottom () ;
			side1.Set (head.x, head.y - FB_SIDE_LENGTH) ;
			side2.Set (head.x + FB_SIDE_LENGTH, head.y) ;
			break ;
	}
	FillTriangle (head, side1, side2) ;
	
	// finishing
	SetHighColor (high) ;
	SetLowColor (low) ;
} ;

//==========================================================
//	class WindowSet
//==========================================================
WindowSet::WindowSet (settings* crt) 
		: BView (BRect (0, 0, 300, 300), "window_set", B_FOLLOW_ALL, B_WILL_DRAW) {
	fCrt = crt ;
	
	SetFont (be_plain_font) ;
	SetViewColor (235, 235, 235) ;
	
	BRect rect = Bounds() ;
	const float cline =  be_plain_font->StringWidth ("Operation to Launch") + 20 ;
	BMessage *msg = NULL ;
	float width, height ;
	
	// location
	rect.Set (cline - 32, SMALL_SPACE, cline + 20, BIG_SPACE + 32) ;
	msg = new BMessage (FB_WINDOW_MSG) ;
	msg->AddInt32 ("id", 0) ;
	LocationControl *locCtrl = new LocationControl (rect, "location", msg, 
			B_FOLLOW_LEFT|B_FOLLOW_TOP) ;
	AddChild (locCtrl) ;
	locCtrl->GetPreferredSize (&width, &height) ;
	locCtrl->ResizeTo (width, height) ;
	
	// direction
	rect.OffsetBy (0, height + SMALL_SPACE) ;
	msg = new BMessage (FB_WINDOW_MSG) ;
	msg->AddInt32 ("id", 1) ;
	DirectionControl *dirCtrl = new DirectionControl (rect, "direction", msg, 
			B_FOLLOW_LEFT|B_FOLLOW_TOP) ;
	AddChild (dirCtrl) ;
	dirCtrl->GetPreferredSize (&width, &height) ;
	dirCtrl->ResizeTo (width, height) ;
	
	// forInvoke
	rect.OffsetBy (0, height + SMALL_SPACE) ;
	rect.bottom = rect.top + be_plain_font->Size() + 10 ;
	rect.left = cline - StringWidth ("Operation to Launch:") - 15 ;
	rect.right = rect.left + 300 ;
	BMenuField *forInvoke = new BMenuField (rect, "forInvoke", "Operation to Launch:", 
			CreateMouseMenu (), B_FOLLOW_LEFT|B_FOLLOW_TOP) ;
	AddChild (forInvoke) ;
	forInvoke->SetAlignment (B_ALIGN_RIGHT) ;
	forInvoke->SetDivider (cline - rect.left - 3) ;
	forInvoke->GetPreferredSize (&width, &height) ;
	forInvoke->ResizeTo (width, height) ;
	
	// forExpand
	rect.OffsetBy (0, height + SMALL_SPACE) ;
	rect.left = cline - StringWidth ("Operation to Expand:") - 15 ;
	rect.right = rect.left + 300 ;
	BMenuField *forExpand = new BMenuField (rect, "forExpand", "Operation to Expand:", 
			CreateMouseMenu (), B_FOLLOW_LEFT|B_FOLLOW_TOP) ;
	AddChild (forExpand) ;
	forExpand->SetAlignment (B_ALIGN_RIGHT) ;
	forExpand->SetDivider (cline - rect.left - 3) ;
	forExpand->GetPreferredSize (&width, &height) ;
	forExpand->ResizeTo (width, height) ;
	
	// minimize
	rect.OffsetBy (0, height + SMALL_SPACE) ;
	rect.left = cline - 22 ;
	msg = new BMessage (FB_WINDOW_MSG) ;
	msg->AddInt32 ("id", 2) ;
	BCheckBox* minimize = new BCheckBox (rect, "minimize", 
			"Minimize When Not Active", msg) ;
	AddChild (minimize) ;
	minimize->GetPreferredSize (&width, &height) ;
	minimize->ResizeTo (width, height) ;
	
	// autofocus
	rect.OffsetBy (0, height + SMALL_SPACE) ;
	msg = new BMessage (FB_WINDOW_MSG) ;
	msg->AddInt32 ("id", 3) ;
	BCheckBox* autofocus = new BCheckBox (rect, "autofocus", 
			"Activate Automatically On Mouse Over", msg) ;
	AddChild (autofocus) ;
	autofocus->GetPreferredSize (&width, &height) ;
	autofocus->ResizeTo (width, height) ;
	
	// drawLabel
	rect.OffsetBy (0, height + SMALL_SPACE) ;
	msg = new BMessage (FB_WINDOW_MSG) ;
	msg->AddInt32 ("id", 4) ;
	BCheckBox* drawLabel = new BCheckBox (rect, "drawLabel", "Draw Label", msg) ;
	AddChild (drawLabel) ;
	drawLabel->GetPreferredSize (&width, &height) ;
	drawLabel->ResizeTo (width, height) ;
	
	// dropExtention
	rect.OffsetBy (0, height + SMALL_SPACE) ;
	msg = new BMessage (FB_WINDOW_MSG) ;
	msg->AddInt32 ("id", 5) ;
	BCheckBox* drawBorder = new BCheckBox (rect, "drawBorder", 
			"Draw Border Around Each Layer Item", msg) ;
	AddChild (drawBorder) ;
	drawBorder->GetPreferredSize (&width, &height) ;
	drawBorder->ResizeTo (width, height) ;
	
	// which
	rect.OffsetBy (0, height + SMALL_SPACE) ;
	msg = new BMessage (FB_WINDOW_MSG) ;
	msg->AddInt32 ("id", 6) ;
	BCheckBox *which = new BCheckBox (rect, "which", "Use Large Icon", msg) ;
	AddChild (which) ;
	which->GetPreferredSize (&width, &height) ;
	which->ResizeTo (width, height) ;
	
	fCtrls[0] = locCtrl ;
	fCtrls[1] = dirCtrl ;
	fCtrls[2] = minimize ;
	fCtrls[3] = autofocus ;
	fCtrls[4] = drawLabel ;
	fCtrls[5] = drawBorder ;
	fCtrls[6] = which ;
	fFields[0] = forInvoke ;
	fFields[1] = forExpand ;
} ;

WindowSet:: ~WindowSet () {
	// do nothing
} ;

BMenu* WindowSet:: CreateMouseMenu () const {
	BMenu* menu = new BMenu ("a menu") ;
	BMenuItem *item = NULL ;
	BMessage message (FB_WINDOW_MSG) ;
	message.AddInt32 ("id", 100) ;
	
	// FB_NO_USE
	item = new BMenuItem ("No Use", new BMessage (message)) ;
	menu->AddItem (item) ;
	
	// FB_PRIMARY_DOWN
	item = new BMenuItem ("Primary Mouse Down", new BMessage (message)) ;
	menu->AddItem (item) ;
	
	// FB_SECONDARY_DOWN
	item = new BMenuItem ("Secondary Mouse Down", new BMessage (message)) ;
	menu->AddItem (item) ;
	
	// FB_DOUBLE_CLICK
	item = new BMenuItem ("Double Click", new BMessage (message)) ;
	menu->AddItem (item) ;
	
	// customize menu
	menu->SetLabelFromMarked (true) ;
	menu->SetRadioMode (true) ;
	menu->SetTriggersEnabled (true) ;
	
	return menu ;
} ;

void WindowSet:: AttachedToWindow () {
	SettingsToViews () ;
	
	// set message target
	for (int32 i=0 ; i<7 ; i++) {
		fCtrls[i]->SetTarget (this) ;
	}
	for (int32 i=0 ; i<2 ; i++) {
		fFields[i]->Menu()->SetTargetForItems (this) ;
	}
	
	BView::AttachedToWindow () ;
} ;

void WindowSet:: SettingsToViews () {
	fCtrls[0]->SetValue (fCrt->location) ;
	fCtrls[1]->SetValue (fCrt->direction) ;
	fCtrls[2]->SetValue (fCrt->flags & FB_MINIMIZE) ;
	fCtrls[3]->SetValue (fCrt->flags & FB_AUTOFOCUS) ;
	fCtrls[4]->SetValue (fCrt->flags & FB_DRAW_LABEL) ;
	fCtrls[5]->SetValue (fCrt->flags & FB_DRAW_BORDER) ;
	fCtrls[6]->SetValue (fCrt->which==B_LARGE_ICON) ;
	
	fFields[0]->Menu()->ItemAt (fCrt->forInvoke)->SetMarked (true) ;
	fFields[1]->Menu()->ItemAt (fCrt->forExpand)->SetMarked (true) ;
} ;

void WindowSet:: MessageReceived (BMessage *msg) {
	int32 id = -1 ;
	int32 value = -1 ;
	BMenuItem *item = NULL ;
	switch (msg->what) {
		case FB_WINDOW_MSG : 
			msg->FindInt32 ("id", &id) ;
			if (id<7 && id>=0) value = fCtrls[id]->Value() ;
			switch (id) {
				case 0 : 
					fCrt->location = (bar_location)value ;
					break ;
				case 1 : 
					fCrt->direction = (bar_direction)value ;
					break ;
				case 2 : 
					fCrt->set_flags (FB_MINIMIZE, value) ;
					break ;
				case 3 : 
					fCrt->set_flags (FB_AUTOFOCUS, value) ;
					break ;
				case 4 : 
					fCrt->set_flags (FB_DRAW_LABEL, value) ;
					break ;
				case 5 : 
					fCrt->set_flags (FB_DRAW_BORDER, value) ;
					break ;
				case 6 : 
					fCrt->which = (value) ? B_LARGE_ICON : B_MINI_ICON ;
					break ;
				case 100 : 
					item = fFields[0]->Menu()->FindMarked () ;
					if (item) fCrt->forInvoke = 
							(mouse_operation)fFields[0]->Menu()->IndexOf (item) ;
					item = fFields[1]->Menu()->FindMarked () ;
					if (item) fCrt->forExpand = 
							(mouse_operation)fFields[1]->Menu()->IndexOf (item) ;
					break ;
			}
			break ;
		default : 
			BView::MessageReceived (msg) ;
	}
} ;