//	WorkspaceSet.cpp
//
//	Tetsuo Yamada

#include	"WorkspaceSet.h"


//==========================================================
//	class FBMenuField
//==========================================================
FBMenuField:: FBMenuField (BRect frame, const char *name, 
				const char *label, BMenu *menu, uint32 mode, uint32 flags) 
			: BMenuField (frame, name, label, menu, mode, flags) {
	//	do nothing
} ;

FBMenuField:: ~FBMenuField () {
	// do nothing
} ;

void FBMenuField:: MakeFocus (bool focused) {
	if (focused) {
		BView *parent = Parent() ;
		BRect rect = parent->Bounds() ;
		BRect frame = Frame() ;
		frame.OffsetBy (0, rect.top) ;
		float half = rect.Height() ;
		
		if (rect.bottom < frame.bottom) {
			parent->ScrollBy (0, max_c (frame.bottom - rect.bottom, half)) ;
		} else if (rect.top > frame.top) {
			parent->ScrollBy (0, -1*(max_c (rect.top - frame.top, half))) ;
		}
	}
	BMenuField::MakeFocus (focused) ;
} ;

//==========================================================
//	class WorkspaceSet
//==========================================================
WorkspaceSet ::WorkspaceSet (settings *crt) 
			: BView (BRect (0, 0, 300, 200), "workspace_set", 
						B_FOLLOW_ALL, B_WILL_DRAW|B_FRAME_EVENTS) {
	fCrt = crt ;
	fCount = count_workspaces() ;
	
	SetViewColor (235, 235, 235) ;
	SetFont (be_plain_font) ;
	
	char label [64] ;
	BRect rect ;
	
	// bottom BBoxes
	rect = Bounds() ;
	rect.InsetBy (SMALL_SPACE, SMALL_SPACE) ;
	rect.bottom = rect.top + 44 ;
	BBox *aBox = new BBox (rect, NULL, B_FOLLOW_LEFT_RIGHT|B_FOLLOW_TOP) ;
	aBox->SetLabel ("Select Layer for All Workspaces") ;
	AddChild (aBox) ;
	
	rect = Bounds() ;
	rect.InsetBy (SMALL_SPACE, SMALL_SPACE) ;
	rect.top = aBox->Frame().bottom + BIG_SPACE ;
	rect.bottom -= 20 ;
	BBox *eBox = new BBox (rect, NULL, B_FOLLOW_ALL) ;
	eBox->SetLabel ("Select Layer for Each Workspace") ;
	AddChild (eBox) ;
	
	// note that...
	rect.top = rect.bottom + SMALL_SPACE ;
	rect.bottom += 20 ;
	BStringView *note = new BStringView (rect, "note", 
			"Note that you cannot disable FakeBar in all Workspaces"B_UTF8_ELLIPSIS, 
			B_FOLLOW_LEFT_RIGHT|B_FOLLOW_BOTTOM) ;
	AddChild (note) ;
	note->SetAlignment (B_ALIGN_CENTER) ;
	
	// each Box's scroll view and it's child...
	rect = eBox->Bounds() ;
	rect.InsetBy (BIG_SPACE + 2, BIG_SPACE + 2) ;
	rect.top += BIG_SPACE ;
	rect.right -= B_V_SCROLL_BAR_WIDTH ;
	BView *view = new BView (rect, "each_bottom", B_FOLLOW_ALL, B_WILL_DRAW) ;
	view->SetViewColor (255, 255, 255) ;
	
	BScrollView *scr = new BScrollView ("scr", view, B_FOLLOW_ALL, 0, 
			false, true) ;
	eBox->AddChild (scr) ;
	
	// all MenuField
	rect = aBox->Bounds() ;
	rect.InsetBy (BIG_SPACE, BIG_SPACE) ;
	rect.top += SMALL_SPACE ;
	sprintf (label, "Workspace 1 - %d:", fCount) ;
	fAll = new BMenuField (rect, "all", label, new BMenu ("custom"), 
			B_FOLLOW_H_CENTER|B_FOLLOW_V_CENTER) ;
	fAll->SetAlignment (B_ALIGN_RIGHT) ;
	aBox->AddChild (fAll) ;
	
	// each MenuFields
	fFields = new BMenuField*[fCount] ;
	float divider = StringWidth ("Workspace 100:") ;
	rect.Set (0, 0, view->Bounds().Width(), 20) ;
	for (int32 i=0 ; i<fCount ; i++) {
		sprintf (label, "Workspace %d:", i + 1) ;
		fFields[i] = new FBMenuField (rect, "each", label, new BMenu ("menu"), 
				B_FOLLOW_H_CENTER|B_FOLLOW_TOP) ;
		fFields[i]->SetAlignment (B_ALIGN_RIGHT) ;
		view->AddChild (fFields[i]) ;
		rect.OffsetBy (0, 24) ;
	}
} ;

WorkspaceSet:: ~WorkspaceSet () {
	delete[] fFields ;
} ;

void WorkspaceSet:: AttachedToWindow () {
	BMessage container ;
	LayerManager *manager = cast_as (Window(), LayerManager) ;
	
	manager->GetLayerList (&container) ;
	for (int32 i=0 ; i<fCount ; i++) {
		BuildMenu (fFields[i], i, &container) ;
		fFields[i]->Menu()->SetTargetForItems (this) ;
	}
	BuildMenu (fAll, -1, &container) ;
	fAll->Menu()->SetTargetForItems (this) ;
	
	SettingsToViews () ;
	AdjustScrollBar () ;
	BView::AttachedToWindow () ;
} ;

void WorkspaceSet:: SettingsToViews () {
	BMenuItem *item = NULL ;
	
	fAll->MenuBar()->ItemAt (0)->SetLabel ("custom") ;
	item = fAll->Menu()->FindMarked() ;
	if (item) item->SetMarked (false) ;
	
	for (int32 i=0 ; i<fCount ; i++) {
		if (!(fCrt->is_enabled_in (i))) {
			fFields[i]->Menu()->FindItem ("Disable FakeBar")->SetMarked (true) ;
			continue ;
		}
		
		item = fFields[i]->Menu()->FindItem (fCrt->layers[i]) ;
		if (item && !is_kind_of (item, BSeparatorItem)) {
			item->SetMarked (true) ;
		} else {
			fFields[i]->Menu()->FindItem ("No Layer")->SetMarked (true) ;
		}
	}
} ;

void WorkspaceSet:: BuildMenu (BMenuField* target, int32 index, 
											BMessage *container) const {
	BMessage message (FB_WORKSPACE_MSG) ;
	int32 j = 0 ;
	BMenu *menu = target->Menu() ;
	const char *name = NULL ;
	int32 count = 0 ;
	
	message.AddInt32 ("index", index) ;
	
	menu->SetLabelFromMarked (true) ;
	menu->SetRadioMode (true) ;
	
	count = menu->CountItems () ;
	for (int32 i=count - 1 ; i>=0 ; i--) delete (BMenuItem*)(menu->RemoveItem (i)) ;
	
	menu->AddItem (new BMenuItem ("Disable FakeBar", new BMessage (message))) ;
	menu->AddSeparatorItem () ;
	menu->AddItem (new BMenuItem ("No Layer", new BMessage (message))) ;
	menu->AddSeparatorItem () ;
	
	while (container->FindString ("name", j++, &name)==B_OK) {
		if (strcmp (name, B_EMPTY_STRING)!=0)
			menu->AddItem (new BMenuItem (name, new BMessage (message))) ;
	}
} ;

void WorkspaceSet:: MessageReceived (BMessage *msg) {
	int32 index = 0 ;
	const char *label = NULL ;
	switch (msg->what) {
		case FB_WORKSPACE_MSG : 
			msg->FindInt32 ("index", &index) ;
			if (index==-1) label = fAll->Menu()->FindMarked()->Label() ;
			else label = fFields[index]->Menu()->FindMarked()->Label() ;
			if (strcmp (label, "Disable FakeBar")==0) {
				if (index==-1) {
					fCrt->workspaces = 0x0 ;
					SettingsToViews () ;
				} else {
					fCrt->workspaces = (fCrt->workspaces & ~(0x1 << index)) ;
				}
			} else if (strcmp (label, "No Layer")==0) {
				if (index==-1) {
					fCrt->workspaces = B_ALL_WORKSPACES ;
					for (int32 i=0 ; i<32 ; i++) fCrt->set_layer (B_EMPTY_STRING, i) ;
					SettingsToViews () ;
				} else {
					fCrt->workspaces = (fCrt->workspaces | (0x1 << index)) ;
					fCrt->set_layer (B_EMPTY_STRING, index) ;
				}
			} else {
				if (index==-1) {
					fCrt->workspaces = B_ALL_WORKSPACES ;
					for (int32 i=0 ; i<32 ; i++) fCrt->set_layer (label, i) ;
					SettingsToViews () ;
				} else {
					fCrt->workspaces = (fCrt->workspaces | (0x1 << index)) ;
					fCrt->set_layer (label, index) ;
				}
			}
			break ;
		default : 
			BView::MessageReceived (msg) ;
			break ;
	}
} ;

void WorkspaceSet:: FrameResized (float width, float height) {
	BView::FrameResized (width, height) ;
	AdjustScrollBar () ;
} ;

void WorkspaceSet:: AdjustScrollBar () {
	BView *bottom = FindView ("each_bottom") ;
	BScrollBar *bar = bottom->ScrollBar (B_VERTICAL) ;
	BRect bounds = bottom->Bounds() ;
	BRect page = fFields[fCount - 1]->Frame() ;
	page.top = 0 ;
	
	bar->SetSteps (10, bounds.Height()) ;
	bar->SetValue (0) ;
	if (page.Height() > bounds.Height()) {
		bar->SetProportion (bounds.Height()/page.Height()) ;
		bar->SetRange (0, page.Height() - bounds.Height()) ;
	} else {
		bar->SetProportion (1) ;
		bar->SetRange (0, 0) ;
	}
} ;