//	FSBase.cpp
//
//	Tetsuo Yamada

#include	"FSBase.h"
#include	<storage/AppFileInfo.h>
#include	<storage/Path.h>


//==================================================================
//	Globals
//==================================================================

status_t get_type (const entry_ref *ref, char *type, mode_t *mode, bool traverse) {
	status_t err = B_OK ;
	BEntry entry ;
	BNode node ;
	BNodeInfo info ;
	struct stat st ;
	bool typed = false ;
	
	err = entry.SetTo (ref, traverse) ;
	if (err!=B_OK) return err ;
	
	err = entry.InitCheck () ;
	if (err!=B_OK) return err ;
	
	if (!entry.Exists()) {
		err = B_ENTRY_NOT_FOUND ;
		return err ;
	}
	
	err = node.SetTo (&entry) ;
	if (err!=B_OK) return err ;
	
	err = node.InitCheck () ;
	if (err!=B_OK) return err ;
	
	// first attack
	err = info.SetTo (&node) ;
	if (err!=B_OK) return err ;
	
	err = info.InitCheck () ;
	if (err!=B_OK) return err ;
	
	err = info.GetType (type) ;
	if (err==B_OK) typed = true ;
	
	if (!mode) return err ;
	
	// second attack
	err = node.GetStat (&st) ;
	if (err!=B_OK) return ((typed) ? B_OK : err) ;
	
	if (S_ISDIR (st.st_mode)) {
		// it's a directory
		if (!typed) strcpy (type, "application/x-vnd.Be-directory") ;
		typed = true ;
	} else if (S_ISLNK (st.st_mode)) {
		// it's a symlink
		if (!typed) strcpy (type, "application/x-vnd.Be-symlink") ;
		typed = true ;
	} else if (S_ISREG (st.st_mode)) {
		// it's a "regular" file???
		if (!typed) strcpy (type, "application/octet-stream") ;
		typed = true ;
	}
	
	if (mode) *mode = st.st_mode ;
	
	return (typed) ? B_OK : B_ERROR ;
} ;

status_t get_type (const char* path, char *type, mode_t *mode, bool traverse) {
	status_t err = B_OK ;
	BEntry entry ;
	BNode node ;
	BNodeInfo info ;
	struct stat st ;
	bool typed = false ;
	
	err = entry.SetTo (path, traverse) ;
	if (err!=B_OK) return err ;
	
	err = entry.InitCheck () ;
	if (err!=B_OK) return err ;
	
	if (!entry.Exists()) {
		err = B_ENTRY_NOT_FOUND ;
		return err ;
	}
	
	err = node.SetTo (&entry) ;
	if (err!=B_OK) return err ;
	
	err = node.InitCheck () ;
	if (err!=B_OK) return err ;
	
	// first attack
	err = info.SetTo (&node) ;
	if (err!=B_OK) return err ;
	
	err = info.InitCheck () ;
	if (err!=B_OK) return err ;
	
	err = info.GetType (type) ;
	if (err==B_OK) typed = true ;
	
	if (!mode) return err ;
	
	// second attack
	err = node.GetStat (&st) ;
	if (err!=B_OK) return (typed) ? B_OK : err ;
	
	if (S_ISDIR (st.st_mode)) {
		// it's a directory
		if (!typed) strcpy (type, "application/x-vnd.Be-directory") ;
		typed = true ;
	} else if (S_ISLNK (st.st_mode)) {
		// it's a symlink
		if (!typed) strcpy (type, "application/x-vnd.Be-symlink") ;
		typed = true ;
	} else if (S_ISREG (st.st_mode)) {
		// it's a "regular" file???
		if (!typed) strcpy (type, "application/octet-stream") ;
		typed = true ;
	}
	
	if (mode) *mode = st.st_mode ;
	
	return (typed) ? B_OK : B_ERROR ;
} ;

status_t get_signature (const entry_ref *ref, char *signature, bool *refIsExe) {
	status_t err = B_OK ;
	BFile file ;
	BAppFileInfo info ;
	entry_ref copy, app ;
	BEntry entry ;
	
	err = entry.SetTo (ref, true) ;
	if (err!=B_OK) return err ;
	
	err = entry.InitCheck () ;
	if (err!=B_OK) return err ;
	
	if (!entry.Exists()) return B_ENTRY_NOT_FOUND ;
	
	if (entry.IsDirectory()) {
		strcpy (signature, "application/x-vnd.Be-TRAK") ;
		if (refIsExe) *refIsExe = false ;
		return B_OK ;
	}
	
	copy = *ref ;
	
	err = be_roster->FindApp (&copy, &app) ;
	if (err!=B_OK) return err ;
	
	if (refIsExe) *refIsExe = (copy==app) ;
	
	err = file.SetTo (&app, B_READ_WRITE) ;
	if (err!=B_OK) return err ;
	
	err = file.InitCheck () ;
	if (err!=B_OK) return err ;
	
	err = info.SetTo (&file) ;
	if (err!=B_OK) return err ;
	
	err = info.InitCheck () ;
	if (err!=B_OK) return err ;
	
	err = info.GetSignature (signature) ;
	if (err!=B_OK) return err ;
	
	return B_OK ;
} ;

status_t get_path (const entry_ref *ref, char *buf, bool traverse) {
	status_t err = B_OK ;
	BEntry entry ;
	BPath path ;
	
	err = entry.SetTo (ref, traverse) ;
	if (err!=B_OK) return err ;
	
	err = entry.InitCheck () ;
	if (err!=B_OK) return err ;
	
	err = entry.GetPath (&path) ;
	if (err!=B_OK) return err ;
	
	err = path.InitCheck () ;
	if (err!=B_OK) return err ;
	
	strcpy (buf, path.Path()) ;
	return B_OK ;
} ;

status_t get_relative_path (const entry_ref *src, const entry_ref *dst, char *buf) {
	status_t err = B_OK ;
	char srcPath [B_PATH_NAME_LENGTH] ;
	char dstPath [B_PATH_NAME_LENGTH] ;
	
	sprintf (buf, "") ;
	
	err = get_path (src, srcPath) ;
	if (err==B_OK) err = get_path (dst, dstPath) ;
	if (err!=B_OK) return err ;
	
	//	get first index of string which is different each other.
	//	these 2 lines can be more efficient...
	int32 index = 0 ;
	while (strncmp (srcPath, dstPath, index + 1)==0) index++ ;
	
	for (int32 i=index ; srcPath[i]!='\0' ; i++) 
		if (srcPath[i]=='/') sprintf (buf, "%s../", buf) ;
	
	strcat (buf, dstPath + index) ;
	return B_OK ;
} ;

const char* ref_name (const char *path) {
	for (int32 i=strlen (path) - 1 ; i>=0 ; i--) {
		if (path [i]=='/') return path + i + 1 ;
	}
	return path ;
} ;