//	FSBase.h
//
//	Tetsuo Yamada
//
//	Collection of global functions and constants for file system...

#ifndef		P_FS_BASE_h_
#define		P_FS_BASE_h_


#include	<app/Message.h>
#include	<app/Application.h>
#include	<app/Roster.h>
#include	<kernel/fs_info.h>
#include	<kernel/OS.h>
#include	<storage/Entry.h>
#include	<storage/Mime.h>
#include	<storage/Directory.h>
#include	<storage/FindDirectory.h>
#include	<storage/NodeInfo.h>
#include	<support/Errors.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

//==================================================================
//	Constant definitions
//==================================================================
#define		P_FS_ERRORS_BASE	B_ERRORS_END + 1
#define		P_FS_ERRORS_END		P_FS_ERRORS_START + 29

//	my error codes
enum {
	P_CANNOT_HANDLE = P_FS_ERRORS_BASE, 
	P_DELETE
} ;

//==================================================================
//	Globals
//==================================================================

/**	get file type for passed entry_ref and fills char* type.
	if mode_t *mode is specified, fills it with stat.st_mode */
status_t					get_type (const entry_ref *ref, 
									char *type, 
									mode_t *mode = NULL, 
									bool traverse = false) ;
status_t					get_type (const char *path, 
									char *type, 
									mode_t *mode = NULL, 
									bool traverse = false) ;
	
/**	get most preferrable application for the specified ref and fills signature with it. */
status_t					get_signature (const entry_ref *ref, 
									char *signature, 
									bool *refIsExe = NULL) ;

/**	fills buf with the absolute path for the ref */
status_t					get_path (const entry_ref *ref, 
									char *buf, 
									bool traverse = false) ;

/**	fills buf with the relative path from the src to the dst. */
status_t					get_relative_path (const entry_ref *src, 
									const entry_ref *dst, 
									char *buf) ;

/**	get the leaf name portion for the path. */
const char*					ref_name (const char *path) ;

#endif

