//	FindApps.cpp
//
//	Tetsuo Yamada

#include	"FindApps.h"


//===================================================================
//	find_supporting_apps
//===================================================================
status_t find_supporting_apps (const entry_ref *ref, BMessage *msg) {
	
	status_t err = B_OK ;
	char type [B_MIME_TYPE_LENGTH] ;
	char prfr [B_MIME_TYPE_LENGTH] ;	// signature of a preferred app
	char* sign = NULL ;
	BMimeType mime ;
	support_info info ;
	BMessage dummy ;	// used when getting supporting apps from BMimeType
	app_info ainfo ;
	
	// get preferred app
	ainfo.ref = *ref ;
	err = be_roster->FindApp (&(ainfo.ref), &(info.ref)) ;
	if (err==B_OK) {
		err = be_roster->GetAppInfo (&(info.ref), &ainfo) ;
		if (err==B_OK) {
			strcpy (prfr, ainfo.signature) ;
			strcpy (info.signature, prfr) ;
			info.state = P_PREFERRED_APP ;
			msg->AddData ("supporting_apps", B_OBJECT_TYPE, &info, 
					sizeof (support_info), true) ;
			// next line is important, because info.ref.name is not copied.
			info.ref.name = NULL ;
		}
	}
	
	// get supporting/super_supprting apps
	err = get_type (ref, type, NULL, true) ;
	if (err!=B_OK) goto err1 ;
	
	err = mime.SetTo (type) ;
	if (err!=B_OK) goto err1 ;
	
	err = mime.InitCheck () ;
	if (err!=B_OK) goto err1 ;
	
	err = mime.GetSupportingApps (&dummy) ;
	if (err==B_OK) {
		int32 sub = 0 ;
		int32 index = 0 ;
		dummy.FindInt32 ("be:sub", &sub) ;
		
		while (dummy.FindString ("applications", index++, &sign)==B_OK) {
			if (strcmp (sign, prfr)==0) continue ;
			err = be_roster->FindApp (sign, &(info.ref)) ;
			if (err==B_OK) {
				strcpy (info.signature, sign) ;
				info.state = (index<sub) ? P_SUPPORTING_APP : P_SUPER_SUPPORT_APP ;
				msg->AddData ("supporting_apps", B_OBJECT_TYPE, &info, 
						sizeof (support_info), false) ;
				info.ref.name = NULL ;
			}
		}
	}
	
	// get wildcard apps
	err = BMimeType::GetWildcardApps (&dummy) ;
	if (err==B_OK) {
		int32 index = 0 ;
		
		while (dummy.FindString ("applications", index++, &sign)==B_OK) {
			if (strcmp (sign, prfr)==0) continue ;
			err = be_roster->FindApp (sign, &(info.ref)) ;
			if (err==B_OK) {
				strcpy (info.signature, sign) ;
				info.state = P_WILDCARD_APP ;
				msg->AddData ("supporting_apps", B_OBJECT_TYPE, &info, 
						sizeof (support_info), false) ;
				info.ref.name = NULL ;
			}
		}
	}
	
	msg->RemoveName ("applications") ;
	return B_OK ;
	
err1 :
	return err ;

} ;