//	RefRetriever.cpp
//
//	Tetsuo Yamada

#include	"RefRetriever.h"



//================================================================
//	class RefRetriever
//================================================================
RefRetriever:: RefRetriever () {
	fPath = new char [strlen (B_EMPTY_STRING) + 1] ;
	strcpy (fPath, B_EMPTY_STRING) ;
	fNode = -1 ;
	fErr = B_NO_INIT ;
} ;

RefRetriever:: RefRetriever (const char *path) {
	status_t err = B_OK ;
	fNode = -1 ;
	fErr = B_NO_INIT ;
	fPath = new char [strlen (path) + 1] ;
	strcpy (fPath, path) ;
	
	SetParams () ;
} ;

RefRetriever:: RefRetriever (const entry_ref *ref) {
	status_t err = B_OK ;
	fNode = -1 ;
	fErr = B_NO_INIT ;
	fPath = NULL ;
	char path [B_PATH_NAME_LENGTH + 1] ;
	
	if (ref && get_path (ref, path, false)==B_OK) {
		fPath = new char [strlen (path) + 1] ;
		strcpy (fPath, path) ;
		SetParams () ;
	} else {
		fPath = new char [strlen (B_EMPTY_STRING) + 1] ;
		strcpy (fPath, B_EMPTY_STRING) ;
		SetParams () ;
	}
} ;

RefRetriever:: RefRetriever (const RefRetriever &src) {
	status_t err = B_OK ;
	fPath = new char [strlen (src.fPath) + 1] ;
	strcpy (fPath, src.fPath) ;
	fNode = src.fNode ;
	fErr = src.fErr ;
} ;

RefRetriever:: ~RefRetriever () {
	if (fPath) delete[] fPath ;
} ;

void RefRetriever:: SetParams () {
	BEntry entry ;
	node_ref nref ;
	
	fErr = entry.SetTo (fPath) ;
	if (fErr!=B_OK) return ;
	
	fErr = entry.InitCheck () ;
	if (fErr!=B_OK) return ;
	
	if (!entry.Exists()) {
		fErr = B_ENTRY_NOT_FOUND ;
		return ;
	}
	
	fErr = entry.GetNodeRef (&nref) ;
	if (fErr!=B_OK) return ;
	
	fNode = nref.node ;
} ;

status_t RefRetriever:: SetEntryRef (const entry_ref *ref) {
	status_t err = B_OK ;
	char path [B_PATH_NAME_LENGTH + 1] ;
	
	if (ref && (err = get_path (ref, path, false))==B_OK) {
		if (strcmp (path, fPath)!=0) {
			delete[] fPath ;
			fPath = new char [strlen (path) + 1] ;
			strcpy (fPath, path) ;
			
			SetParams () ;
			err = RefUpdated () ;
			if (fErr==B_OK) fErr = err ;
			return fErr ;
		} else {
			return B_OK ;
		}
	} else {
		delete[] fPath ;
		fPath = new char [strlen (B_EMPTY_STRING) + 1] ;
		strcpy (fPath, B_EMPTY_STRING) ;
		fErr = B_NO_INIT ;
		fNode = -1 ;
		return err ;
	}
} ;

status_t RefRetriever:: SetPath (const char *path) {
	// first check
	if (!path) return B_BAD_VALUE ;
	if (strcmp (path, fPath)==0) return B_OK ;
	
	status_t err = B_OK ;
	
	if (fPath) delete[] fPath ;
	fPath = new char [strlen (path) + 1] ;
	strcpy (fPath, path) ;
	
	SetParams () ;
	err = RefUpdated () ;
	if (fErr==B_OK) fErr = err ;
	return fErr ;
} ;

status_t RefRetriever:: GetEntryRef (entry_ref *ref) const {
	return get_ref_for_path (fPath, ref) ;
} ;

status_t RefRetriever:: GetNodeRef (node_ref *ref) const {
	dev_t dev = dev_for_path (fPath) ;
	if (dev<B_OK) return dev ;
	ref->device = dev ;
	ref->node = fNode ;
	return B_OK ;
} ;

const char* RefRetriever:: Path () const {
	return fPath ;
} ;

//	this method is same as global ref_name().
const char *RefRetriever:: RefName () const {
	for (int32 i=strlen (fPath) - 1 ; i>=0 ; i--) {
		if (fPath [i]=='/') return fPath + i + 1 ;
	}
	return fPath ;
} ;

status_t RefRetriever:: Status () const {
	return fErr ;
} ;

status_t RefRetriever:: GetIcon (BBitmap *icon, icon_size which) {
	if (fErr!=B_OK) return fErr ;
	status_t err = B_OK ;
	BEntry entry ;
	entry_ref ref ;
	BDirectory dir ;
	
	err = entry.SetTo (fPath, true) ;
	if (err==B_ENTRY_NOT_FOUND) err = entry.SetTo (fPath, false) ;
	if (err!=B_OK) return err ;
	
	if (entry.Exists()) {
		err = entry.GetRef (&ref) ;
		if (err!=B_OK) return err ;
	} else {
		err = get_ref_for_path (fPath, &ref) ;
		if (err!=B_OK) return err ;
		
		goto Finish ;
	}
	
	err = dir.SetTo (&entry) ;
	if (err!=B_OK) goto Finish ;
	
	if (dir.IsRootDirectory()) {
		// this object referres a volume. get volume icon.
		BVolume volume ;
		
		err = volume.SetTo (dev_for_path (fPath)) ;
		if (err!=B_OK) goto Finish ;
		
		err = volume.GetIcon (icon, which) ;
		if (err!=B_OK) goto Finish ;
		
		return B_OK ;
	}
	
Finish : 
	return BNodeInfo::GetTrackerIcon (&ref, icon, which) ;
} ;

status_t RefRetriever:: CheckNode (bool deep) {
	status_t err = B_OK ;
	BEntry entry ;
	
	err = entry.SetTo (fPath, false) ;
	if (err!=B_OK) goto End ;
	
	err = entry.InitCheck () ;
	if (err!=B_OK) goto End ;
	
	if (!entry.Exists()) {
		err = B_ENTRY_NOT_FOUND ;
		goto End ;
	}
	
End : 
	if (err!=fErr || deep) {
		fErr = err ;
		err = RefUpdated () ;
		if (fErr==B_OK) fErr = err ;
	}
	
	return fErr ;
} ;

status_t RefRetriever:: HandleNodeMonitor (int32 opcode, 
									const node_ref *nref, const BMessage *msg) {
	if (fErr!=B_OK) return P_CANNOT_HANDLE ;
	if (nref->device!=dev_for_path (fPath)) return P_CANNOT_HANDLE ;
	if (fNode!=nref->node) return P_CANNOT_HANDLE ;
	
	status_t err = B_OK ;
	entry_ref ref ;
	
	switch (opcode) {
		case B_ENTRY_REMOVED :
			RefExpired () ;
			return P_DELETE ;
			
		case B_ENTRY_MOVED :
			ref.device = nref->device ;
			err = msg->FindString ("name", &(ref.name)) ;
			if (err!=B_OK) {
				ref.name = NULL ;
				return err ;
			}
			
			err = msg->FindInt64 ("to directory", (int64*)&(ref.directory)) ;
			if (err!=B_OK) {
				ref.name = NULL ;
				return err ;
			}
			
			return SetEntryRef (&ref) ;
		
		case B_ATTR_CHANGED :
		case B_STAT_CHANGED :
			err = RefUpdated () ;
			if (fErr==B_OK) fErr = err ;
			return fErr ;
		
		default :
			return P_CANNOT_HANDLE ;
	}
} ;

status_t RefRetriever:: RefUpdated () {
	return B_OK ;
} ;

void RefRetriever:: RefExpired () {
	// do nothing
} ;

RefRetriever& RefRetriever:: operator=(const RefRetriever &src) {
	status_t err = B_OK ;
	if (fPath) delete[] fPath ;
	fPath = new char [strlen (src.fPath) + 1] ;
	strcpy (fPath, src.fPath) ;
	fNode = src.fNode ;
	fErr = src.fErr ;
	err = RefUpdated () ;
	if (fErr==B_OK) fErr = err ;
	return *this ;
} ;

bool RefRetriever:: operator==(const RefRetriever &alt) const {
	return (strcmp (fPath, alt.fPath)==0 && fNode==alt.fNode && fErr==alt.fErr) ;
} ;

bool RefRetriever:: operator!=(const RefRetriever &alt) const {
	return (strcmp (fPath, alt.fPath) || fNode!=alt.fNode || fErr!=alt.fErr) ;
} ;