//	RefRetriever.h
//
//	Tetsuo Yamada
//
//	Simple class representing an abs path to a node.
//	since it is a path string, illegal path is acceptable.

#ifndef		P_REF_RETRIEVER_h_
#define		P_REF_RETRIEVER_h_


#include	"FSBase.h"
#include	<interface/Bitmap.h>
#include	<storage/NodeMonitor.h>
#include	<storage/Volume.h>


//================================================================
//	class RefRetriever
//================================================================
class RefRetriever {
// fields
private :
	char*					fPath ;
	ino_t					fNode ;
	status_t				fErr ;
	
// methods
public :
							RefRetriever () ;
							RefRetriever (const char *path) ;
							RefRetriever (const entry_ref *ref) ;
							RefRetriever (const RefRetriever &src) ;
	virtual					~RefRetriever () ;
	
	virtual status_t		SetEntryRef (const entry_ref *ref) ;
	virtual status_t		SetPath (const char *path) ;
	
	status_t				GetEntryRef (entry_ref *ref) const ;
	status_t				GetNodeRef (node_ref *ref) const ;
	const char*				Path () const ;
	const char*				RefName () const ;
	
	/**	get icon for this node. volume is ok. */
	status_t				GetIcon (BBitmap *icon, icon_size size = B_LARGE_ICON) ;
	
	/** simply returns current status (status_t fErr). */
	status_t				Status () const ;
	
	/**	B_ENTRY_REMOVED, B_ENTRY_MOVED, B_ATTR_CHANGED, B_STAT_CHANGED opcodes are acceptable.
		All are valid only when applied to this object itself (not parent dir).
		return value: 
			B_OK -> successful
			P_DELETE -> you should NOT use this ref any more
			P_CANNOT_HANDLE -> cannot handle passed message
			And others... (defined in Be API) */
	status_t				HandleNodeMonitor (int32 opcode, 
									const node_ref *nref, 
									const BMessage *msg) ;
	
	/**	checks this item's legality.
		if deep is true, make sure to call RefUpdated() hook.
		otherwise, call RefUpdated() hook when this object's status is changed */
	status_t				CheckNode (bool deep = false) ;
	
protected :
	/**	RefRetriever's version of this method does nothing and returns B_OK.
		Derived class can reimplement this hook 
		to handle some changes on this object.
		This hook can be called from SetEntryRef(), SetPath(), operator=, 
		CheckNode() and HandleNodeMonitor() */
	virtual status_t		RefUpdated () ;
	
	/**	this method is called from HandleNodeMonitor().
		when this method is called, a file this object refers was expired. */
	virtual void			RefExpired () ;
	
private :
	/** set fNode and fErr from fPath */
	void					SetParams () ;
	
// operators
public :
	RefRetriever&			operator=(const RefRetriever &src) ;
	bool					operator==(const RefRetriever &alt) const ;
	bool					operator!=(const RefRetriever &alt) const ;
	
} ;

#endif

